@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2013, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -54,6 +54,8 @@ void vt_unimci_init()
 
 void vt_unimci_finalize()
 {
+  if( vt_unimci_is_initialized )
+    vt_unimci_is_initialized = 0;
 }
 
 void vt_unimci_check_msg(uint8_t record, uint64_t* time)
@@ -100,7 +102,7 @@ void vt_unimci_check_msg(uint8_t record, uint64_t* time)
       vt_cntl_msg( 1, "Application terminated due to "UNIMCI_CHECKER_NAME" "
                       "detected an error, see the trace for details" );
       vt_def_comment( VT_CURRENT_THREAD,
-                      VT_UNIFY_STRID_VT_COMMENT"WARNING: This trace is incomplete, "
+                      VT_UNIFY_STRID_VT_COMMENT"Warning: This trace is incomplete, "
                       "because "UNIMCI_CHECKER_NAME" detected an error." );
 
       /* this should invoke vt_close() to shutdown VampirTrace */
