@@ -3,6 +3,7 @@
  * Copyright (c) 2009-2012 Mellanox Technologies.  All rights reserved.
  * Copyright (c) 2012      Los Alamos National Security, LLC.
  *                         All rights reserved.
+ * Copyright (c) 2014 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -384,7 +385,7 @@ int base_bcol_basesmuma_setup_ctl_struct(
     input_file.file_name=cs->sm_ctl_structs->map_path;
     input_file.size=cs->sm_ctl_structs->map_size;
     input_file.size_ctl_structure=0;
-    input_file.data_seg_alignment=CACHE_LINE_SIZE;
+    input_file.data_seg_alignment=BASESMUMA_CACHE_LINE_SIZE;
     input_file.mpool_size=cs->sm_ctl_structs->map_size;
     ret=bcol_basesmuma_smcm_allgather_connection(
         sm_bcol_module,
@@ -394,6 +395,9 @@ int base_bcol_basesmuma_setup_ctl_struct(
         sm_bcol_module->super.sbgp_partner_module->group_comm,
         input_file, cs->clt_base_fname,
         false);
+    if (OMPI_SUCCESS != ret) {
+        goto exit_ERROR;
+    }
 
     /* fill in the pointer to other ranks scartch shared memory */
     sm_bcol_module->shared_memory_scratch_space=
