@@ -9,12 +9,13 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2013 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006-2009 Mellanox Technologies. All rights reserved.
- * Copyright (c) 2006-2013 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2006-2007 Voltaire All rights reserved.
  * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2013      NVIDIA Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -28,6 +29,7 @@
 
 #include "opal/util/bit_ops.h"
 #include "opal/mca/installdirs/installdirs.h"
+#include "opal/util/os_dirpath.h"
 #include "opal/util/output.h"
 #include "opal/util/show_help.h"
 #include "btl_openib.h"
@@ -250,6 +252,13 @@ int btl_openib_register_mca_params(void)
                   "(0 = do not warn; any other value = warn)",
                   true, &mca_btl_openib_component.warn_nonexistent_if));
 
+    /* If we print a warning about not having enough registered memory
+       available, do we want to abort? */
+    CHECK(reg_bool("abort_not_enough_reg_mem", NULL,
+                  "If there is not enough registered memory available on the system for Open MPI to function properly, Open MPI will issue a warning.  If this MCA parameter is set to true, then Open MPI will also abort all MPI jobs "
+                  "(0 = warn, but do not abort; any other value = warn and abort)",
+                  false, &mca_btl_openib_component.abort_not_enough_reg_mem));
+
     CHECK(reg_uint("poll_cq_batch", NULL,
                    "Retrieve up to poll_cq_batch completions from CQ",
                    MCA_BTL_OPENIB_CQ_POLL_BATCH_DEFAULT, &mca_btl_openib_component.cq_poll_batch,
@@ -491,6 +500,13 @@ int btl_openib_register_mca_params(void)
     mca_btl_openib_component.enable_srq_resize = false;
 #endif
 
+#if HAVE_DECL_IBV_LINK_LAYER_ETHERNET
+    CHECK(reg_bool("rroce_enable", NULL,
+                   "Enable/Disable routing between different subnets"
+                   "(0 = disable, nonzero = enable)", false,
+                   &mca_btl_openib_component.rroce_enable));
+#endif
+
     CHECK(reg_uint("buffer_alignment", NULL,
                    "Preferred communication buffer alignment, in bytes "
                    "(must be > 0 and power of two)",
@@ -515,6 +531,12 @@ int btl_openib_register_mca_params(void)
                    "Maximum size (in bytes) of a single fragment of a long message when using the RDMA protocols (must be > 0 and <= hw capabilities).",
                    0, &mca_btl_openib_component.max_hw_msg_size, 0));
 
+    /* Help debug memory registration issues */
+    CHECK(reg_int("memory_registration_verbose", NULL,
+                  "Output some verbose memory registration information "
+                  "(0 = no output, nonzero = output)", 0,
+		  &mca_btl_openib_component.memory_registration_verbose_level, 0));
+
     /* Info only */
     tmp = mca_base_component_var_register(&mca_btl_openib_component.super.btl_version,
                                           "have_fork_support",
@@ -542,33 +564,80 @@ int btl_openib_register_mca_params(void)
     /* Default to bandwidth auto-detection */
     mca_btl_openib_module.super.btl_bandwidth = 0;
     mca_btl_openib_module.super.btl_latency = 4;
+#if OPAL_CUDA_SUPPORT /* CUDA_ASYNC_RECV */
+    /* Default is enabling CUDA asynchronous send copies */
+    CHECK(reg_bool("cuda_async_send", NULL,
+                   "Enable or disable CUDA async send copies "
+                   "(true = async; false = sync)",
+                   true, &mca_btl_openib_component.cuda_async_send));
+
+    /* Default is enabling CUDA asynchronous receive copies */
+    CHECK(reg_bool("cuda_async_recv", NULL,
+                   "Enable or disable CUDA async recv copies "
+                   "(true = async; false = sync)",
+                   true, &mca_btl_openib_component.cuda_async_recv));
+    /* Also make the max send size larger for better GPU buffer performance */
+    mca_btl_openib_module.super.btl_max_send_size = 128 * 1024;
+    /* Turn of message coalescing - not sure if it works with GPU buffers */
+    mca_btl_openib_component.use_message_coalescing = 0;
+
+    /* Indicates if library was built with GPU Direct RDMA support.  Not changeable.  */
+    mca_btl_openib_component.cuda_have_gdr = OPAL_INT_TO_BOOL(OPAL_CUDA_GDR_SUPPORT);
+    (void) mca_base_component_var_register(&mca_btl_openib_component.super.btl_version, "have_cuda_gdr",
+                                           "Whether CUDA GPU Direct RDMA support is built into library or not",
+                                           MCA_BASE_VAR_TYPE_BOOL, NULL, 0,
+                                           MCA_BASE_VAR_FLAG_DEFAULT_ONLY,
+                                           OPAL_INFO_LVL_5,
+                                           MCA_BASE_VAR_SCOPE_CONSTANT,
+                                           &mca_btl_openib_component.cuda_have_gdr);
+
+    /* Indicates if driver has GPU Direct RDMA support.  Not changeable.  */
+    if (OPAL_SUCCESS == opal_os_dirpath_access("/sys/kernel/mm/memory_peers/nv_mem/version", S_IRUSR)) {
+        mca_btl_openib_component.driver_have_gdr = 1;
+    } else {
+        mca_btl_openib_component.driver_have_gdr = 0;
+    }
+    (void) mca_base_component_var_register(&mca_btl_openib_component.super.btl_version, "have_driver_gdr",
+                                           "Whether Infiniband driver has GPU Direct RDMA support",
+                                           MCA_BASE_VAR_TYPE_BOOL, NULL, 0,
+                                           MCA_BASE_VAR_FLAG_DEFAULT_ONLY,
+                                           OPAL_INFO_LVL_5,
+                                           MCA_BASE_VAR_SCOPE_CONSTANT,
+                                           &mca_btl_openib_component.driver_have_gdr);
+
+    /* Default for GPU Direct RDMA is off for now */
+    CHECK(reg_bool("want_cuda_gdr", NULL,
+                   "Enable or disable CUDA GPU Direct RDMA support "
+                   "(true = enabled; false = disabled)",
+                   false, &mca_btl_openib_component.cuda_want_gdr));
+
+    if (mca_btl_openib_component.cuda_want_gdr && !mca_btl_openib_component.cuda_have_gdr) {
+        opal_show_help("help-mpi-btl-openib.txt",
+                       "CUDA_no_gdr_support", true,
+                       ompi_process_info.nodename);
+        return OMPI_ERROR;
+    }
+    if (mca_btl_openib_component.cuda_want_gdr && !mca_btl_openib_component.driver_have_gdr) {
+        opal_show_help("help-mpi-btl-openib.txt",
+                       "driver_no_gdr_support", true,
+                       ompi_process_info.nodename);
+        return OMPI_ERROR;
+    }
+#if OPAL_CUDA_GDR_SUPPORT
+    if (mca_btl_openib_component.cuda_want_gdr) {
+        mca_btl_openib_module.super.btl_flags |= MCA_BTL_FLAGS_CUDA_GET;
+        mca_btl_openib_module.super.btl_cuda_eager_limit = SIZE_MAX; /* magic number - indicates set it to minimum */
+        mca_btl_openib_module.super.btl_cuda_rdma_limit = 30000;  /* default switchover is 30,000 to pipeline */
+    } else {
+        mca_btl_openib_module.super.btl_cuda_eager_limit = 0; /* Turns off any of the GPU Direct RDMA code */
+        mca_btl_openib_module.super.btl_cuda_rdma_limit = 0;  /* Unused */
+    }
+#endif /* OPAL_CUDA_GDR_SUPPORT */
+#endif /* OPAL_CUDA_SUPPORT */
     CHECK(mca_btl_base_param_register(
             &mca_btl_openib_component.super.btl_version,
             &mca_btl_openib_module.super));
 
-#if OMPI_CUDA_SUPPORT /* CUDA_ASYNC_RECV */
-    /* Default is enabling CUDA asynchronous send copies */
-    CHECK(reg_bool("cuda_async_send", NULL,
-                  "Enable or disable CUDA async send copies "
-                  "(true = async; false = sync)",
-                  true, &mca_btl_openib_component.cuda_async_send));
-    if (mca_btl_openib_component.cuda_async_send) {
-        mca_btl_openib_module.super.btl_flags |= MCA_BTL_FLAGS_CUDA_COPY_ASYNC_SEND;
-    }
-    /* Default is enabling CUDA asynchronous receive copies */
-    CHECK(reg_bool("cuda_async_recv", NULL,
-                  "Enable or disable CUDA async recv copies "
-                  "(true = async; false = sync)",
-                  true, &mca_btl_openib_component.cuda_async_recv));
-    if (mca_btl_openib_component.cuda_async_recv) {
-        mca_btl_openib_module.super.btl_flags |= MCA_BTL_FLAGS_CUDA_COPY_ASYNC_RECV;
-    }
-    /* Also make the max send size larger for better GPU buffer performance */
-   mca_btl_openib_module.super.btl_max_send_size = 128 * 1024;
-   /* Turn of message coalescing - not sure if it works with GPU buffers */
-   mca_btl_openib_component.use_message_coalescing = 0;
-#endif /* OMPI_CUDA_SUPPORT */
-
     /* setup all the qp stuff */
     /* round mid_qp_size to smallest power of two */
     mid_qp_size = opal_next_poweroftwo (mca_btl_openib_module.super.btl_eager_limit / 4) >> 1;
@@ -584,12 +653,7 @@ int btl_openib_register_mca_params(void)
             (uint32_t)mca_btl_openib_module.super.btl_eager_limit,
             (uint32_t)mca_btl_openib_module.super.btl_max_send_size);
 
-    mca_btl_openib_component.default_recv_qps = strdup(default_qps);
-    if(NULL == mca_btl_openib_component.default_recv_qps) {
-        BTL_ERROR(("Unable to allocate memory for default receive queues string.\n"));
-        return OMPI_ERROR;
-    }
-
+    mca_btl_openib_component.default_recv_qps = default_qps;
     CHECK(reg_string("receive_queues", NULL,
                      "Colon-delimited, comma-delimited list of receive queues: P,4096,8,6,4:P,32768,8,6,4",
                      default_qps, &mca_btl_openib_component.receive_queues,
@@ -711,6 +775,35 @@ int btl_openib_verify_mca_params (void)
         mca_btl_openib_component.buffer_alignment = 64;
     }
 
+#if OPAL_CUDA_SUPPORT /* CUDA_ASYNC_RECV */
+    if (mca_btl_openib_component.cuda_async_send) {
+        mca_btl_openib_module.super.btl_flags |= MCA_BTL_FLAGS_CUDA_COPY_ASYNC_SEND;
+    } else {
+        mca_btl_openib_module.super.btl_flags &= ~MCA_BTL_FLAGS_CUDA_COPY_ASYNC_SEND;
+    }
+
+    if (mca_btl_openib_component.cuda_async_recv) {
+        mca_btl_openib_module.super.btl_flags |= MCA_BTL_FLAGS_CUDA_COPY_ASYNC_RECV;
+    } else {
+        mca_btl_openib_module.super.btl_flags &= ~MCA_BTL_FLAGS_CUDA_COPY_ASYNC_RECV;
+    }
+    /* Cannot have fork support and GDR on at the same time.  If the user asks for both,
+     * then print a message and return error.  If the user does not explicitly ask for
+     * fork support, then turn it off in the presence of GDR.  */
+    if (mca_btl_openib_component.cuda_want_gdr && mca_btl_openib_component.cuda_have_gdr &&
+        mca_btl_openib_component.driver_have_gdr) {
+        if (1 == mca_btl_openib_component.want_fork_support) {
+              opal_show_help("help-mpi-btl-openib.txt", "no_fork_with_gdr",
+                             true, ompi_process_info.nodename);
+              return OMPI_ERR_BAD_PARAM;
+        }
+        if (-1 == mca_btl_openib_component.want_fork_support) {
+            mca_btl_openib_component.want_fork_support = 0;
+        }
+    }
+#endif
+
+#if BTL_OPENIB_MALLOC_HOOKS_ENABLED
     if (mca_btl_openib_component.use_memalign != 32  
         && mca_btl_openib_component.use_memalign != 64
         && mca_btl_openib_component.use_memalign != 0){ 
@@ -719,6 +812,7 @@ int btl_openib_verify_mca_params (void)
                        "btl_openib_memalign is reset to 32");
         mca_btl_openib_component.use_memalign = 32; 
     }
+#endif
 
     return OMPI_SUCCESS;
 }
