@@ -10,7 +10,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007-2008 Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2014      Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -131,7 +132,7 @@ static void mca_btl_tcp_endpoint_dump(mca_btl_base_endpoint_t* btl_endpoint, con
     char src[64];
     char dst[64];
     int sndbuf,rcvbuf,nodelay,flags;
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     struct sockaddr_storage inaddr;
 #else
     struct sockaddr_in inaddr;
@@ -140,7 +141,7 @@ static void mca_btl_tcp_endpoint_dump(mca_btl_base_endpoint_t* btl_endpoint, con
     opal_socklen_t addrlen = sizeof(inaddr);
 
     getsockname(btl_endpoint->endpoint_sd, (struct sockaddr*)&inaddr, &addrlen);
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     {
         char *address;
         address = (char *) opal_net_get_hostname((struct sockaddr*) &inaddr);
@@ -152,7 +153,7 @@ static void mca_btl_tcp_endpoint_dump(mca_btl_base_endpoint_t* btl_endpoint, con
     sprintf(src, "%s", inet_ntoa(inaddr.sin_addr));
 #endif
     getpeername(btl_endpoint->endpoint_sd, (struct sockaddr*)&inaddr, &addrlen);
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     {
         char *address;
         address = (char *) opal_net_get_hostname ((struct sockaddr*) &inaddr);
@@ -511,9 +512,9 @@ static int mca_btl_tcp_endpoint_recv_connect_ack(mca_btl_base_endpoint_t* btl_en
 
 void mca_btl_tcp_set_socket_options(int sd)
 {
-    int optval;
 #if defined(TCP_NODELAY)
-    optval = mca_btl_tcp_component.tcp_not_use_nodelay;
+    int optval;
+    optval = !mca_btl_tcp_component.tcp_not_use_nodelay;
     if(setsockopt(sd, IPPROTO_TCP, TCP_NODELAY, (char *)&optval, sizeof(optval)) < 0) {
         BTL_ERROR(("setsockopt(TCP_NODELAY) failed: %s (%d)", 
                    strerror(opal_socket_errno), opal_socket_errno));
@@ -552,7 +553,7 @@ static int mca_btl_tcp_endpoint_start_connect(mca_btl_base_endpoint_t* btl_endpo
     uint16_t af_family = AF_INET;
     opal_socklen_t addrlen = sizeof(struct sockaddr_in);
     
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     if (AF_INET6 == btl_endpoint->endpoint_addr->addr_family) {
         af_family = AF_INET6;
         addrlen = sizeof (struct sockaddr_in6);
