@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -11,6 +12,8 @@
  *                         All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2012      Oak Ridge National Labs.  All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -57,10 +60,6 @@
  * halving is used to be nice to the app memory wise.  There are much
  * better algorithms for large messages with cummutative operations,
  * so this should be investigated further.
- *
- * NOTE: We default to a simple reduce/scatterv if one of the rcounts
- * is zero.  This is because the existing algorithms do not currently
- * support a count of zero in the array.
  */
 int
 mca_coll_basic_reduce_scatter_intra(void *sbuf, void *rbuf, int *rcounts,
@@ -74,8 +73,6 @@ mca_coll_basic_reduce_scatter_intra(void *sbuf, void *rbuf, int *rcounts,
     int *disps = NULL;
     char *recv_buf = NULL, *recv_buf_free = NULL;
     char *result_buf = NULL, *result_buf_free = NULL;
-    bool zerocounts = false;
-
     /* Initialize */
     rank = ompi_comm_rank(comm);
     size = ompi_comm_size(comm);
@@ -87,14 +84,8 @@ mca_coll_basic_reduce_scatter_intra(void *sbuf, void *rbuf, int *rcounts,
     disps[0] = 0;
     for (i = 0; i < (size - 1); ++i) {
         disps[i + 1] = disps[i] + rcounts[i];
-        if (0 == rcounts[i]) {
-            zerocounts = true;
-        }
     }
     count = disps[size - 1] + rcounts[size - 1];
-    if (0 == rcounts[size - 1]) {
-        zerocounts = true;
-    }
 
     /* short cut the trivial case */
     if (0 == count) {
@@ -113,7 +104,7 @@ mca_coll_basic_reduce_scatter_intra(void *sbuf, void *rbuf, int *rcounts,
     }
 
     if ((op->o_flags & OMPI_OP_FLAGS_COMMUTE) &&
-        (buf_size < COMMUTATIVE_LONG_MSG) && (!zerocounts)) {
+        (buf_size < COMMUTATIVE_LONG_MSG)) {
         int tmp_size, remain = 0, tmp_rank;
 
         /* temporary receive buffer.  See coll_basic_reduce.c for details on sizing */
@@ -241,7 +232,7 @@ mca_coll_basic_reduce_scatter_intra(void *sbuf, void *rbuf, int *rcounts,
 
                 /* actual data transfer.  Send from result_buf,
                    receive into recv_buf */
-                if (send_count > 0 && recv_count != 0) {
+                if (recv_count > 0) {
                     err = MCA_PML_CALL(irecv(recv_buf + tmp_disps[recv_index] * extent,
                                              recv_count, dtype, peer,
                                              MCA_COLL_BASE_TAG_REDUCE_SCATTER,
@@ -252,7 +243,7 @@ mca_coll_basic_reduce_scatter_intra(void *sbuf, void *rbuf, int *rcounts,
                         goto cleanup;
                     }                                             
                 }
-                if (recv_count > 0 && send_count != 0) {
+                if (send_count > 0) {
                     err = MCA_PML_CALL(send(result_buf + tmp_disps[send_index] * extent,
                                             send_count, dtype, peer, 
                                             MCA_COLL_BASE_TAG_REDUCE_SCATTER,
@@ -264,18 +255,17 @@ mca_coll_basic_reduce_scatter_intra(void *sbuf, void *rbuf, int *rcounts,
                         goto cleanup;
                     }                                             
                 }
-                if (send_count > 0 && recv_count != 0) {
+
+                /* if we received something on this step, push it into
+                   the results buffer */
+                if (recv_count > 0) {
                     err = ompi_request_wait(&request, MPI_STATUS_IGNORE);
                     if (OMPI_SUCCESS != err) {
                         free(tmp_rcounts);
                         free(tmp_disps);
                         goto cleanup;
                     }                                             
-                }
 
-                /* if we received something on this step, push it into
-                   the results buffer */
-                if (recv_count > 0) {
                     ompi_op_reduce(op, 
                                    recv_buf + tmp_disps[recv_index] * extent, 
                                    result_buf + tmp_disps[recv_index] * extent,
