@@ -6,9 +6,11 @@
 
  $HEADER$
  */
+
 #include "ompi_config.h"
 #include "coll_hcoll.h"
 
+int hcoll_comm_attr_keyval;
 
 /*
  * Initial query function that is invoked during MPI_INIT, allowing
@@ -16,7 +18,6 @@
  */
 int mca_coll_hcoll_init_query(bool enable_progress_threads, bool enable_mpi_threads)
 {
-
     return OMPI_SUCCESS;
 }
 
@@ -48,24 +49,45 @@ static void mca_coll_hcoll_module_construct(mca_coll_hcoll_module_t *hcoll_modul
 
 static void mca_coll_hcoll_module_destruct(mca_coll_hcoll_module_t *hcoll_module)
 {
-    OBJ_RELEASE(hcoll_module->previous_barrier_module);
-    OBJ_RELEASE(hcoll_module->previous_bcast_module);
-    OBJ_RELEASE(hcoll_module->previous_reduce_module);
-    OBJ_RELEASE(hcoll_module->previous_allreduce_module);
-    OBJ_RELEASE(hcoll_module->previous_allgather_module);
-    OBJ_RELEASE(hcoll_module->previous_allgatherv_module);
-    OBJ_RELEASE(hcoll_module->previous_gather_module);
-    OBJ_RELEASE(hcoll_module->previous_gatherv_module);
-    OBJ_RELEASE(hcoll_module->previous_alltoall_module);
-    OBJ_RELEASE(hcoll_module->previous_alltoallv_module);
-    OBJ_RELEASE(hcoll_module->previous_alltoallw_module);
-    OBJ_RELEASE(hcoll_module->previous_reduce_scatter_module);
-    OBJ_RELEASE(hcoll_module->previous_ibarrier_module);
-    OBJ_RELEASE(hcoll_module->previous_ibcast_module);
-    OBJ_RELEASE(hcoll_module->previous_iallreduce_module);
-    OBJ_RELEASE(hcoll_module->previous_iallgather_module);
-    hcoll_destroy_context(hcoll_module->hcoll_context,
-                          (rte_grp_handle_t) hcoll_module->comm);
+    mca_coll_hcoll_module_t *module;
+    ompi_communicator_t *comm;
+    int context_destroyed;
+
+    if (hcoll_module->comm == &ompi_mpi_comm_world.comm){
+        if (OMPI_SUCCESS != ompi_attr_free_keyval(COMM_ATTR, &hcoll_comm_attr_keyval, 0)) {
+            HCOL_VERBOSE(1,"hcoll ompi_attr_free_keyval failed");
+        }
+    }
+
+    /* If the hcoll_context is null then we are destroying the hcoll_module
+       that didn't initialized fallback colls/modules.
+       Then just clear and return. Otherwise release module pointers and
+       destroy hcoll context*/
+
+    if (hcoll_module->hcoll_context != NULL){
+        OBJ_RELEASE(hcoll_module->previous_barrier_module);
+        OBJ_RELEASE(hcoll_module->previous_bcast_module);
+        OBJ_RELEASE(hcoll_module->previous_reduce_module);
+        OBJ_RELEASE(hcoll_module->previous_allreduce_module);
+        OBJ_RELEASE(hcoll_module->previous_allgather_module);
+        OBJ_RELEASE(hcoll_module->previous_allgatherv_module);
+        OBJ_RELEASE(hcoll_module->previous_gather_module);
+        OBJ_RELEASE(hcoll_module->previous_gatherv_module);
+        OBJ_RELEASE(hcoll_module->previous_alltoall_module);
+        OBJ_RELEASE(hcoll_module->previous_alltoallv_module);
+        OBJ_RELEASE(hcoll_module->previous_alltoallw_module);
+        OBJ_RELEASE(hcoll_module->previous_reduce_scatter_module);
+        OBJ_RELEASE(hcoll_module->previous_ibarrier_module);
+        OBJ_RELEASE(hcoll_module->previous_ibcast_module);
+        OBJ_RELEASE(hcoll_module->previous_iallreduce_module);
+        OBJ_RELEASE(hcoll_module->previous_iallgather_module);
+
+        context_destroyed = 0;
+        hcoll_destroy_context(hcoll_module->hcoll_context,
+                              (rte_grp_handle_t)hcoll_module->comm,
+                              &context_destroyed);
+        assert(context_destroyed);
+    }
     mca_coll_hcoll_module_clear(hcoll_module);
 }
 
@@ -79,9 +101,10 @@ static void mca_coll_hcoll_module_destruct(mca_coll_hcoll_module_t *hcoll_module
 } while(0)
 
 
-static int __save_coll_handlers(mca_coll_hcoll_module_t *hcoll_module)
+static int mca_coll_hcoll_save_coll_handlers(mca_coll_hcoll_module_t *hcoll_module)
 {
-    ompi_communicator_t *comm = hcoll_module->comm;
+    ompi_communicator_t *comm;
+    comm = hcoll_module->comm;
 
     HCOL_SAVE_PREV_COLL_API(barrier);
     HCOL_SAVE_PREV_COLL_API(bcast);
@@ -104,76 +127,51 @@ static int __save_coll_handlers(mca_coll_hcoll_module_t *hcoll_module)
 }
 
 
+
+/*
+** Communicator free callback
+*/
+int hcoll_comm_attr_del_fn(MPI_Comm comm, int keyval, void *attr_val, void *extra)
+{
+
+    mca_coll_hcoll_module_t *hcoll_module;
+    hcoll_module = (mca_coll_hcoll_module_t*) attr_val;
+
+    hcoll_group_destroy_notify(hcoll_module->hcoll_context);
+    return OMPI_SUCCESS;
+
+}
 /*
  * Initialize module on the communicator
  */
 static int mca_coll_hcoll_module_enable(mca_coll_base_module_t *module,
                                         struct ompi_communicator_t *comm)
 {
-    mca_coll_hcoll_module_t *hcoll_module = (mca_coll_hcoll_module_t*) module;
-    hcoll_module->comm = comm;
+    int ret;
 
-    if (OMPI_SUCCESS != __save_coll_handlers(hcoll_module)){
-        HCOL_ERROR("coll_hcol: __save_coll_handlers failed");
+    if (OMPI_SUCCESS != mca_coll_hcoll_save_coll_handlers((mca_coll_hcoll_module_t *)module)){
+        HCOL_ERROR("coll_hcol: mca_coll_hcoll_save_coll_handlers failed");
         return OMPI_ERROR;
     }
 
-    hcoll_set_runtime_tag_offset(-100,mca_pml.pml_max_tag);
-    hcoll_set_rte_halt_flag_address(&ompi_mpi_finalized);
-    hcoll_set_rte_halt_flag_size(sizeof(ompi_mpi_finalized));
-
-    hcoll_module->hcoll_context =
-        hcoll_create_context((rte_grp_handle_t)comm);
-    if (NULL == hcoll_module->hcoll_context){
-        HCOL_VERBOSE(1,"hcoll_create_context returned NULL");
+    ret = ompi_attr_set_c(COMM_ATTR, comm, &comm->c_keyhash, hcoll_comm_attr_keyval, (void *)module, false);
+    if (OMPI_SUCCESS != ret) {
+        HCOL_VERBOSE(1,"hcoll ompi_attr_set_c failed");
         return OMPI_ERROR;
     }
 
-
-#if 0
-    {
-        printf("\033[33mrank %d: DOING EXTRA TEST\033[0m\n",ompi_comm_rank(comm));
-        fflush(stdout);
-        sleep(1);
-        rte_ec_handle_t handle;
-        rte_grp_handle_t world_group = hcoll_rte_functions.rte_world_group_fn();
-        int peer;
-        const int max_count = 10000000;
-        const int step = max_count/100;
-        int buf = 0;
-        int i;
-        rte_request_handle_t req;
-        peer = (ompi_comm_rank(comm)+1)%2;
-        hcoll_rte_functions.get_ec_handles_fn(1,&peer,world_group,&handle);
-
-        for (i=1; i<max_count+1; i++){
-            if (0 == ompi_comm_rank(comm)){
-                if (i/step*step == i){
-                    printf("%d %% done...\n",i/step);fflush(stdout);
-                }
-                buf = 1;
-                hcoll_rte_functions.send_fn(DTE_INT32,1,&buf,handle,world_group,0,&req);
-            } else {
-                hcoll_rte_functions.recv_fn(DTE_INT32,1,&buf,handle,world_group,0,&req);
-            }
-            int completed = 0;
-            hcoll_rte_functions.test_fn(&req,&completed);
-            while(!completed){
-                hcoll_rte_functions.test_fn(&req,&completed);
-                /*hcoll_rte_functions.rte_progress_fn();*/
-                opal_progress();
-            }
-        }
-        printf("\033[32mrank %d: EXTRA TEST PASS\033[0m\n",ompi_comm_rank(comm));
-        fflush(stdout);
-        sleep(1);
-    }
-#endif
     return OMPI_SUCCESS;
 }
 
+int mca_coll_hcoll_progress(void)
+{
+    if (ompi_mpi_finalized){
+        hcoll_rte_p2p_disabled_notify();
+    }
 
-
+    (*hcoll_progress_fn)();
+    return OMPI_SUCCESS;
+}
 
 
 /*
@@ -186,41 +184,85 @@ mca_coll_hcoll_comm_query(struct ompi_communicator_t *comm, int *priority)
 {
     mca_coll_base_module_t *module;
     mca_coll_hcoll_module_t *hcoll_module;
-    static bool libhcoll_initialized = false;
+    ompi_attribute_fn_ptr_union_t del_fn;
+    ompi_attribute_fn_ptr_union_t copy_fn;
+    mca_coll_hcoll_component_t *cm;
+    int err;
+    int rc;
+    cm = &mca_coll_hcoll_component;
     *priority = 0;
     module = NULL;
 
-    if (!mca_coll_hcoll_component.hcoll_enable){
-        goto exit;
+    if (!cm->hcoll_enable){
+        return NULL;
     }
 
-    if (!libhcoll_initialized)
+    if (OMPI_COMM_IS_INTER(comm) || ompi_comm_size(comm) < cm->hcoll_np
+        || ompi_comm_size(comm) < 2){
+        return NULL;
+    }
+
+
+    if (!cm->libhcoll_initialized)
     {
         /* libhcoll should be initialized here since current implmentation of
            mxm bcol in libhcoll needs world_group fully functional during init
-
            world_group, i.e. ompi_comm_world, is not ready at hcoll component open
            call */
-        opal_progress_register(hcoll_progress_fn);
-        int rc = hcoll_init();
+        opal_progress_register(mca_coll_hcoll_progress);
+
+        hcoll_set_runtime_tag_offset(MCA_COLL_BASE_TAG_HCOLL_BASE, mca_pml.pml_max_tag);
+
+        HCOL_VERBOSE(10,"Calling hcoll_init();");
+        rc = hcoll_init();
 
         if (HCOLL_SUCCESS != rc){
-            mca_coll_hcoll_component.hcoll_enable = 0;
+            cm->hcoll_enable = 0;
             opal_progress_unregister(hcoll_progress_fn);
-            HCOL_VERBOSE(0,"Hcol library init failed");
+            HCOL_ERROR("Hcol library init failed");
             return NULL;
         }
-        libhcoll_initialized = true;
+
+        copy_fn.attr_communicator_copy_fn = (MPI_Comm_internal_copy_attr_function*) MPI_COMM_NULL_COPY_FN;
+        del_fn.attr_communicator_delete_fn = hcoll_comm_attr_del_fn;
+        err = ompi_attr_create_keyval(COMM_ATTR, copy_fn, del_fn, &hcoll_comm_attr_keyval, NULL ,0, NULL);
+        if (OMPI_SUCCESS != err) {
+            cm->hcoll_enable = 0;
+            hcoll_finalize();
+            opal_progress_unregister(hcoll_progress_fn);
+            HCOL_ERROR("Hcol comm keyval create failed");
+            return NULL;
+        }
+
     }
     hcoll_module = OBJ_NEW(mca_coll_hcoll_module_t);
     if (!hcoll_module){
-        goto exit;
+        if (!cm->libhcoll_initialized) {
+            cm->hcoll_enable = 0;
+            hcoll_finalize();
+            opal_progress_unregister(hcoll_progress_fn);
+        }
+        return NULL;
     }
 
-    if (ompi_comm_size(comm) < 2 || OMPI_COMM_IS_INTER(comm)){
-        goto exit;
-    }
+    hcoll_module->comm = comm;
 
+    HCOL_VERBOSE(10,"Creating hcoll_context for comm %p, comm_id %d, comm_size %d",
+                 (void*)comm,comm->c_contextid,ompi_comm_size(comm));
+
+    hcoll_module->hcoll_context =
+        hcoll_create_context((rte_grp_handle_t)comm);
+
+    if (NULL == hcoll_module->hcoll_context){
+        HCOL_VERBOSE(1,"hcoll_create_context returned NULL");
+        OBJ_RELEASE(hcoll_module);
+        if (!cm->libhcoll_initialized) {
+            cm->hcoll_enable = 0;
+            hcoll_finalize();
+            opal_progress_unregister(hcoll_progress_fn);
+        }
+        return NULL;
+    }
 
 
     hcoll_module->super.coll_module_enable = mca_coll_hcoll_module_enable;
@@ -233,12 +275,15 @@ mca_coll_hcoll_comm_query(struct ompi_communicator_t *comm, int *priority)
     hcoll_module->super.coll_ibcast = hcoll_collectives.coll_ibcast ? mca_coll_hcoll_ibcast : NULL;
     hcoll_module->super.coll_iallgather = hcoll_collectives.coll_iallgather ? mca_coll_hcoll_iallgather : NULL;
     hcoll_module->super.coll_iallreduce = hcoll_collectives.coll_iallreduce ? mca_coll_hcoll_iallreduce : NULL;
+    hcoll_module->super.coll_gather = hcoll_collectives.coll_gather ? mca_coll_hcoll_gather : NULL;
 
-    *priority = mca_coll_hcoll_component.hcoll_priority;
+    *priority = cm->hcoll_priority;
     module = &hcoll_module->super;
 
+    if (!cm->libhcoll_initialized) {
+        cm->libhcoll_initialized = true;
+    }
 
-exit:
     return module;
 }
 
