@@ -21,7 +21,7 @@
  * This file contains various support functions for doing CUDA
  * operations.  Some of the features are only available in CUDA 4.1
  * and later, so some code is conditionalized around the
- * OMPI_CUDA_SUPPORT_41 macro.
+ * OPAL_CUDA_SUPPORT_41 macro.
  */
 #include "ompi_config.h"
 
@@ -81,16 +81,20 @@ struct cudaFunctionTable {
     int (*cuEventDestroy)(CUevent);
     int (*cuStreamWaitEvent)(CUstream, CUevent, unsigned int);
     int (*cuMemGetAddressRange)(CUdeviceptr*, size_t*, CUdeviceptr);
-#if OMPI_CUDA_SUPPORT_41
+#if OPAL_CUDA_SUPPORT_41
     int (*cuIpcGetEventHandle)(CUipcEventHandle*, CUevent);
     int (*cuIpcOpenEventHandle)(CUevent*, CUipcEventHandle);
     int (*cuIpcOpenMemHandle)(CUdeviceptr*, CUipcMemHandle, unsigned int);
     int (*cuIpcCloseMemHandle)(CUdeviceptr);
     int (*cuIpcGetMemHandle)(CUipcMemHandle*, CUdeviceptr);
-#endif /* OMPI_CUDA_SUPPORT_41 */
+#endif /* OPAL_CUDA_SUPPORT_41 */
     int (*cuCtxGetDevice)(CUdevice *);
     int (*cuDeviceCanAccessPeer)(int *, CUdevice, CUdevice);
     int (*cuDeviceGet)(CUdevice *, int);
+#if OPAL_CUDA_GDR_SUPPORT
+    int (*cuPointerSetAttribute)(const void *, CUpointer_attribute, CUdeviceptr);
+#endif /* OPAL_CUDA_GDR_SUPPORT */
+    int (*cuCtxSetCurrent)(CUcontext);
 } cudaFunctionTable;
 typedef struct cudaFunctionTable cudaFunctionTable_t;
 cudaFunctionTable_t cuFunc;
@@ -100,7 +104,7 @@ static bool stage_three_init_complete = false;
 static bool common_cuda_initialized = false;
 static int mca_common_cuda_verbose;
 static int mca_common_cuda_output = 0;
-static bool mca_common_cuda_enabled = false;
+bool mca_common_cuda_enabled = false;
 static bool mca_common_cuda_register_memory = true;
 static bool mca_common_cuda_warning = false;
 static opal_list_t common_cuda_memory_registrations;
@@ -129,7 +133,7 @@ OBJ_CLASS_INSTANCE(common_cuda_mem_regs_t,
                    NULL,
                    NULL);
 
-#if OMPI_CUDA_SUPPORT_41
+#if OPAL_CUDA_SUPPORT_41
 static int mca_common_cuda_async = 1;
 
 /* Array of CUDA events to be queried for IPC stream, sending side and
@@ -154,7 +158,10 @@ int cuda_event_ipc_first_used, cuda_event_dtoh_first_used, cuda_event_htod_first
 int cuda_event_ipc_num_used, cuda_event_dtoh_num_used, cuda_event_htod_num_used;
 
 /* Size of array holding events */
-int cuda_event_max = 200;
+int cuda_event_max = 400;
+static int cuda_event_ipc_most = 0;
+static int cuda_event_dtoh_most = 0;
+static int cuda_event_htod_most = 0;
 
 /* Handle to libcuda.so */
 opal_lt_dlhandle libcuda_handle;
@@ -182,7 +189,7 @@ static void cuda_dump_memhandle(int, void *, char *) __opal_attribute_unused__ ;
 #define CUDA_DUMP_EVTHANDLE(a)
 #endif /* OPAL_ENABLE_DEBUG */
 
-#endif /* OMPI_CUDA_SUPPORT_41 */
+#endif /* OPAL_CUDA_SUPPORT_41 */
 
 
 /**
@@ -219,7 +226,7 @@ static int mca_common_cuda_init(opal_common_cuda_function_table_t *ftable)
  * will result in this initialization failing and status will be set 
  * showing that.
  */
-static int mca_common_cuda_stage_one_init(void)
+int mca_common_cuda_stage_one_init(void)
 {
     opal_lt_dladvise advise;
     int retval, i, j;
@@ -231,6 +238,9 @@ static int mca_common_cuda_stage_one_init(void)
     int errsize;
     bool stage_one_init_passed = false;
 
+    if (true == stage_one_init_complete) {
+        return 0;
+    }
     stage_one_init_complete = true;
 
     /* Set different levels of verbosity in the cuda related code. */
@@ -263,7 +273,7 @@ static int mca_common_cuda_stage_one_init(void)
                                  MCA_BASE_VAR_SCOPE_READONLY,
                                  &mca_common_cuda_warning);
 
-#if OMPI_CUDA_SUPPORT_41
+#if OPAL_CUDA_SUPPORT_41
     /* Use this flag to test async vs sync copies */
     mca_common_cuda_async = 1;
     (void) mca_base_var_register("ompi", "mpi", "common_cuda", "memcpy_async",
@@ -274,18 +284,24 @@ static int mca_common_cuda_stage_one_init(void)
                                  &mca_common_cuda_async);
 
     /* Use this parameter to increase the number of outstanding events allows */
-    cuda_event_max = 200;
     (void) mca_base_var_register("ompi", "mpi", "common_cuda", "event_max",
                                  "Set number of oustanding CUDA events",
                                  MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
                                  OPAL_INFO_LVL_9,
                                  MCA_BASE_VAR_SCOPE_READONLY,
                                  &cuda_event_max);
-#endif /* OMPI_CUDA_SUPPORT_41 */
+#endif /* OPAL_CUDA_SUPPORT_41 */
 
     mca_common_cuda_output = opal_output_open(NULL);
     opal_output_set_verbosity(mca_common_cuda_output, mca_common_cuda_verbose);
 
+    /* First check if the support is enabled.  In the case that the user has
+     * turned it off, we do not need to continue with any CUDA specific
+     * initialization.  Do this after MCA parameter registration. */
+    if (!ompi_mpi_cuda_support) {
+        return 1;
+    }
+
     if (0 != (retval = opal_lt_dlinit())) {
         if (OPAL_ERR_NOT_SUPPORTED == retval) {
             opal_show_help("help-mpi-common-cuda.txt", "dlopen disabled", true);
@@ -413,6 +429,8 @@ static int mca_common_cuda_stage_one_init(void)
     if (true != stage_one_init_passed) {
         return 1;
     }
+    opal_cuda_add_initialization_function(&mca_common_cuda_init);
+    OBJ_CONSTRUCT(&common_cuda_memory_registrations, opal_list_t);
 
     /* Map in the functions that we need.  Note that if there is an error
      * the macro OMPI_CUDA_DLSYM will print an error and call return.  */
@@ -431,16 +449,20 @@ static int mca_common_cuda_stage_one_init(void)
     OMPI_CUDA_DLSYM(libcuda_handle, cuMemFree);
     OMPI_CUDA_DLSYM(libcuda_handle, cuMemAlloc);
     OMPI_CUDA_DLSYM(libcuda_handle, cuMemGetAddressRange);
-#if OMPI_CUDA_SUPPORT_41
+#if OPAL_CUDA_SUPPORT_41
     OMPI_CUDA_DLSYM(libcuda_handle, cuIpcGetEventHandle);
     OMPI_CUDA_DLSYM(libcuda_handle, cuIpcOpenEventHandle);
     OMPI_CUDA_DLSYM(libcuda_handle, cuIpcOpenMemHandle);
     OMPI_CUDA_DLSYM(libcuda_handle, cuIpcCloseMemHandle);
     OMPI_CUDA_DLSYM(libcuda_handle, cuIpcGetMemHandle);
-#endif /* OMPI_CUDA_SUPPORT_41 */
+#endif /* OPAL_CUDA_SUPPORT_41 */
     OMPI_CUDA_DLSYM(libcuda_handle, cuCtxGetDevice);
     OMPI_CUDA_DLSYM(libcuda_handle, cuDeviceCanAccessPeer);
     OMPI_CUDA_DLSYM(libcuda_handle, cuDeviceGet);
+#if OPAL_CUDA_GDR_SUPPORT
+    OMPI_CUDA_DLSYM(libcuda_handle, cuPointerSetAttribute);
+#endif /* OPAL_CUDA_GDR_SUPPORT */
+    OMPI_CUDA_DLSYM(libcuda_handle, cuCtxSetCurrent);
     return 0;
 }
 
@@ -516,7 +538,7 @@ static int mca_common_cuda_stage_three_init(void)
         return OMPI_ERROR;
     }
 
-#if OMPI_CUDA_SUPPORT_41
+#if OPAL_CUDA_SUPPORT_41
     if (true == mca_common_cuda_enabled) {
         /* Set up an array to store outstanding IPC async copy events */
         cuda_event_ipc_array = NULL;
@@ -553,7 +575,7 @@ static int mca_common_cuda_stage_three_init(void)
         }
     }
 
-#endif /* OMPI_CUDA_SUPPORT_41 */
+#endif /* OPAL_CUDA_SUPPORT_41 */
     if (true == mca_common_cuda_enabled) {
         /* Set up an array to store outstanding async dtoh events.  Used on the
          * sending side for asynchronous copies. */
@@ -702,8 +724,6 @@ void mca_common_cuda_register(void *ptr, size_t amount, char *msg) {
             ompi_mpi_cuda_support = 0;
             return;
         }
-        opal_cuda_add_initialization_function(&mca_common_cuda_init);
-        OBJ_CONSTRUCT(&common_cuda_memory_registrations, opal_list_t);
     }
 
     if (!common_cuda_initialized) {
@@ -773,7 +793,7 @@ void mca_common_cuda_unregister(void *ptr, char *msg) {
     }
 }
 
-#if OMPI_CUDA_SUPPORT_41
+#if OPAL_CUDA_SUPPORT_41
 /*
  * Get the memory handle of a local section of memory that can be sent
  * to the remote size so it can access the memory.  This is the
@@ -829,6 +849,21 @@ int cuda_getmemhandle(void *base, size_t size, mca_mpool_base_registration_t *ne
     cuda_reg->base.bound = (unsigned char *)pbase + psize - 1;
     memcpy(&cuda_reg->memHandle, &memHandle, sizeof(memHandle));
 
+#if OPAL_CUDA_SYNC_MEMOPS
+    /* With CUDA 6.0, we can set an attribute on the memory pointer that will
+     * ensure any synchronous copies are completed prior to any other access
+     * of the memory region.  This means we do not need to record an event
+     * and send to the remote side.
+     */
+    memType = 1; /* Just use this variable since we already have it */
+    result = cuFunc.cuPointerSetAttribute(&memType, CU_POINTER_ATTRIBUTE_SYNC_MEMOPS,
+                                          (CUdeviceptr)base);
+    if (CUDA_SUCCESS != result) {
+        opal_show_help("help-mpi-common-cuda.txt", "cuPointerSetAttribute failed",
+                       true, ompi_process_info.nodename, result, base);
+        return OMPI_ERROR;
+    }
+#else
     /* Need to record the event to ensure that any memcopies into the
      * device memory have completed.  The event handle associated with
      * this event is sent to the remote process so that it will wait
@@ -842,6 +877,7 @@ int cuda_getmemhandle(void *base, size_t size, mca_mpool_base_registration_t *ne
                        true, result, base);
         return OMPI_ERROR;
     }
+#endif /* OPAL_CUDA_SYNC_MEMOPS */
 
     return OMPI_SUCCESS;
 }
@@ -965,6 +1001,10 @@ void mca_common_cuda_destruct_event(uint64_t *event)
  */
 void mca_common_wait_stream_synchronize(mca_mpool_common_cuda_reg_t *rget_reg)
 {
+#if OPAL_CUDA_SYNC_MEMOPS
+    /* No need for any of this with SYNC_MEMOPS feature */
+    return;
+#else /* OPAL_CUDA_SYNC_MEMOPS */
     CUipcEventHandle evtHandle;
     CUevent event;
     CUresult result;
@@ -1002,6 +1042,7 @@ void mca_common_wait_stream_synchronize(mca_mpool_common_cuda_reg_t *rget_reg)
         opal_show_help("help-mpi-common-cuda.txt", "cuEventDestroy failed",
                        true, result);
     }
+#endif /* OPAL_CUDA_SYNC_MEMOPS */
 }
 
 /*
@@ -1023,6 +1064,15 @@ int mca_common_cuda_memcpy(void *dst, void *src, size_t amount, char *msg,
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
+    if (cuda_event_ipc_num_used > cuda_event_ipc_most) {
+        cuda_event_ipc_most = cuda_event_ipc_num_used;
+        /* Just print multiples of 10 */
+        if (0 == (cuda_event_ipc_most % 10)) {
+            opal_output_verbose(20, mca_common_cuda_output,
+                                "Maximum ipc events used is now %d", cuda_event_ipc_most);
+        }
+    }
+
     /* This is the standard way to run.  Running with synchronous copies is available
      * to measure the advantages of asynchronous copies. */
     if (OPAL_LIKELY(mca_common_cuda_async)) {
@@ -1130,6 +1180,15 @@ int mca_common_cuda_record_dtoh_event(char *msg, struct mca_btl_base_descriptor_
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
+    if (cuda_event_dtoh_num_used > cuda_event_dtoh_most) {
+        cuda_event_dtoh_most = cuda_event_dtoh_num_used;
+        /* Just print multiples of 10 */
+        if (0 == (cuda_event_dtoh_most % 10)) {
+            opal_output_verbose(20, mca_common_cuda_output,
+                                "Maximum DtoH events used is now %d", cuda_event_dtoh_most);
+        }
+    }
+
     result = cuFunc.cuEventRecord(cuda_event_dtoh_array[cuda_event_dtoh_first_avail], dtohStream);
     if (CUDA_SUCCESS != result) {
         opal_show_help("help-mpi-common-cuda.txt", "cuEventRecord failed",
@@ -1165,6 +1224,15 @@ int mca_common_cuda_record_htod_event(char *msg, struct mca_btl_base_descriptor_
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
+    if (cuda_event_htod_num_used > cuda_event_htod_most) {
+        cuda_event_htod_most = cuda_event_htod_num_used;
+        /* Just print multiples of 10 */
+        if (0 == (cuda_event_htod_most % 10)) {
+            opal_output_verbose(20, mca_common_cuda_output,
+                                "Maximum HtoD events used is now %d", cuda_event_htod_most);
+        }
+    }
+
     result = cuFunc.cuEventRecord(cuda_event_htod_array[cuda_event_htod_first_avail], htodStream);
     if (CUDA_SUCCESS != result) {
         opal_show_help("help-mpi-common-cuda.txt", "cuEventRecord failed",
@@ -1248,7 +1316,7 @@ int progress_one_cuda_dtoh_event(struct mca_btl_base_descriptor_t **frag) {
     CUresult result;
 
     if (cuda_event_dtoh_num_used > 0) {
-        opal_output_verbose(20, mca_common_cuda_output,
+        opal_output_verbose(30, mca_common_cuda_output,
                            "CUDA: progress_one_cuda_dtoh_event, outstanding_events=%d",
                             cuda_event_dtoh_num_used);
 
@@ -1256,7 +1324,7 @@ int progress_one_cuda_dtoh_event(struct mca_btl_base_descriptor_t **frag) {
 
         /* We found an event that is not ready, so return. */
         if (CUDA_ERROR_NOT_READY == result) {
-            opal_output_verbose(20, mca_common_cuda_output,
+            opal_output_verbose(30, mca_common_cuda_output,
                                 "CUDA: cuEventQuery returned CUDA_ERROR_NOT_READY");
             *frag = NULL;
             return 0;
@@ -1268,7 +1336,7 @@ int progress_one_cuda_dtoh_event(struct mca_btl_base_descriptor_t **frag) {
         }
 
         *frag = cuda_event_dtoh_frag_array[cuda_event_dtoh_first_used];
-        opal_output_verbose(10, mca_common_cuda_output,
+        opal_output_verbose(30, mca_common_cuda_output,
                             "CUDA: cuEventQuery returned %d", result);
 
         /* Bump counters, loop around the circular buffer if necessary */
@@ -1290,7 +1358,7 @@ int progress_one_cuda_htod_event(struct mca_btl_base_descriptor_t **frag) {
     CUresult result;
 
     if (cuda_event_htod_num_used > 0) {
-        opal_output_verbose(20, mca_common_cuda_output,
+        opal_output_verbose(30, mca_common_cuda_output,
                            "CUDA: progress_one_cuda_htod_event, outstanding_events=%d",
                             cuda_event_htod_num_used);
 
@@ -1298,7 +1366,7 @@ int progress_one_cuda_htod_event(struct mca_btl_base_descriptor_t **frag) {
 
         /* We found an event that is not ready, so return. */
         if (CUDA_ERROR_NOT_READY == result) {
-            opal_output_verbose(20, mca_common_cuda_output,
+            opal_output_verbose(30, mca_common_cuda_output,
                                 "CUDA: cuEventQuery returned CUDA_ERROR_NOT_READY");
             *frag = NULL;
             return 0;
@@ -1310,7 +1378,7 @@ int progress_one_cuda_htod_event(struct mca_btl_base_descriptor_t **frag) {
         }
 
         *frag = cuda_event_htod_frag_array[cuda_event_htod_first_used];
-        opal_output_verbose(10, mca_common_cuda_output,
+        opal_output_verbose(30, mca_common_cuda_output,
                             "CUDA: cuEventQuery returned %d", result);
 
         /* Bump counters, loop around the circular buffer if necessary */
@@ -1437,7 +1505,7 @@ static float mydifftime(struct timespec ts_start, struct timespec ts_end) {
 }
 #endif /* CUDA_COMMON_TIMING */
 
-#endif /* OMPI_CUDA_SUPPORT_41 */
+#endif /* OPAL_CUDA_SUPPORT_41 */
 
 /* Routines that get plugged into the opal datatype code */
 static int mca_common_cuda_is_gpu_buffer(const void *pUserBuf)
@@ -1445,6 +1513,7 @@ static int mca_common_cuda_is_gpu_buffer(const void *pUserBuf)
     int res;
     CUmemorytype memType;
     CUdeviceptr dbuf = (CUdeviceptr)pUserBuf;
+    CUcontext ctx = NULL;
 
     res = cuFunc.cuPointerGetAttribute(&memType,
                                        CU_POINTER_ATTRIBUTE_MEMORY_TYPE, dbuf);
@@ -1459,6 +1528,42 @@ static int mca_common_cuda_is_gpu_buffer(const void *pUserBuf)
     /* Must be a device pointer */
     assert(memType == CU_MEMORYTYPE_DEVICE);
 
+    /* This piece of code was added in to handle in a case involving
+     * OMP threads.  The user had initialized CUDA and then spawned
+     * two threads.  The first thread had the CUDA context, but the
+     * second thread did not.  We therefore had no context to act upon
+     * and future CUDA driver calls would fail.  Therefore, if we have
+     * GPU memory, but no context, get the context from the GPU memory
+     * and set the current context to that.  It is rare that we will not
+     * have a context. */
+    res = cuFunc.cuCtxGetCurrent(&ctx);
+    if (OPAL_UNLIKELY(NULL == ctx)) {
+        if (CUDA_SUCCESS == res) {
+            res = cuFunc.cuPointerGetAttribute(&ctx,
+                                               CU_POINTER_ATTRIBUTE_CONTEXT, dbuf);
+            if (res != CUDA_SUCCESS) {
+                opal_output(0, "CUDA: error calling cuPointerGetAttribute: "
+                            "res=%d, ptr=%p aborting...", res, pUserBuf);
+                ompi_rte_abort(1, NULL);
+            } else {
+                res = cuFunc.cuCtxSetCurrent(ctx);
+                if (res != CUDA_SUCCESS) {
+                    opal_output(0, "CUDA: error calling cuCtxSetCurrent: "
+                                "res=%d, ptr=%p aborting...", res, pUserBuf);
+                    ompi_rte_abort(1, NULL);
+                } else {
+                    opal_output_verbose(10, mca_common_cuda_output,
+                                        "CUDA: cuCtxSetCurrent passed: ptr=%p", pUserBuf);
+                }
+            }
+        } else {
+            /* Print error and proceed */
+            opal_output(0, "CUDA: error calling cuCtxGetCurrent: "
+                        "res=%d, ptr=%p aborting...", res, pUserBuf);
+            ompi_rte_abort(1, NULL);
+        }
+    }
+
     /* First access on a device pointer finalizes CUDA support initialization.
      * If initialization fails, disable support. */
     if (!stage_three_init_complete) {
@@ -1502,3 +1607,109 @@ static int mca_common_cuda_memmove(void *dest, void *src, size_t size)
     cuFunc.cuMemFree(tmp);
     return 0;
 }
+
+int mca_common_cuda_get_device(int *devicenum)
+{
+    CUdevice cuDev;
+    int res;
+
+    res = cuFunc.cuCtxGetDevice(&cuDev);
+    if(res != CUDA_SUCCESS){
+        opal_output(0, "CUDA: cuCtxGetDevice failed: res=%d",
+                    res);
+        return res;
+    }
+    *devicenum = cuDev;
+    return 0;
+}
+
+int mca_common_cuda_device_can_access_peer(int *access, int dev1, int dev2)
+{
+    int res;
+    res = cuFunc.cuDeviceCanAccessPeer(access, (CUdevice)dev1, (CUdevice)dev2);
+    if(res != CUDA_SUCCESS){
+        opal_output(0, "CUDA: cuDeviceCanAccessPeer failed: res=%d",
+                    res);
+        return res;
+    }
+    return 0;
+}
+
+int mca_common_cuda_get_address_range(void *pbase, size_t *psize, void *base)
+{
+    CUresult result;
+    result = cuFunc.cuMemGetAddressRange((CUdeviceptr *)pbase, psize, (CUdeviceptr)base);
+    if (CUDA_SUCCESS != result) {
+        opal_show_help("help-mpi-common-cuda.txt", "cuMemGetAddressRange failed",
+                       true, result, base);
+        return OMPI_ERROR;
+    } else {
+        opal_output_verbose(50, mca_common_cuda_output,
+                            "CUDA: cuMemGetAddressRange passed: addr=%p, pbase=%p, psize=%lu ",
+                            base, *(char **)pbase, *psize);
+    }
+    return 0;
+}
+
+#if OPAL_CUDA_GDR_SUPPORT
+/* Check to see if the memory was freed between the time it was stored in
+ * the registration cache and now.  Return true if the memory was previously
+ * freed.  This is indicated by the BUFFER_ID value in the registration cache
+ * not matching the BUFFER_ID of the buffer we are checking.  Return false
+ * if the registration is still good.
+ */
+bool mca_common_cuda_previously_freed_memory(mca_mpool_base_registration_t *reg)
+{
+    int res;
+    unsigned long long bufID;
+    unsigned char *dbuf = reg->base;
+
+    res = cuFunc.cuPointerGetAttribute(&bufID, CU_POINTER_ATTRIBUTE_BUFFER_ID,
+                                       (CUdeviceptr)dbuf);
+    /* If we cannot determine the BUFFER_ID, then print a message and default
+     * to forcing the registration to be kicked out. */
+    if (res != CUDA_SUCCESS) {
+        opal_show_help("help-mpi-common-cuda.txt", "bufferID failed",
+                       true, ompi_process_info.nodename, res);
+        return true;
+    }
+    opal_output_verbose(50, mca_common_cuda_output,
+                        "CUDA: base=%p, bufID=%llu, reg->gpu_bufID=%llu, %s", dbuf, bufID, reg->gpu_bufID,
+                        (reg->gpu_bufID == bufID ? "BUFFER_ID match":"BUFFER_ID do not match"));
+    if (bufID != reg->gpu_bufID) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
+/*
+ * Get the buffer ID from the memory and store it in the registration.
+ * This is needed to ensure the cached registration is not stale.  If
+ * we fail to get buffer ID, print an error and set buffer ID to 0.
+ * Also set SYNC_MEMOPS on any GPU registration to ensure that
+ * synchronous copies complete before the buffer is accessed.
+ */
+void mca_common_cuda_get_buffer_id(mca_mpool_base_registration_t *reg)
+{
+    int res;
+    unsigned long long bufID = 0;
+    unsigned char *dbuf = reg->base;
+    int enable = 1;
+
+    res = cuFunc.cuPointerGetAttribute(&bufID, CU_POINTER_ATTRIBUTE_BUFFER_ID,
+                                       (CUdeviceptr)dbuf);
+    if (res != CUDA_SUCCESS) {
+        opal_show_help("help-mpi-common-cuda.txt", "bufferID failed", true, res);
+    }
+    reg->gpu_bufID = bufID;
+
+    res = cuFunc.cuPointerSetAttribute(&enable, CU_POINTER_ATTRIBUTE_SYNC_MEMOPS,
+                                       (CUdeviceptr)dbuf);
+    if (CUDA_SUCCESS != res) {
+        opal_show_help("help-mpi-common-cuda.txt", "cuPointerSetAttribute failed",
+                       true, ompi_process_info.nodename, res, dbuf);
+    }
+}
+#endif /* OPAL_CUDA_GDR_SUPPORT */       
+
