@@ -7,6 +7,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -16,14 +17,17 @@
 
 #include "ompi_config.h"
 
-#include "osc_pt2pt.h"
-#include "osc_pt2pt_buffer.h"
-
 #include "opal/class/opal_free_list.h"
+#include "opal/types.h"
+
+#include "osc_pt2pt_buffer.h"
 
 static void ompi_osc_pt2pt_buffer_construct(ompi_osc_pt2pt_buffer_t *buf)
 {
-    buf->payload = buf + 1;
+    /* adjust payload location to account for alignment issues */
+    buf->payload = (void* )(((char*) buf) + 
+                            sizeof(ompi_osc_pt2pt_buffer_t) + 
+                            (sizeof(ompi_osc_pt2pt_buffer_t) % sizeof(ompi_ptr_t)));
 }
 
 
