@@ -77,6 +77,11 @@ mca_pml_ob1_t mca_pml_ob1 = {
     }
 };
 
+#if OPAL_CUDA_SUPPORT
+void mca_pml_ob1_cuda_add_ipc_support(struct mca_btl_base_module_t* btl,
+                                      int32_t flags, ompi_proc_t* errproc,
+                                      char* btlinfo);
+#endif /* OPAL_CUDA_SUPPORT */
 
 void mca_pml_ob1_error_handler( struct mca_btl_base_module_t* btl,
                                 int32_t flags, ompi_proc_t* errproc,
@@ -359,6 +364,48 @@ int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
             rc = OMPI_ERR_BAD_PARAM;
             goto cleanup_and_return;
         }
+#if OPAL_CUDA_GDR_SUPPORT
+        /* If size is SIZE_MAX, then we know we want to set this to the minimum possible
+         * value which is the size of the PML header. */
+        if (SIZE_MAX == sm->btl_module->btl_cuda_eager_limit) {
+            sm->btl_module->btl_cuda_eager_limit = sizeof(mca_pml_ob1_hdr_t);
+        }
+        /* If size is 0, then this value is unused.  If it is non-zero then do some
+         * extra checking of it. */
+        if (0 != sm->btl_module->btl_cuda_eager_limit) {
+            if (sm->btl_module->btl_cuda_eager_limit < sizeof(mca_pml_ob1_hdr_t)) {
+                opal_show_help("help-mpi-pml-ob1.txt", "cuda_eager_limit_too_small",
+                               true, 
+                               sm->btl_component->btl_version.mca_component_name,
+                               ompi_process_info.nodename,
+                               sm->btl_component->btl_version.mca_component_name,
+                               sm->btl_module->btl_cuda_eager_limit,
+                               sm->btl_component->btl_version.mca_component_name,
+                               sizeof(mca_pml_ob1_hdr_t),
+                               sm->btl_component->btl_version.mca_component_name);
+                rc = OMPI_ERR_BAD_PARAM;
+                goto cleanup_and_return;
+            }
+        }
+        if (0 == sm->btl_module->btl_cuda_rdma_limit) {
+            /* All is fine.  0 means to ignore value so set to SIZE_MAX */
+            sm->btl_module->btl_cuda_rdma_limit = SIZE_MAX;
+        } else {
+            if (sm->btl_module->btl_cuda_rdma_limit < sm->btl_module->btl_cuda_eager_limit) {
+                opal_show_help("help-mpi-pml-ob1.txt", "cuda_rdma_limit_too_small",
+                               true, 
+                               sm->btl_component->btl_version.mca_component_name,
+                               ompi_process_info.nodename,
+                               sm->btl_component->btl_version.mca_component_name,
+                               sm->btl_module->btl_cuda_rdma_limit,
+                               sm->btl_component->btl_version.mca_component_name,
+                               sm->btl_module->btl_cuda_eager_limit,
+                               sm->btl_component->btl_version.mca_component_name);
+                rc = OMPI_ERR_BAD_PARAM;
+                goto cleanup_and_return;
+            }
+        }
+#endif /* OPAL_CUDA_GDR_SUPPORT */
     }
 
 
@@ -609,6 +656,12 @@ void mca_pml_ob1_process_pending_rdma(void)
 void mca_pml_ob1_error_handler(
         struct mca_btl_base_module_t* btl, int32_t flags,
         ompi_proc_t* errproc, char* btlinfo ) { 
+#if OPAL_CUDA_SUPPORT
+    if (flags & MCA_BTL_ERROR_FLAGS_ADD_CUDA_IPC) {
+        mca_pml_ob1_cuda_add_ipc_support(btl, flags, errproc, btlinfo);
+        return;
+    }
+#endif /* OPAL_CUDA_SUPPORT */
     ompi_rte_abort(-1, NULL);
 }
 
@@ -627,7 +680,8 @@ int mca_pml_ob1_ft_event( int state )
     if(OPAL_CRS_CHECKPOINT == state) {
         if( opal_cr_timing_barrier_enabled ) {
             OPAL_CR_SET_TIMER(OPAL_CR_TIMER_CRCPBR1);
-            orte_grpcomm.barrier();
+            ompi_rte_barrier(coll);
+            OMPI_WAIT_FOR_COMPLETION(coll->active);
         }
 
         OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2P0);
@@ -638,7 +692,8 @@ int mca_pml_ob1_ft_event( int state )
         if( !first_continue_pass ) { 
             if( opal_cr_timing_barrier_enabled ) {
                 OPAL_CR_SET_TIMER(OPAL_CR_TIMER_COREBR0);
-                orte_grpcomm.barrier();
+                ompi_rte_barrier(coll);
+                OMPI_WAIT_FOR_COMPLETION(coll->active);
             }
             OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2P2);
         }
@@ -738,7 +793,8 @@ int mca_pml_ob1_ft_event( int state )
         if( !first_continue_pass ) {
             if( opal_cr_timing_barrier_enabled ) {
                 OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2PBR1);
-                orte_grpcomm.barrier();
+                ompi_rte_barrier(coll);
+                OMPI_WAIT_FOR_COMPLETION(coll->active);
             }
             OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2P3);
         }
@@ -765,7 +821,7 @@ int mca_pml_ob1_ft_event( int state )
             }
 
             /* Is this barrier necessary ? JJH */
-            if (OMPI_SUCCESS != (ret = orte_grpcomm.barrier())) {
+            if (OMPI_SUCCESS != (ret = ompi_rte_barrier())) {
                 opal_output(0, "pml:ob1: ft_event(Restart): Failed in orte_grpcomm.barrier (%d)", ret);
                 return ret;
             }
@@ -781,7 +837,8 @@ int mca_pml_ob1_ft_event( int state )
         if( !first_continue_pass ) {
             if( opal_cr_timing_barrier_enabled ) {
                 OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2PBR2);
-                orte_grpcomm.barrier();
+                ompi_rte_barrier(coll);
+                OMPI_WAIT_FOR_COMPLETION(coll->active);
             }
             OPAL_CR_SET_TIMER(OPAL_CR_TIMER_CRCP1);
         }
