@@ -320,12 +320,12 @@ mca_pml_ob1_send_request_schedule(mca_pml_ob1_send_request_t* sendreq)
     mca_pml_ob1_send_request_schedule_exclusive(sendreq);
 }
 
-#if OMPI_CUDA_SUPPORT
+#if OPAL_CUDA_SUPPORT
 int mca_pml_ob1_send_request_start_cuda(
     mca_pml_ob1_send_request_t* sendreq, 
     mca_bml_base_btl_t* bml_btl,
     size_t size);
-#endif /* OMPI_CUDA_SUPPORT */
+#endif /* OPAL_CUDA_SUPPORT */
 
 /**
  *  Start the specified request
@@ -366,6 +366,12 @@ mca_pml_ob1_send_request_start_btl( mca_pml_ob1_send_request_t* sendreq,
     size_t eager_limit = btl->btl_eager_limit - sizeof(mca_pml_ob1_hdr_t);
     int rc;
 
+#if OPAL_CUDA_GDR_SUPPORT
+    if (btl->btl_cuda_eager_limit && (sendreq->req_send.req_base.req_convertor.flags & CONVERTOR_CUDA)) {
+        eager_limit = btl->btl_cuda_eager_limit - sizeof(mca_pml_ob1_hdr_t);
+    }
+#endif /* OPAL_CUDA_GDR_SUPPORT */
+
     if( OPAL_LIKELY(size <= eager_limit) ) {
         switch(sendreq->req_send.req_send_mode) {
         case MCA_PML_BASE_SEND_SYNCHRONOUS:
@@ -411,11 +417,11 @@ mca_pml_ob1_send_request_start_btl( mca_pml_ob1_send_request_t* sendreq,
                                                          MCA_PML_OB1_HDR_FLAGS_CONTIG);
             }
         } else {
-#if OMPI_CUDA_SUPPORT
+#if OPAL_CUDA_SUPPORT
             if (sendreq->req_send.req_base.req_convertor.flags & CONVERTOR_CUDA) {
                 return mca_pml_ob1_send_request_start_cuda(sendreq, bml_btl, size);
             }
-#endif /* OMPI_CUDA_SUPPORT */
+#endif /* OPAL_CUDA_SUPPORT */
             rc = mca_pml_ob1_send_request_start_rndv(sendreq, bml_btl, size, 0);
         }
     }
