@@ -12,6 +12,8 @@
  * Copyright (c) 2009-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2012 Inria.  All rights reserved.
  * Copyright (c) 2011      Universite Bordeaux 1
+ * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -59,10 +61,13 @@
 #define ompi_comm_connect_f pompi_comm_connect_f
 #define ompi_comm_create_errhandler_f pompi_comm_create_errhandler_f
 #define ompi_comm_create_keyval_f pompi_comm_create_keyval_f
+#define ompi_comm_create_group_f pompi_comm_create_group_f
 #define ompi_comm_create_f pompi_comm_create_f
 #define ompi_comm_delete_attr_f pompi_comm_delete_attr_f
 #define ompi_comm_disconnect_f pompi_comm_disconnect_f
+#define ompi_comm_dup_with_info_f pompi_comm_dup_with_info_f
 #define ompi_comm_dup_f pompi_comm_dup_f
+#define ompi_comm_idup_f pompi_comm_idup_f
 #define ompi_comm_free_keyval_f pompi_comm_free_keyval_f
 #define ompi_comm_free_f pompi_comm_free_f
 #define ompi_comm_get_attr_f pompi_comm_get_attr_f
@@ -199,6 +204,11 @@
 #define ompi_igatherv_f pompi_igatherv_f
 #define ompi_improbe_f pompi_improbe_f
 #define ompi_imrecv_f pompi_imrecv_f
+#define ompi_ineighbor_allgather_f pompi_ineighbor_allgather_f
+#define ompi_ineighbor_allgatherv_f pompi_ineighbor_allgatherv_f
+#define ompi_ineighbor_alltoall_f pompi_ineighbor_alltoall_f
+#define ompi_ineighbor_alltoallv_f pompi_ineighbor_alltoallv_f
+#define ompi_ineighbor_alltoallw_f pompi_ineighbor_alltoallw_f
 #define ompi_ireduce_f pompi_ireduce_f
 #define ompi_ireduce_scatter_f pompi_ireduce_scatter_f
 #define ompi_ireduce_scatter_block_f pompi_ireduce_scatter_block_f
@@ -230,6 +240,11 @@
 #define ompi_lookup_name_f pompi_lookup_name_f
 #define ompi_mprobe_f pompi_mprobe_f
 #define ompi_mrecv_f pompi_mrecv_f
+#define ompi_neighbor_allgather_f pompi_neighbor_allgather_f
+#define ompi_neighbor_allgatherv_f pompi_neighbor_allgatherv_f
+#define ompi_neighbor_alltoall_f pompi_neighbor_alltoall_f
+#define ompi_neighbor_alltoallv_f pompi_neighbor_alltoallv_f
+#define ompi_neighbor_alltoallw_f pompi_neighbor_alltoallw_f
 #define ompi_op_commutative_f pompi_op_commutative_f
 #define ompi_op_create_f pompi_op_create_f
 #define ompi_open_port_f pompi_open_port_f
