@@ -29,9 +29,9 @@
 #include "opal/datatype/opal_convertor.h"
 #include "opal/threads/mutex.h"
 #include "opal/dss/dss.h"
-#include "opal/mca/db/db.h"
 #include "opal/util/arch.h"
 #include "opal/util/show_help.h"
+#include "opal/mca/db/db.h"
 
 #include "ompi/proc/proc.h"
 #include "ompi/datatype/ompi_datatype.h"
@@ -126,6 +126,82 @@ int ompi_proc_init(void)
     return OMPI_SUCCESS;
 }
 
+static int ompi_proc_set_locality(ompi_proc_t *proc)
+{
+    opal_hwloc_locality_t *hwlocale, locality;
+    ompi_vpid_t vpid, *vptr;
+    int ret;
+
+    /* get the locality information - do not use modex recv for
+     * this request as that will automatically cause the hostname
+     * to be loaded as well
+     */
+    hwlocale = &(proc->proc_flags);
+    if (OMPI_SUCCESS == opal_db.fetch((opal_identifier_t*)&proc->proc_name, OPAL_DB_LOCALITY,
+                                      (void**)&hwlocale, OPAL_HWLOC_LOCALITY_T)) {
+        return OMPI_SUCCESS;
+    }
+
+    /* if we don't already have it, compute and save it for future use */
+    vptr = &vpid;
+    if (OMPI_SUCCESS != (ret = opal_db.fetch((opal_identifier_t*)&proc->proc_name, OMPI_RTE_NODE_ID,
+                                             (void**)&vptr, OPAL_UINT32))) {
+        return ret;
+    }
+    /* if we are on different nodes, then we are probably non-local */
+    if (vpid != OMPI_RTE_MY_NODEID) {
+        locality = OPAL_PROC_NON_LOCAL;
+#ifdef OMPI_RTE_HOST_ID
+        /* see if coprocessors were detected - if the hostid isn't
+         * present, then no coprocessors were detected and we can
+         * ignore this test
+         */
+        vptr = &vpid;
+        if (OMPI_SUCCESS == opal_db.fetch((opal_identifier_t*)&proc->proc_name, OMPI_RTE_HOST_ID,
+                                                (void**)&vptr, OPAL_UINT32)) {
+            /* if this matches my host id, then we are on the same host,
+             * but not on the same board
+             */
+            if (vpid == ompi_process_info.my_hostid) {
+                locality = OPAL_PROC_ON_HOST;
+            } else {
+                locality = OPAL_PROC_NON_LOCAL;
+            }
+        }
+#endif
+    } else {
+#if OPAL_HAVE_HWLOC
+        {
+            char *cpu_bitmap;
+
+            /* retrieve the binding for the other proc */
+            if (OMPI_SUCCESS != opal_db.fetch((opal_identifier_t*)&proc->proc_name, OPAL_DB_CPUSET,
+                                              (void**)&cpu_bitmap, OPAL_STRING)) {
+                /* we don't know their cpuset, so nothing more we can say */
+                locality = OPAL_PROC_ON_NODE;
+            } else if (NULL == cpu_bitmap || NULL == ompi_process_info.cpuset) {
+                /* one or both of us is not bound, so all we can say is we are on the
+                 * same node
+                 */
+                locality = OPAL_PROC_ON_NODE;
+            } else {
+                /* we share a node - see what else we share */
+                locality = opal_hwloc_base_get_relative_locality(opal_hwloc_topology,
+                                                                 ompi_process_info.cpuset,
+                                                                 cpu_bitmap);
+            }
+        }
+#else
+        /* all we know is that we share this node */
+        locality = OPAL_PROC_ON_NODE;
+#endif
+    }
+    ret = opal_db.store((opal_identifier_t*)&proc, OPAL_SCOPE_INTERNAL,
+                        OPAL_DB_LOCALITY, &locality, OPAL_HWLOC_LOCALITY_T);
+    /* set the proc's local value as well */
+    proc->proc_flags = locality;
+    return ret;
+}
 
 /**
  * The process creation is split into two steps. The second step
@@ -141,7 +217,6 @@ int ompi_proc_complete_init(void)
     ompi_proc_t *proc = NULL;
     opal_list_item_t *item = NULL;
     int ret, errcode = OMPI_SUCCESS;
-    opal_hwloc_locality_t *hwlocale;
 
     OPAL_THREAD_LOCK(&ompi_proc_lock);
 
@@ -151,27 +226,31 @@ int ompi_proc_complete_init(void)
         proc = (ompi_proc_t*)item;
         
         if (proc->proc_name.vpid != OMPI_PROC_MY_NAME->vpid) {
-            /* get the locality information - do not use modex recv for
-             * this request as that will automatically cause the hostname
-             * to be loaded as well
-             */
-            hwlocale = &(proc->proc_flags);
-            ret = opal_db.fetch((opal_identifier_t*)&proc->proc_name, OMPI_DB_LOCALITY,
-                                (void**)&hwlocale, OPAL_HWLOC_LOCALITY_T);
+            /* get the locality information */
+            ret = ompi_proc_set_locality(proc);
             if (OMPI_SUCCESS != ret) {
                 errcode = ret;
                 break;
             }
 
             if (ompi_process_info.num_procs < ompi_hostname_cutoff) {
-                /* retrieve the hostname */
+                /* IF the number of procs falls below the specified cutoff,
+                 * then we assume the job is small enough that retrieving
+                 * the hostname (which will typically cause retrieval of
+                 * ALL modex info for this proc) will have no appreciable
+                 * impact on launch scaling
+                 */
                 ret = ompi_modex_recv_string_pointer(OMPI_DB_HOSTNAME, proc, (void**)&(proc->proc_hostname), OPAL_STRING);
                 if (OMPI_SUCCESS != ret) {
                     break;
                 }
             } else {
                 /* just set the hostname to NULL for now - we'll fill it in
-                 * as modex_recv's are called for procs we will talk to
+                 * as modex_recv's are called for procs we will talk to, thus
+                 * avoiding retrieval of ALL modex info for this proc until
+                 * required. Transports that delay calling modex_recv until
+                 * first message will therefore scale better than those that
+                 * call modex_recv on all procs during init.
                  */
                 proc->proc_hostname = NULL;
             }
@@ -374,7 +453,6 @@ int ompi_proc_refresh(void) {
     opal_list_item_t *item = NULL;
     ompi_vpid_t i = 0;
     int ret=OMPI_SUCCESS;
-    opal_hwloc_locality_t *hwlocale;
 
     OPAL_THREAD_LOCK(&ompi_proc_lock);
 
@@ -395,25 +473,29 @@ int ompi_proc_refresh(void) {
             proc->proc_hostname = ompi_process_info.nodename;
             proc->proc_arch = opal_local_arch;
         } else {
-            /* get the locality information - do not use modex recv for
-             * this request as that will automatically cause the hostname
-             * to be loaded as well
-             */
-            hwlocale = &(proc->proc_flags);
-            ret = opal_db.fetch((opal_identifier_t*)&proc->proc_name, OMPI_DB_LOCALITY,
-                                (void**)&hwlocale, OPAL_HWLOC_LOCALITY_T);
+            /* get the locality information */
+            ret = ompi_proc_set_locality(proc);
             if (OMPI_SUCCESS != ret) {
                 break;
             }
             if (ompi_process_info.num_procs < ompi_hostname_cutoff) {
-                /* retrieve the hostname */
+                /* IF the number of procs falls below the specified cutoff,
+                 * then we assume the job is small enough that retrieving
+                 * the hostname (which will typically cause retrieval of
+                 * ALL modex info for this proc) will have no appreciable
+                 * impact on launch scaling
+                 */
                 ret = ompi_modex_recv_string_pointer(OMPI_DB_HOSTNAME, proc, (void**)&(proc->proc_hostname), OPAL_STRING);
                 if (OMPI_SUCCESS != ret) {
                     break;
                 }
             } else {
                 /* just set the hostname to NULL for now - we'll fill it in
-                 * as modex_recv's are called for procs we will talk to
+                 * as modex_recv's are called for procs we will talk to, thus
+                 * avoiding retrieval of ALL modex info for this proc until
+                 * required. Transports that delay calling modex_recv until
+                 * first message will therefore scale better than those that
+                 * call modex_recv on all procs during init.
                  */
                 proc->proc_hostname = NULL;
             }
@@ -441,7 +523,9 @@ int ompi_proc_refresh(void) {
 }
 
 int
-ompi_proc_pack(ompi_proc_t **proclist, int proclistsize, opal_buffer_t* buf)
+ompi_proc_pack(ompi_proc_t **proclist, int proclistsize,
+               bool full_info,
+               opal_buffer_t* buf)
 {
     int i, rc;
     
@@ -466,17 +550,58 @@ ompi_proc_pack(ompi_proc_t **proclist, int proclistsize, opal_buffer_t* buf)
             OPAL_THREAD_UNLOCK(&ompi_proc_lock);
             return rc;
         }
-        rc = opal_dss.pack(buf, &(proclist[i]->proc_arch), 1, OPAL_UINT32);
-        if(rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
-            OPAL_THREAD_UNLOCK(&ompi_proc_lock);
-            return rc;
-        }
-        rc = opal_dss.pack(buf, &(proclist[i]->proc_hostname), 1, OPAL_STRING);
-        if(rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
-            OPAL_THREAD_UNLOCK(&ompi_proc_lock);
-            return rc;
+        if (full_info) {
+            int32_t num_entries;
+            opal_value_t *kv;
+            opal_list_t data;
+
+            /* fetch all global info we know about the peer - while
+             * the remote procs may already know some of it, we cannot
+             * be certain they do. So we must include a full dump of
+             * everything we know about this proc, excluding INTERNAL
+             * data that each process computes about its peers
+             */
+            OBJ_CONSTRUCT(&data, opal_list_t);
+            rc = opal_db.fetch_multiple((opal_identifier_t*)&proclist[i]->proc_name,
+                                        OPAL_SCOPE_GLOBAL, NULL, &data);
+            if (OPAL_SUCCESS != rc) {
+                OMPI_ERROR_LOG(rc);
+                num_entries = 0;
+            } else {
+                /* count the number of entries we will send */
+                num_entries = opal_list_get_size(&data);
+            }
+
+            /* put the number of entries into the buffer */
+            rc = opal_dss.pack(buf, &num_entries, 1, OPAL_INT32);
+            if (OPAL_SUCCESS != rc) {
+                OMPI_ERROR_LOG(rc);
+                break;
+            }
+    
+            /* if there are entries, store them */
+            while (NULL != (kv = (opal_value_t*)opal_list_remove_first(&data))) {
+                if (OPAL_SUCCESS != (rc = opal_dss.pack(buf, &kv, 1, OPAL_VALUE))) {
+                    OMPI_ERROR_LOG(rc);
+                    break;
+                }
+                OBJ_RELEASE(kv);
+            }
+            OBJ_DESTRUCT(&data);
+
+        } else {
+            rc = opal_dss.pack(buf, &(proclist[i]->proc_arch), 1, OPAL_UINT32);
+            if(rc != OPAL_SUCCESS) {
+                OMPI_ERROR_LOG(rc);
+                OPAL_THREAD_UNLOCK(&ompi_proc_lock);
+                return rc;
+            }
+            rc = opal_dss.pack(buf, &(proclist[i]->proc_hostname), 1, OPAL_STRING);
+            if(rc != OPAL_SUCCESS) {
+                OMPI_ERROR_LOG(rc);
+                OPAL_THREAD_UNLOCK(&ompi_proc_lock);
+                return rc;
+            }
         }
     }
     OPAL_THREAD_UNLOCK(&ompi_proc_lock);
@@ -525,14 +650,15 @@ ompi_proc_find_and_add(const ompi_process_name_t * name, bool* isnew)
 int
 ompi_proc_unpack(opal_buffer_t* buf, 
                  int proclistsize, ompi_proc_t ***proclist,
+                 bool full_info,
                  int *newproclistsize, ompi_proc_t ***newproclist)
 {
     int i;
     size_t newprocs_len = 0;
     ompi_proc_t **plist=NULL, **newprocs = NULL;
-    
+
     /* do not free plist *ever*, since it is used in the remote group
-        structure of a communicator */
+       structure of a communicator */
     plist = (ompi_proc_t **) calloc (proclistsize, sizeof (ompi_proc_t *));
     if ( NULL == plist ) {
         return OMPI_ERR_OUT_OF_RESOURCE;
@@ -543,7 +669,7 @@ ompi_proc_unpack(opal_buffer_t* buf,
         free(plist);
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
-    
+
     /* cycle through the array of provided procs and unpack
      * their info - as packed by ompi_proc_pack
      */
@@ -554,7 +680,7 @@ ompi_proc_unpack(opal_buffer_t* buf,
         char *new_hostname;
         bool isnew = false;
         int rc;
-        
+
         rc = opal_dss.unpack(buf, &new_name, &count, OMPI_NAME);
         if (rc != OPAL_SUCCESS) {
             OMPI_ERROR_LOG(rc);
@@ -562,21 +688,22 @@ ompi_proc_unpack(opal_buffer_t* buf,
             free(newprocs);
             return rc;
         }
-        rc = opal_dss.unpack(buf, &new_arch, &count, OPAL_UINT32);
-        if (rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
-            free(plist);
-            free(newprocs);
-            return rc;
+        if (!full_info) {
+            rc = opal_dss.unpack(buf, &new_arch, &count, OPAL_UINT32);
+            if (rc != OPAL_SUCCESS) {
+                OMPI_ERROR_LOG(rc);
+                free(plist);
+                free(newprocs);
+                return rc;
+            }
+            rc = opal_dss.unpack(buf, &new_hostname, &count, OPAL_STRING);
+            if (rc != OPAL_SUCCESS) {
+                OMPI_ERROR_LOG(rc);
+                free(plist);
+                free(newprocs);
+                return rc;
+            }
         }
-        rc = opal_dss.unpack(buf, &new_hostname, &count, OPAL_STRING);
-        if (rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
-            free(plist);
-            free(newprocs);
-            return rc;
-        }
-        
         /* see if this proc is already on our ompi_proc_list */
         plist[i] = ompi_proc_find_and_add(&new_name, &isnew);
         if (isnew) {
@@ -585,7 +712,65 @@ ompi_proc_unpack(opal_buffer_t* buf,
              * to us
              */
             newprocs[newprocs_len++] = plist[i];
-            
+
+            if (full_info) {
+                int32_t num_recvd_entries;
+                int32_t cnt;
+                int32_t j;
+
+                /* unpack the number of entries for this proc */
+                cnt = 1;
+                if (OPAL_SUCCESS != (rc = opal_dss.unpack(buf, &num_recvd_entries, &cnt, OPAL_INT32))) {
+                    OMPI_ERROR_LOG(rc);
+                    break;
+                }
+
+                /*
+                 * Extract the attribute names and values
+                 */
+                for (j = 0; j < num_recvd_entries; j++) {
+                    opal_value_t *kv;
+                    cnt = 1;
+                    if (OPAL_SUCCESS != (rc = opal_dss.unpack(buf, &kv, &cnt, OPAL_VALUE))) {
+                        OMPI_ERROR_LOG(rc);
+                        break;
+                    }
+                    /* if this is me, dump the data - we already have it in the db */
+                    if (OPAL_EQUAL == ompi_rte_compare_name_fields(OMPI_RTE_CMP_ALL,
+                                                                   OMPI_PROC_MY_NAME, &new_name)) {
+                        OBJ_RELEASE(kv);
+                    } else {
+                        /* store it in the database */
+                        if (OPAL_SUCCESS != (rc = opal_db.store_pointer((opal_identifier_t*)&new_name, kv))) {
+                            OMPI_ERROR_LOG(rc);
+                            OBJ_RELEASE(kv);
+                        }
+                        /* do not release the kv - the db holds that pointer */
+                    }
+                }
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
+                rc = opal_db.fetch((opal_identifier_t*)&new_name, "OMPI_ARCH",
+                                   (void**)&new_arch, OPAL_UINT32);
+                if( OPAL_SUCCESS == rc ) {
+                    new_arch = opal_local_arch;
+                }
+#else
+                new_arch = opal_local_arch;
+#endif
+                if (ompi_process_info.num_procs < ompi_hostname_cutoff) {
+                    /* retrieve the hostname */
+                    rc = opal_db.fetch_pointer((opal_identifier_t*)&new_name, OMPI_DB_HOSTNAME,
+                                               (void**)&new_hostname, OPAL_STRING);
+                    if( OPAL_SUCCESS != rc ) {
+                        new_hostname = NULL;
+                    }
+                } else {
+                    /* just set the hostname to NULL for now - we'll fill it in
+                     * as modex_recv's are called for procs we will talk to
+                     */
+                    new_hostname = NULL;
+                }
+            }
             /* update all the values */
             plist[i]->proc_arch = new_arch;
             /* if arch is different than mine, create a new convertor for this proc */
@@ -604,28 +789,45 @@ ompi_proc_unpack(opal_buffer_t* buf,
                 return OMPI_ERR_NOT_SUPPORTED;
 #endif
             }
-            if (0 == strcmp(ompi_proc_local_proc->proc_hostname,new_hostname)) {
+
+            if (0 == strcmp(ompi_proc_local_proc->proc_hostname, new_hostname)) {
                 plist[i]->proc_flags |= (OPAL_PROC_ON_NODE | OPAL_PROC_ON_CU | OPAL_PROC_ON_CLUSTER);
             }
-            
+
             /* Save the hostname */
             plist[i]->proc_hostname = new_hostname;
-            
-            /* eventually, we will update the orte/mca/ess framework's data
-             * to contain the info for the new proc. For now, we ignore
-             * this step since the MPI layer already has all the info
-             * it requires
-             */
+
+        } else {
+            if (full_info) {
+                int32_t num_recvd_entries;
+                int32_t j, cnt;
+
+                /* discard all keys: they are already locally known */
+                cnt = 1;
+                if (OPAL_SUCCESS == (rc = opal_dss.unpack(buf, &num_recvd_entries, &cnt, OPAL_INT32))) {
+                    for (j = 0; j < num_recvd_entries; j++) {
+                        opal_value_t *kv;
+                        cnt = 1;
+                        if (OPAL_SUCCESS != (rc = opal_dss.unpack(buf, &kv, &cnt, OPAL_VALUE))) {
+                            OMPI_ERROR_LOG(rc);
+                            continue;
+                        }
+                        OBJ_RELEASE(kv);
+                    }
+                } else {
+                    OMPI_ERROR_LOG(rc);
+                }
+            }
         }
     }
-    
+
     if (NULL != newproclistsize) *newproclistsize = newprocs_len;
     if (NULL != newproclist) {
         *newproclist = newprocs;
     } else if (newprocs != NULL) {
         free(newprocs);
     }
-    
+
     *proclist = plist;
     return OMPI_SUCCESS;
 }
