@@ -15,6 +15,7 @@
  * Copyright (c) 2006-2009 University of Houston. All rights reserved.
  * Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2011      Sandia National Laboratories. All rights reserved.
+ * Copyright (c) 2014      Intel, Inc. All rights reserved.
  *
  * $COPYRIGHT$
  * 
@@ -41,6 +42,8 @@
 #include "opal/mca/hwloc/base/base.h"
 #include "opal/runtime/opal_progress.h"
 #include "opal/threads/threads.h"
+#include "opal/util/arch.h"
+#include "opal/util/argv.h"
 #include "opal/util/output.h"
 #include "opal/util/error.h"
 #include "opal/util/stacktrace.h"
@@ -75,6 +78,7 @@
 #include "ompi/mca/io/io.h"
 #include "ompi/mca/io/base/base.h"
 #include "ompi/mca/rte/rte.h"
+#include "ompi/mca/rte/base/base.h"
 #include "ompi/debuggers/debuggers.h"
 #include "ompi/proc/proc.h"
 #include "ompi/mca/pml/base/pml_base_bsend.h"
@@ -445,6 +449,13 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         putenv(av);
     }
 
+    /* open the rte framework */
+    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&ompi_rte_base_framework, 0))) {
+        error = "ompi_rte_base_open() failed";
+        goto error;
+    }
+    /* no select is required as this is a static framework */
+
     /* Setup RTE - note that we are an MPI process  */
     if (OMPI_SUCCESS != (ret = ompi_rte_init(NULL, NULL))) {
         error = "ompi_mpi_init: ompi_rte_init failed";
@@ -474,10 +485,11 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
     }
 #endif
 
-    /* Register errhandler callback - RTE will ignore if it
+    /* Register the default errhandler callback - RTE will ignore if it
      * doesn't support this capability
      */
-    ompi_rte_set_fault_callback(ompi_errhandler_runtime_callback);
+    ompi_rte_register_errhandler(ompi_errhandler_runtime_callback,
+                                 OMPI_RTE_ERRHANDLER_LAST);
 
     /* Figure out the final MPI thread levels.  If we were not
        compiled for support for MPI threads, then don't allow
@@ -603,6 +615,7 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
      */
     coll = OBJ_NEW(ompi_rte_collective_t);
     coll->id = ompi_process_info.peer_modex;
+    coll->active = true;
     if (OMPI_SUCCESS != (ret = ompi_rte_modex(coll))) {
         error = "rte_modex failed";
         goto error;
@@ -611,9 +624,7 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
      * so long as it occurs prior to calling a function that needs
      * the modex info!
      */
-    while (coll->active) {
-        opal_progress();  /* block in progress pending events */
-    }
+    OMPI_WAIT_FOR_COMPLETION(coll->active);
     OBJ_RELEASE(coll);
 
     if (ompi_enable_timing && 0 == OMPI_PROC_MY_NAME->vpid) {
@@ -793,14 +804,13 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
     /* wait for everyone to reach this point */
     coll = OBJ_NEW(ompi_rte_collective_t);
     coll->id = ompi_process_info.peer_init_barrier;
+    coll->active = true;
     if (OMPI_SUCCESS != (ret = ompi_rte_barrier(coll))) {
         error = "rte_barrier failed";
         goto error;
     }
     /* wait for barrier to complete */
-    while (coll->active) {
-        opal_progress();  /* block in progress pending events */
-    }
+    OMPI_WAIT_FOR_COMPLETION(coll->active);
     OBJ_RELEASE(coll);
 
     /* check for timing request - get stop time and report elapsed
