@@ -65,7 +65,7 @@ int ompi_mpi_leave_pinned = -1;
 bool ompi_mpi_leave_pinned_pipeline = false;
 bool ompi_have_sparse_group_storage = OPAL_INT_TO_BOOL(OMPI_GROUP_SPARSE);
 bool ompi_use_sparse_group_storage = OPAL_INT_TO_BOOL(OMPI_GROUP_SPARSE);
-bool ompi_mpi_built_with_cuda_support = OPAL_INT_TO_BOOL(OMPI_CUDA_SUPPORT);
+bool ompi_mpi_built_with_cuda_support = OPAL_INT_TO_BOOL(OPAL_CUDA_SUPPORT);
 bool ompi_mpi_cuda_support;
 
 uint32_t ompi_hostname_cutoff = UINT32_MAX;
@@ -328,7 +328,7 @@ int ompi_mpi_register_params(void)
                                  "Whether CUDA GPU buffer support is enabled or not",
                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
                                  OPAL_INFO_LVL_4,
-                                 MCA_BASE_VAR_SCOPE_LOCAL,
+                                 MCA_BASE_VAR_SCOPE_READONLY,
                                  &ompi_mpi_cuda_support);
     if (ompi_mpi_cuda_support && !ompi_mpi_built_with_cuda_support) {
         opal_show_help("help-mpi-runtime.txt", "no cuda support",
@@ -397,7 +397,8 @@ int ompi_show_all_mca_params(int32_t rank, int requested, char *nodename) {
          * ignore it
          */
         if ((MCA_BASE_VAR_SOURCE_FILE == var->mbv_source ||
-             MCA_BASE_VAR_SOURCE_OVERRIDE) && !show_file_mca_params) {
+             MCA_BASE_VAR_SOURCE_OVERRIDE == var->mbv_source) && 
+            !show_file_mca_params) {
             continue;
         }
         
