@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
  * $COPYRIGHT$
  * 
@@ -40,10 +40,6 @@
 #include "opal/memoryhooks/memory.h"
 #include "opal/runtime/opal_info_support.h"
 
-#if OMPI_RTE_ORTE
-#include "orte/util/show_help.h"
-#endif
-
 #include "ompi/tools/ompi_info/ompi_info.h"
 #include "ompi/include/mpi_portable_platform.h"
 
@@ -92,7 +88,12 @@ void ompi_info_do_config(bool want_all)
     char *fortran_build_f08_subarrays;
     char *fortran_have_optional_args;
     char *fortran_have_bind_c;
+    char *fortran_have_iso_c_binding;
+    char *fortran_have_bind_c_sub;
+    char *fortran_have_bind_c_type;
+    char *fortran_have_bind_c_type_name;
     char *fortran_have_private;
+    char *fortran_have_protected;
     char *fortran_have_abstract;
     char *fortran_have_asynchronous;
     char *fortran_have_procedure;
@@ -165,7 +166,14 @@ void ompi_info_do_config(bool want_all)
     fortran_have_optional_args = OMPI_FORTRAN_HAVE_OPTIONAL_ARGS ?
         "yes" : "no";
     fortran_have_bind_c = OMPI_FORTRAN_HAVE_BIND_C ? "yes" : "no";
+    fortran_have_iso_c_binding = OMPI_FORTRAN_HAVE_ISO_C_BINDING ?
+        "yes" : "no";
+    fortran_have_bind_c_sub = OMPI_FORTRAN_HAVE_BIND_C_SUB ? "yes" : "no";
+    fortran_have_bind_c_type = OMPI_FORTRAN_HAVE_BIND_C_TYPE ? "yes" : "no";
+    fortran_have_bind_c_type_name = OMPI_FORTRAN_HAVE_BIND_C_TYPE_NAME ? 
+        "yes" : "no";
     fortran_have_private = OMPI_FORTRAN_HAVE_PRIVATE ? "yes" : "no";
+    fortran_have_protected = OMPI_FORTRAN_HAVE_PROTECTED ? "yes" : "no";
     fortran_have_abstract = OMPI_FORTRAN_HAVE_ABSTRACT ? "yes" : "no";
     fortran_have_asynchronous = OMPI_FORTRAN_HAVE_ASYNCHRONOUS ? "yes" : "no";
     fortran_have_procedure = OMPI_FORTRAN_HAVE_PROCEDURE ? "yes" : "no";
@@ -176,7 +184,9 @@ void ompi_info_do_config(bool want_all)
        module has */
     if (OMPI_BUILD_FORTRAN_USEMPIF08_BINDINGS) {
 
-        /* Do we have everything? */
+        /* Do we have everything? (not including PROTECTED, which
+           isn't *needed* for the mpi_f08 module compliance -- it's
+           just *nice to have*) */
         if (OMPI_BUILD_FORTRAN_F08_SUBARRAYS &&
             OMPI_FORTRAN_HAVE_PRIVATE &&
             OMPI_FORTRAN_HAVE_ABSTRACT &&
@@ -258,18 +268,15 @@ void ompi_info_do_config(bool want_all)
         fortran_have_ignore_tkr = strdup("no");
     }
 
-    if (OPAL_HAVE_SOLARIS_THREADS || OPAL_HAVE_POSIX_THREADS) {        /* should just test OPAL_HAVE_THREADS */
+    if (OPAL_HAVE_POSIX_THREADS) {        /* should just test OPAL_HAVE_THREADS */
 #if OMPI_RTE_ORTE
-        (void)asprintf(&threads, "%s (MPI_THREAD_MULTIPLE: %s, OPAL support: %s, OMPI progress: %s, ORTE progress: %s, Event lib: yes)",
-                       OPAL_HAVE_SOLARIS_THREADS ? "solaris" :
+        (void)asprintf(&threads, "%s (MPI_THREAD_MULTIPLE: %s, OPAL support: %s, OMPI progress: %s, ORTE progress: yes, Event lib: yes)",
                        (OPAL_HAVE_POSIX_THREADS ? "posix" : "type unknown"), /* "type unknown" can presumably never happen */
                        OMPI_ENABLE_THREAD_MULTIPLE ? "yes" : "no",
                        OPAL_ENABLE_MULTI_THREADS ? "yes" : "no",
-                       OMPI_ENABLE_PROGRESS_THREADS ? "yes" : "no",
-                       ORTE_ENABLE_PROGRESS_THREADS ? "yes" : "no");
+                       OMPI_ENABLE_PROGRESS_THREADS ? "yes" : "no");
 #else
         (void)asprintf(&threads, "%s (MPI_THREAD_MULTIPLE: %s, OPAL support: %s, OMPI progress: %s, Event lib: yes)",
-                       OPAL_HAVE_SOLARIS_THREADS ? "solaris" :
                        (OPAL_HAVE_POSIX_THREADS ? "posix" : "type unknown"), /* "type unknown" can presumably never happen */
                        OMPI_ENABLE_THREAD_MULTIPLE ? "yes" : "no",
                        OPAL_ENABLE_MULTI_THREADS ? "yes" : "no",
@@ -312,6 +319,8 @@ void ompi_info_do_config(bool want_all)
     opal_info_out("Fort mpi_f08 subarrays", "bindings:use_mpi_f08:subarrays-supported", 
                   fortran_build_f08_subarrays);
 
+    opal_info_out("Wrapper compiler rpath", "compiler:all:rpath", 
+                  WRAPPER_RPATH_SUPPORT);
     opal_info_out("C compiler", "compiler:c:command", OPAL_CC);
     opal_info_out("C compiler absolute", "compiler:c:absolute", 
                   OPAL_CC_ABSOLUTE);
@@ -364,12 +373,27 @@ void ompi_info_do_config(bool want_all)
     opal_info_out("Fort optional args", 
                   "compiler:fortran:optional_arguments",
                   fortran_have_optional_args);
-    opal_info_out("Fort BIND(C)", 
+    opal_info_out("Fort BIND(C) (all)", 
                   "compiler:fortran:bind_c",
                   fortran_have_bind_c);
+    opal_info_out("Fort ISO_C_BINDING",
+                  "compiler:fortran:iso_c_binding",
+                  fortran_have_iso_c_binding);
+    opal_info_out("Fort SUBROUTINE BIND(C)", 
+                  "compiler:fortran:subroutine_bind_c",
+                  fortran_have_bind_c_sub);
+    opal_info_out("Fort TYPE,BIND(C)", 
+                  "compiler:fortran:type_bind_c",
+                  fortran_have_bind_c_type);
+    opal_info_out("Fort T,BIND(C,name=\"a\")", 
+                  "compiler:fortran:type_name_bind_c",
+                  fortran_have_bind_c_type_name);
     opal_info_out("Fort PRIVATE", 
                   "compiler:fortran:private",
                   fortran_have_private);
+    opal_info_out("Fort PROTECTED", 
+                  "compiler:fortran:protected",
+                  fortran_have_protected);
     opal_info_out("Fort ABSTRACT", 
                   "compiler:fortran:abstract",
                   fortran_have_abstract);
