@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /* 
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -12,6 +12,8 @@
  *                         All rights reserved.
  * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -117,7 +119,7 @@ int ompi_win_create(void *base, size_t size, int disp_unit,
 
 int ompi_win_free(ompi_win_t *win);
 
-OMPI_DECLSPEC int ompi_win_set_name(ompi_win_t *win, char *win_name);
+OMPI_DECLSPEC int ompi_win_set_name(ompi_win_t *win, const char *win_name);
 OMPI_DECLSPEC int ompi_win_get_name(ompi_win_t *win, char *win_name, int *length);
 
 OMPI_DECLSPEC int ompi_win_group(ompi_win_t *win, ompi_group_t **group);
