@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2009-2011 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009-2013 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -67,7 +67,7 @@
  **********************************************************************/
 
 /* Do we have posix or solaris thread lib */
-#define OPAL_HAVE_THREADS (OPAL_HAVE_POSIX_THREADS || OPAL_HAVE_SOLARIS_THREADS)
+#define OPAL_HAVE_THREADS (OPAL_HAVE_POSIX_THREADS)
 
 /*
  * BEGIN_C_DECLS should be used at the beginning of your declarations,
@@ -157,6 +157,12 @@
 #    define __opal_attribute_no_instrument_function__
 #endif
 
+#if OPAL_HAVE_ATTRIBUTE_NOINLINE
+#    define __opal_attribute_noinline__  __attribute__((__noinline__))
+#else
+#    define __opal_attribute_noinline__
+#endif
+
 #if OPAL_HAVE_ATTRIBUTE_NONNULL
 #    define __opal_attribute_nonnull__(a)    __attribute__((__nonnull__(a)))
 #    define __opal_attribute_nonnull_all__   __attribute__((__nonnull__))
@@ -534,9 +540,9 @@ static inline uint16_t ntohs(uint16_t netvar) { return netvar; }
  * this makes it simpler to check throughout the code base.
  */
 #if OPAL_ENABLE_IPV6 && defined(HAVE_STRUCT_SOCKADDR_IN6)
-#define OPAL_WANT_IPV6 1
+#define OPAL_ENABLE_IPV6 1
 #else
-#define OPAL_WANT_IPV6 0
+#define OPAL_ENABLE_IPV6 0
 #endif
 
 #if !defined(HAVE_STRUCT_SOCKADDR_STORAGE) && defined(HAVE_STRUCT_SOCKADDR_IN)
