@@ -11,7 +11,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2013 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2012-2013 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2012-2014 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
  * 
@@ -80,17 +80,6 @@ const char *var_type_names[] = {
     "double"
 };
 
-const char *var_type_formats[] = {
-    "%d",
-    "%u",
-    "%lu",
-    "%llu",
-    "%" PRIsize_t,
-    "%s",
-    "%d",
-    "%lf"
-};
-
 const size_t var_type_sizes[] = {
     sizeof (int),
     sizeof (unsigned),
@@ -131,7 +120,7 @@ static int fixup_files(char **file_list, char * path, bool rel_path_search);
 static int read_files (char *file_list, opal_list_t *file_values);
 static int mca_base_var_cache_files (bool rel_path_search);
 static int var_set_initial (mca_base_var_t *var);
-static int var_get (int index, mca_base_var_t **var_out, bool original);
+static int var_get (int vari, mca_base_var_t **var_out, bool original);
 static int var_value_string (mca_base_var_t *var, char **value_string);
 
 /*
@@ -420,7 +409,7 @@ static int mca_base_var_cache_files(bool rel_path_search)
 /*
  * Look up an integer MCA parameter.
  */
-int mca_base_var_get_value (int index, const void *value,
+int mca_base_var_get_value (int vari, const void *value,
                             mca_base_var_source_t *source,
                             const char **source_file)
 {
@@ -428,7 +417,7 @@ int mca_base_var_get_value (int index, const void *value,
     void **tmp = (void **) value;
     int ret;
 
-    ret = var_get (index, &var, true);
+    ret = var_get (vari, &var, true);
     if (OPAL_SUCCESS != ret) {
         return ret;
     }
@@ -577,8 +566,8 @@ static int var_set_from_string (mca_base_var_t *var, char *src)
     case MCA_BASE_VAR_TYPE_SIZE_T:
         ret = int_from_string(src, var->mbv_enumerator, &int_value);
         if (OPAL_ERR_VALUE_OUT_OF_BOUNDS == ret ||
-            (MCA_BASE_VAR_TYPE_INT == var->mbv_type &&
-             (sizeof(int) < 8 && (int_value & 0xffffffff00000000ull)))) {
+            (MCA_BASE_VAR_TYPE_INT == var->mbv_type && ((int) int_value != (int64_t) int_value)) ||
+            (MCA_BASE_VAR_TYPE_UNSIGNED_INT == var->mbv_type && ((unsigned int) int_value != int_value))) {
             if (var->mbv_enumerator) {
                 char *valid_values;
                 (void) var->mbv_enumerator->dump(var->mbv_enumerator, &valid_values);
@@ -623,13 +612,13 @@ static int var_set_from_string (mca_base_var_t *var, char *src)
 /*
  * Set a variable
  */
-int mca_base_var_set_value (int index, const void *value, size_t size, mca_base_var_source_t source,
+int mca_base_var_set_value (int vari, const void *value, size_t size, mca_base_var_source_t source,
                             const char *source_file)
 {
     mca_base_var_t *var;
     int ret;
 
-    ret = var_get (index, &var, true);
+    ret = var_get (vari, &var, true);
     if (OPAL_SUCCESS != ret) {
         return ret;
     }
@@ -669,12 +658,12 @@ int mca_base_var_set_value (int index, const void *value, size_t size, mca_base_
 /*
  * Deregister a parameter
  */
-int mca_base_var_deregister(int index)
+int mca_base_var_deregister(int vari)
 {
     mca_base_var_t *var;
     int ret;
 
-    ret = var_get (index, &var, false);
+    ret = var_get (vari, &var, false);
     if (OPAL_SUCCESS != ret) {
         return ret;
     }
@@ -708,7 +697,7 @@ int mca_base_var_deregister(int index)
     return OPAL_SUCCESS;
 }
 
-static int var_get (int index, mca_base_var_t **var_out, bool original)
+static int var_get (int vari, mca_base_var_t **var_out, bool original)
 {
     mca_base_var_t *var;
 
@@ -721,11 +710,11 @@ static int var_get (int index, mca_base_var_t **var_out, bool original)
         return OPAL_ERROR;
     }
 
-    if (index < 0) {
+    if (vari < 0) {
         return OPAL_ERR_BAD_PARAM;
     }
 
-    var = opal_pointer_array_get_item (&mca_base_vars, index);
+    var = opal_pointer_array_get_item (&mca_base_vars, vari);
     if (NULL == var) {
         return OPAL_ERR_BAD_PARAM;
     }
@@ -759,7 +748,7 @@ int mca_base_var_env_name(const char *param_name,
 /*
  * Find the index for an MCA parameter based on its names.
  */
-static int var_find_by_name (const char *full_name, int *index, bool invalidok)
+static int var_find_by_name (const char *full_name, int *vari, bool invalidok)
 {
     mca_base_var_t *var;
     void *tmp;
@@ -774,7 +763,7 @@ static int var_find_by_name (const char *full_name, int *index, bool invalidok)
     (void) var_get ((int)(uintptr_t) tmp, &var, false);
 
     if (invalidok || VAR_IS_VALID(var[0])) {
-        *index = (int)(uintptr_t) tmp;
+        *vari = (int)(uintptr_t) tmp;
         return OPAL_SUCCESS;
     }
 
@@ -786,7 +775,7 @@ static int var_find (const char *project_name, const char *framework_name,
                      bool invalidok)
 {
     char *full_name;
-    int ret, index;
+    int ret, vari;
 
     ret = mca_base_var_generate_full_name4 (NULL, framework_name, component_name,
                                             variable_name, &full_name);
@@ -794,12 +783,17 @@ static int var_find (const char *project_name, const char *framework_name,
         return OPAL_ERROR;
     }
 
-    ret = var_find_by_name(full_name, &index, invalidok);
+    ret = var_find_by_name(full_name, &vari, invalidok);
 
     /* NTH: should we verify the name components match? */
 
     free (full_name);
-    return (OPAL_SUCCESS != ret) ? ret : index;
+
+    if (OPAL_SUCCESS != ret) {
+        return ret;
+    }
+
+    return vari;
 }
 
 /*
@@ -814,17 +808,17 @@ int mca_base_var_find (const char *project_name, const char *framework_name,
 /*
  * Find the index for an MCA parameter based on full name.
  */
-int mca_base_var_find_by_name (const char *full_name, int *index)
+int mca_base_var_find_by_name (const char *full_name, int *vari)
 {
-    return var_find_by_name (full_name, index, false);
+    return var_find_by_name (full_name, vari, false);
 }
 
-int mca_base_var_set_flag (int index, mca_base_var_flag_t flag, bool set)
+int mca_base_var_set_flag (int vari, mca_base_var_flag_t flag, bool set)
 {
     mca_base_var_t *var;
     int ret;
 
-    ret = var_get (index, &var, true);
+    ret = var_get (vari, &var, true);
     if (OPAL_SUCCESS != ret || VAR_IS_SYNONYM(var[0])) {
         return OPAL_ERR_BAD_PARAM;
     }
@@ -838,9 +832,9 @@ int mca_base_var_set_flag (int index, mca_base_var_flag_t flag, bool set)
 /*
  * Return info on a parameter at an index
  */
-int mca_base_var_get (int index, const mca_base_var_t **var)
+int mca_base_var_get (int vari, const mca_base_var_t **var)
 {
-    return var_get (index, (mca_base_var_t **) var, false);
+    return var_get (vari, (mca_base_var_t **) var, false);
 }
 
 /*
@@ -1274,13 +1268,14 @@ static int register_variable (const char *project_name, const char *framework_na
         }
     }
 
+    /* go ahead and mark this variable as valid */
+    var->mbv_flags |= MCA_BASE_VAR_FLAG_VALID;
+
     ret = var_set_initial (var);
     if (OPAL_SUCCESS != ret) {
         return ret;
     }
 
-    var->mbv_flags |= MCA_BASE_VAR_FLAG_VALID;
-
     /* All done */
     return var_index;
 }
@@ -1335,7 +1330,7 @@ int mca_base_var_register_synonym (int synonym_for, const char *project_name,
                                    const char *synonym_name,
                                    mca_base_var_syn_flag_t flags)
 {
-    mca_base_var_flag_t var_flags = MCA_BASE_VAR_FLAG_SYNONYM;
+    mca_base_var_flag_t var_flags = (mca_base_var_flag_t) MCA_BASE_VAR_FLAG_SYNONYM;
     mca_base_var_t *var;
     int ret;
 
@@ -1364,11 +1359,12 @@ static int var_set_from_env (mca_base_var_t *var)
 {
     const char *var_full_name = var->mbv_full_name;
     bool deprecated = VAR_IS_DEPRECATED(var[0]);
+    bool is_synonym = VAR_IS_SYNONYM(var[0]);
     char *source, *source_env;
     char *value, *value_env;
     int ret;
     
-    if (VAR_IS_SYNONYM(var[0])) {
+    if (is_synonym) {
         ret = var_get (var->mbv_synonym_for, &var, true);
         if (OPAL_SUCCESS != ret) {
             return OPAL_ERROR;
@@ -1434,19 +1430,26 @@ static int var_set_from_env (mca_base_var_t *var)
     }
 
     if (deprecated) {
+        const char *new_variable = "None (going away)";
+
+        if (is_synonym) {
+            new_variable = var->mbv_full_name;
+        }
+
         switch (var->mbv_source) {
         case MCA_BASE_VAR_SOURCE_ENV:
             opal_show_help("help-mca-var.txt", "deprecated-mca-env",
-                           true, var->mbv_full_name);
+                           true, var_full_name, new_variable);
             break;
         case MCA_BASE_VAR_SOURCE_COMMAND_LINE:
             opal_show_help("help-mca-var.txt", "deprecated-mca-cli",
-                           true, var->mbv_full_name);
+                           true, var_full_name, new_variable);
             break;
         case MCA_BASE_VAR_SOURCE_FILE:
         case MCA_BASE_VAR_SOURCE_OVERRIDE:
             opal_show_help("help-mca-var.txt", "deprecated-mca-file",
-                           true, var->mbv_full_name, var->mbv_source_file);
+                           true, var_full_name, var->mbv_source_file,
+                           new_variable);
             break;
 
         case MCA_BASE_VAR_SOURCE_DEFAULT:
@@ -1674,15 +1677,45 @@ static int var_value_string (mca_base_var_t *var, char **value_string)
     }
 
     if (NULL == var->mbv_enumerator) {
-        if (MCA_BASE_VAR_TYPE_STRING == var->mbv_type) {
-            ret = asprintf (value_string, "%s", value->stringval ? value->stringval : "");
-        } else {
-            ret = asprintf (value_string, var_type_formats[var->mbv_type], value[0]);
+        switch (var->mbv_type) {
+        case MCA_BASE_VAR_TYPE_INT:
+            ret = asprintf (value_string, "%d", value->intval);
+            break;
+        case MCA_BASE_VAR_TYPE_UNSIGNED_INT:
+            ret = asprintf (value_string, "%u", value->uintval);
+            break;
+        case MCA_BASE_VAR_TYPE_UNSIGNED_LONG:
+            ret = asprintf (value_string, "%lu", value->ulval);
+            break;
+        case MCA_BASE_VAR_TYPE_UNSIGNED_LONG_LONG:
+            ret = asprintf (value_string, "%llu", value->ullval);
+            break;
+        case MCA_BASE_VAR_TYPE_SIZE_T:
+            ret = asprintf (value_string, "%" PRIsize_t, value->sizetval);
+            break;
+        case MCA_BASE_VAR_TYPE_STRING:
+            ret = asprintf (value_string, "%s", 
+                            value->stringval ? value->stringval : "");
+            break;
+        case MCA_BASE_VAR_TYPE_BOOL:
+            ret = asprintf (value_string, "%d", value->boolval);
+            break;
+        case MCA_BASE_VAR_TYPE_DOUBLE:
+            ret = asprintf (value_string, "%lf", value->lfval);
+            break;
+        default:
+            ret = -1;
+            break;
         }
 
         ret = (0 > ret) ? OPAL_ERR_OUT_OF_RESOURCE : OPAL_SUCCESS;
     } else {
-        ret = var->mbv_enumerator->string_from_value(var->mbv_enumerator, value->intval, &tmp);
+        /* we use an enumerator to handle string->bool and bool->string conversion */
+        if (MCA_BASE_VAR_TYPE_BOOL == var->mbv_type) {
+            ret = var->mbv_enumerator->string_from_value(var->mbv_enumerator, value->boolval, &tmp);
+        } else {
+            ret = var->mbv_enumerator->string_from_value(var->mbv_enumerator, value->intval, &tmp);
+        }
 
         *value_string = strdup (tmp);
         if (NULL == value_string) {
@@ -1752,7 +1785,7 @@ int mca_base_var_get_count (void)
     return mca_base_var_count;
 }
 
-int mca_base_var_dump(int index, char ***out, mca_base_var_dump_type_t output_type)
+int mca_base_var_dump(int vari, char ***out, mca_base_var_dump_type_t output_type)
 {
     const char *framework, *component, *full_name;
     int i, line_count, line = 0, enum_count = 0;
@@ -1761,7 +1794,7 @@ int mca_base_var_dump(int index, char ***out, mca_base_var_dump_type_t output_ty
     mca_base_var_t *var, *original;
     mca_base_var_group_t *group;
 
-    ret = var_get(index, &var, false);
+    ret = var_get(vari, &var, false);
     if (OPAL_SUCCESS != ret) {
         return ret;
     }
