@@ -9,8 +9,9 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2009-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009-2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2010      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2014      Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -19,6 +20,8 @@
  */
 
 #include "opal_config.h"
+#include "opal/constants.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -28,6 +31,9 @@
 #ifdef HAVE_SHLWAPI_H
 #include <shlwapi.h>
 #endif
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
 #ifdef HAVE_SYS_MOUNT_H
 #include <sys/mount.h>
 #endif
@@ -43,6 +49,9 @@
 #ifdef HAVE_SYS_STATFS_H
 #include <sys/statfs.h>
 #endif
+#ifdef HAVE_SYS_STATVFS_H
+#include <sys/statvfs.h>
+#endif
 #ifdef HAVE_SYS_MOUNT_H
 #include <sys/mount.h>
 #endif
@@ -52,6 +61,23 @@
 #include "opal/util/os_path.h"
 #include "opal/util/argv.h"
 
+/*
+ * Sanity check to ensure we have either statfs or statvfs
+ */
+#if !defined(HAVE_STATFS) && !defined(HAVE_STATVFS)
+#error Must have either statfs() or statvfs()
+#endif
+
+/*
+ * Note that some OS's (e.g., NetBSD and Solaris) have statfs(), but
+ * no struct statfs (!).  So check to make sure we have struct statfs
+ * before allowing the use of statfs().
+ */
+#if defined(HAVE_STATFS) && (defined(HAVE_STRUCT_STATFS_F_FSTYPENAME) || \
+                             defined(HAVE_STRUCT_STATFS_F_TYPE))
+#define USE_STATFS 1
+#endif
+
 static void path_env_load(char *path, int *pargc, char ***pargv);
 static char *list_env_get(char *var, char **list);
 
@@ -82,7 +108,7 @@ char *opal_path_find(char *fname, char **pathv, int mode, char **envv)
 
     /* If absolute path is given, return it without searching. */
     if( opal_path_is_absolute(fname) ) {
-        return opal_path_access(fname, "", mode);
+        return opal_path_access(fname, NULL, mode);
     }
 
     /* Initialize. */
@@ -409,7 +435,11 @@ char* opal_find_absolute_path( char* app_name )
  *          with f_fstypename, contains a string of length MFSNAMELEN
  *          return 0 success, -1 on failure with errno set.
  *          compliant with: 4.4BSD.
- * Mac OSX (10.6.2):
+ * NetBSD:
+ *   statvfs (const char *path, struct statvfs *buf);
+ *          with f_fstypename, contains a string of length VFS_NAMELEN
+ *          return 0 success, -1 on failure with errno set.
+ * Mac OSX (10.6.2 through 10.9):
  *   statvfs(const char * restrict path, struct statvfs * restrict buf);
  *          with fsid    Not meaningful in this implementation.
  *          is just a wrapper around statfs()
@@ -441,13 +471,14 @@ bool opal_path_nfs(char *fname)
 {
 #if !defined(__WINDOWS__)
     int i;
-    int rc;
+    int fsrc, vfsrc;
     int trials;
     char * file = strdup (fname);
-#if defined(__SVR4) && defined(__sun)
-    struct statvfs buf;
-#elif defined(__linux__) || defined (__BSD) || (defined(__APPLE__) && defined(__MACH__))
-    struct statfs buf;
+#if defined(USE_STATFS)
+    struct statfs fsbuf;
+#endif
+#if defined(HAVE_STATVFS)
+    struct statvfs vfsbuf;
 #endif
     /*
      * Be sure to update the test (test/util/opal_path_nfs.c) 
@@ -461,26 +492,31 @@ bool opal_path_nfs(char *fname)
         {LL_SUPER_MAGIC,                   MASK4, "lustre"},
         {NFS_SUPER_MAGIC,                  MASK2, "nfs"},
         {PAN_KERNEL_FS_CLIENT_SUPER_MAGIC, MASK4, "panfs"},
-        {GPFS_SUPER_MAGIC, MASK4, "gpfs"}
+        {GPFS_SUPER_MAGIC,                 MASK4, "gpfs"}
     };
 #define FS_TYPES_NUM (int)(sizeof (fs_types)/sizeof (fs_types[0]))
 
     /*
-     * First, get the OS-dependent struct stat(v)fs buf
-     * This may return the ESTALE error on NFS, if the underlying file/path has changed
+     * First, get the OS-dependent struct stat(v)fs buf.  This may
+     * return the ESTALE error on NFS, if the underlying file/path has
+     * changed.
      */
 again:
+#if defined(USE_STATFS)
     trials = 5;
     do {
-#if defined(__SVR4) && defined(__sun)
-        rc = statvfs (file, &buf);
-#elif defined(__linux__) || defined (__BSD) || (defined(__APPLE__) && defined(__MACH__))
-        rc = statfs (file, &buf);
+        fsrc = statfs(file, &fsbuf);
+    } while (-1 == fsrc && ESTALE == errno && (0 < --trials));
+#endif
+#if defined(HAVE_STATVFS)
+    trials = 5;
+    do {
+        vfsrc = statvfs(file, &vfsbuf);
+    } while (-1 == vfsrc && ESTALE == errno && (0 < --trials));
 #endif
-    } while (-1 == rc && ESTALE == errno && (0 < --trials));
 
     /* In case some error with the current filename, try the directory */
-    if (-1 == rc) {
+    if (-1 == fsrc && -1 == vfsrc) {
         char * last_sep;
 
         OPAL_OUTPUT_VERBOSE((10, 0, "opal_path_nfs: stat(v)fs on file:%s failed errno:%d directory:%s\n",
@@ -499,24 +535,43 @@ again:
     }
 
     /* Next, extract the magic value */
-#if defined(__SVR4) && defined(__sun)
-    for (i = 0; i < FS_TYPES_NUM; i++) 
-        if (0 == strncasecmp (fs_types[i].f_fsname, buf.f_basetype, FSTYPSZ))
+    for (i = 0; i < FS_TYPES_NUM; i++) {
+#if defined(USE_STATFS)
+        /* These are uses of struct statfs */
+#    if defined(HAVE_STRUCT_STATFS_F_FSTYPENAME)
+        if (0 == fsrc &&
+            0 == strncasecmp(fs_types[i].f_fsname, fsbuf.f_fstypename, 
+                             sizeof(fsbuf.f_fstypename))) {
             goto found;
-#elif (defined(__APPLE__) && defined(__MACH__))
-    for (i = 0; i < FS_TYPES_NUM; i++)
-        if (0 == strncasecmp (fs_types[i].f_fsname, buf.f_fstypename, MFSTYPENAMELEN))
-            goto found;
-#elif defined(__BSD)
-    for (i = 0; i < FS_TYPES_NUM; i++)
-        if (0 == strncasecmp (fs_types[i].f_fsname, buf.f_fstypename, MFSNAMELEN))
-            goto found;
-#elif defined(__linux__)
-    for (i = 0; i < FS_TYPES_NUM; i++)
-        if (fs_types[i].f_fsid == (buf.f_type & fs_types[i].f_mask))
+        }
+#    endif
+#    if defined(HAVE_STRUCT_STATFS_F_TYPE)
+        if (0 == fsrc &&
+            fs_types[i].f_fsid == (fsbuf.f_type & fs_types[i].f_mask)) {
             goto found;
+        }
+#    endif
 #endif
 
+#if defined(HAVE_STATVFS)
+        /* These are uses of struct statvfs */
+#    if defined(HAVE_STRUCT_STATVFS_F_BASETYPE)
+        if (0 == vfsrc &&
+            0 == strncasecmp(fs_types[i].f_fsname, vfsbuf.f_basetype,
+                             sizeof(vfsbuf.f_basetype))) {
+            goto found;
+        }
+#    endif
+#    if defined(HAVE_STRUCT_STATVFS_F_FSTYPENAME)
+        if (0 == vfsrc &&
+            0 == strncasecmp(fs_types[i].f_fsname, vfsbuf.f_fstypename,
+                             sizeof(vfsbuf.f_fstypename))) {
+            goto found;
+        }
+#    endif
+#endif
+    }
+
     free (file);
     return false;
 
