@@ -11,11 +11,11 @@
  *                         All rights reserved.
  * Copyright (c) 2007-2010 Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2007      Evergrid, Inc. All rights reserved.
- * Copyright (c) 2008-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2013 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2010      IBM Corporation.  All rights reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
- * Copyright (c) 2013      Intel, Inc.  All rights reserved.
+ * Copyright (c) 2013-2014 Intel, Inc. All rights reserved
  *
  * $COPYRIGHT$
  *
@@ -113,6 +113,7 @@
 #include "opal/class/opal_pointer_array.h"
 #include "opal/util/opal_environ.h"
 #include "opal/util/show_help.h"
+#include "opal/util/sys_limits.h"
 #include "opal/util/fd.h"
 
 #include "orte/util/show_help.h"
@@ -375,7 +376,7 @@ static void send_error_show_help(int fd, int exit_status,
 
     /* Send it */
     va_start(ap, topic);
-    (void)write_help_msg(fd, &msg, file, topic, ap);
+    write_help_msg(fd, &msg, file, topic, ap);
     va_end(ap);
 
     exit(exit_status);
@@ -387,13 +388,10 @@ static int do_child(orte_app_context_t* context,
                     orte_job_t *jobdat, int write_fd,
                     orte_iof_base_io_conf_t opts)
 {
-    int i;
+    int i, rc;
     sigset_t sigs;
     long fd, fdmax = sysconf(_SC_OPEN_MAX);
-#if OPAL_HAVE_HWLOC
-    int rc;
     char *param, *msg;
-#endif
 
     if (orte_forward_job_control) {
         /* Set a new process group for this child, so that a
@@ -435,6 +433,23 @@ static int do_child(orte_app_context_t* context,
 
             /* Set process affinity, if given */
             if (NULL != child->cpu_bitmap) {
+                if (0 == strlen(child->cpu_bitmap)) {
+                    /* this proc is not bound */
+                    if (opal_hwloc_report_bindings) {
+                        opal_output(0, "MCW rank %d is not bound (or bound to all available processors)", child->name.vpid);
+                        /* avoid reporting it twice */
+                        (void) mca_base_var_env_name ("hwloc_base_report_bindings", &param);
+                        opal_unsetenv(param, &environ_copy);
+                        free(param);
+                    }
+                    /* Set an info MCA param that tells
+                       the launched processes that it was bound by us (e.g., so that
+                       MPI_INIT doesn't try to bind itself) */
+                    (void) mca_base_var_env_name ("orte_bound_at_launch", &param);
+                    opal_setenv(param, "1", true, &environ_copy);
+                    free(param);
+                    goto PROCEED;
+                }
                 /* convert the list to a cpu bitmap */
                 cpuset = hwloc_bitmap_alloc();
                 if (0 != (rc = hwloc_bitmap_list_sscanf(cpuset, child->cpu_bitmap))) {
@@ -444,9 +459,12 @@ static int do_child(orte_app_context_t* context,
                     if (NULL == msg) {
                         msg = "failed to convert bitmap list to hwloc bitmap";
                     }
-                    if (OPAL_BINDING_REQUIRED(jobdat->map->binding)) {
-                        /* If binding is required, send an error up the pipe (which exits
-                           -- it doesn't return). */
+                    if (OPAL_BINDING_REQUIRED(jobdat->map->binding) &&
+                        (OPAL_BIND_GIVEN & jobdat->map->binding)) {
+                        /* If binding is required and a binding directive was explicitly
+                         * given (i.e., we are not binding due to a default policy),
+                         * send an error up the pipe (which exits -- it doesn't return).
+                         */
                         send_error_show_help(write_fd, 1, "help-orte-odls-default.txt",
                                              "binding generic error",
                                              orte_process_info.nodename, 
@@ -462,8 +480,8 @@ static int do_child(orte_app_context_t* context,
                 }
                 /* bind as specified */
                 rc = hwloc_set_cpubind(opal_hwloc_topology, cpuset, 0);
-                if (rc < 0) {
-                    char *tmp = NULL;
+                /* if we got an error and this wasn't a default binding policy, then report it */
+                if (rc < 0  && (OPAL_BIND_GIVEN & jobdat->map->binding)) {
                     if (errno == ENOSYS) {
                         msg = "hwloc indicates cpu binding not supported";
                     } else if (errno == EXDEV) {
@@ -484,16 +502,8 @@ static int do_child(orte_app_context_t* context,
                                             "help-orte-odls-default.txt", "not bound",
                                             orte_process_info.nodename, context->app, msg,
                                             __FILE__, __LINE__);
-                        if (NULL != tmp) {
-                            free(tmp);
-                            free(msg);
-                        }
                         goto PROCEED;
                     }
-                    if (NULL != tmp) {
-                        free(tmp);
-                        free(msg);
-                    }
                 }
                 if (0 == rc && opal_hwloc_report_bindings) {
                     char tmp1[1024], tmp2[1024];
@@ -506,10 +516,13 @@ static int do_child(orte_app_context_t* context,
                         opal_output(0, "MCW rank %d is not bound",
                                     child->name.vpid);
                     } else {
-                        opal_hwloc_base_cset2str(tmp1, sizeof(tmp1), mycpus);
-                        opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), mycpus);
-                        opal_output(0, "MCW rank %d bound to %s: %s",
-                                    child->name.vpid, tmp1, tmp2);
+                        if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2str(tmp1, sizeof(tmp1), mycpus)) {
+                            opal_output(0, "MCW rank %d is not bound (or bound to all available processors)", child->name.vpid);
+                        } else {
+                            opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), mycpus);
+                            opal_output(0, "MCW rank %d bound to %s: %s",
+                                        child->name.vpid, tmp1, tmp2);
+                        }
                     }
                     hwloc_bitmap_free(mycpus);
                     /* avoid reporting it twice */
@@ -517,8 +530,11 @@ static int do_child(orte_app_context_t* context,
                     opal_unsetenv(param, &environ_copy);
                     free(param);
                 }
-                /* set memory affinity policy */
-                if (ORTE_SUCCESS != opal_hwloc_base_set_process_membind_policy()) {
+                /* set memory affinity policy - if we get an error, don't report
+                 * anything unless the user actually specified the binding policy
+                 */
+                rc = opal_hwloc_base_set_process_membind_policy();
+                if (ORTE_SUCCESS != rc  && (OPAL_BIND_GIVEN & jobdat->map->binding)) {
                     if (errno == ENOSYS) {
                         msg = "hwloc indicates memory binding not supported";
                     } else if (errno == EXDEV) {
@@ -576,6 +592,19 @@ static int do_child(orte_app_context_t* context,
 #if OPAL_HAVE_HWLOC
  PROCEED:
 #endif
+
+    /* if the user requested it, set the system resource limits */
+    if (OPAL_SUCCESS != (rc = opal_util_init_sys_limits(&msg))) {
+        send_error_show_help(write_fd, 1, "help-orte-odls-default.txt",
+                             "set limit",
+                             orte_process_info.nodename, context->app, 
+                             __FILE__, __LINE__, msg);
+    }
+    /* ensure we only do this once */
+    (void) mca_base_var_env_name("opal_set_max_sys_limits", &param);
+    opal_unsetenv(param, &environ_copy);
+    free(param);
+
     /* close all file descriptors w/ exception of stdin/stdout/stderr,
        the pipe used for the IOF INTERNAL messages, and the pipe up to
        the parent. */
