@@ -62,8 +62,6 @@ static bool recv_issued=false;
 
 int orte_plm_base_comm_start(void)
 {
-    int rc;
-
     if (recv_issued) {
         return ORTE_SUCCESS;
     }
@@ -72,30 +70,24 @@ int orte_plm_base_comm_start(void)
                          "%s plm:base:receive start comm",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
     
-    if (ORTE_SUCCESS != (rc = orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD,
-                                                      ORTE_RML_TAG_PLM,
-                                                      ORTE_RML_PERSISTENT,
-                                                      orte_plm_base_recv,
-                                                      NULL))) {
-        ORTE_ERROR_LOG(rc);
-    }
+    orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD,
+                            ORTE_RML_TAG_PLM,
+                            ORTE_RML_PERSISTENT,
+                            orte_plm_base_recv,
+                            NULL);
     if (ORTE_PROC_IS_HNP) {
-        if (ORTE_SUCCESS != (rc = orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD,
-                                                          ORTE_RML_TAG_ORTED_CALLBACK,
-                                                          ORTE_RML_PERSISTENT,
-                                                          orte_plm_base_daemon_callback, NULL))) {
-            ORTE_ERROR_LOG(rc);
-        }
-        if (ORTE_SUCCESS != (rc = orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD,
-                                                          ORTE_RML_TAG_REPORT_REMOTE_LAUNCH,
-                                                          ORTE_RML_PERSISTENT,
-                                                          orte_plm_base_daemon_failed, NULL))) {
-            ORTE_ERROR_LOG(rc);
-        }
+        orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD,
+                                ORTE_RML_TAG_ORTED_CALLBACK,
+                                ORTE_RML_PERSISTENT,
+                                orte_plm_base_daemon_callback, NULL);
+        orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD,
+                                ORTE_RML_TAG_REPORT_REMOTE_LAUNCH,
+                                ORTE_RML_PERSISTENT,
+                                orte_plm_base_daemon_failed, NULL);
     }
     recv_issued = true;
     
-    return rc;
+    return ORTE_SUCCESS;
 }
 
 
@@ -138,7 +130,10 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
     orte_process_name_t name;
     pid_t pid;
     bool running;
-        
+    int8_t flag;
+    int i;
+    char **env;
+
     OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
                          "%s plm:base:receive processing msg",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
@@ -185,7 +180,21 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
             NULL == child_app->prefix_dir) {
             child_app->prefix_dir = strdup(app->prefix_dir);
         }
-                    
+
+        /* if the user asked to forward any envars, cycle through the app contexts
+         * in the comm_spawn request and add them
+         */
+        if (NULL != orte_forwarded_envars) {
+            for (i=0; i < jdata->apps->size; i++) {
+                if (NULL == (app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, i))) {
+                    continue;
+                }
+                env = opal_environ_merge(orte_forwarded_envars, app->env);
+                opal_argv_free(app->env);
+                app->env = env;
+            }
+        }
+
         OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
                              "%s plm:base:receive adding hosts",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
@@ -233,7 +242,7 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
         }
                 
         /* send the response back to the sender */
-        if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_PLM_PROXY, 0,
+        if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_PLM_PROXY,
                                                orte_rml_send_callback, NULL))) {
             ORTE_ERROR_LOG(ret);
             OBJ_RELEASE(answer);
@@ -241,25 +250,22 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
         break;
                 
     case ORTE_PLM_UPDATE_PROC_STATE:
-        OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
-                             "%s plm:base:receive update proc state command from %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             ORTE_NAME_PRINT(sender)));
+        opal_output_verbose(5, orte_plm_base_framework.framework_output,
+                            "%s plm:base:receive update proc state command from %s",
+                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                            ORTE_NAME_PRINT(sender));
         count = 1;
         while (ORTE_SUCCESS == (rc = opal_dss.unpack(buffer, &job, &count, ORTE_JOBID))) {
                     
-            OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
-                                 "%s plm:base:receive got update_proc_state for job %s",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                 ORTE_JOBID_PRINT(job)));
+            opal_output_verbose(5, orte_plm_base_framework.framework_output,
+                                "%s plm:base:receive got update_proc_state for job %s",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                ORTE_JOBID_PRINT(job));
                     
             name.jobid = job;
             running = false;
             /* get the job object */
-            if (NULL == (jdata = orte_get_job_data_object(job))) {
-                ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-                goto CLEANUP;
-            }
+            jdata = orte_get_job_data_object(job);
             count = 1;
             while (ORTE_SUCCESS == (rc = opal_dss.unpack(buffer, &vpid, &count, ORTE_VPID))) {
                 if (ORTE_VPID_INVALID == vpid) {
@@ -294,17 +300,20 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                      (unsigned long)vpid, orte_proc_state_to_str(state), (int)exit_code));
 
-                /* get the proc data object */
-                if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, vpid))) {
-                    ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-                    ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+                if (NULL != jdata) {
+                    /* get the proc data object */
+                    if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, vpid))) {
+                        ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+                        ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+                    }
+                    proc->state = state;
+                    proc->pid = pid;
+                    proc->exit_code = exit_code;
+                    ORTE_ACTIVATE_PROC_STATE(&name, state);
                 }
-                proc->state = state;
-                proc->pid = pid;
-                proc->exit_code = exit_code;
-                ORTE_ACTIVATE_PROC_STATE(&name, state);
             }
-            if (running) {
+            /* record that we heard back from a daemon during app launch */
+            if (running && NULL != jdata) {
                 jdata->num_daemons_reported++;
                 if (orte_report_launch_progress) {
                     if (0 == jdata->num_daemons_reported % 100 ||
@@ -327,13 +336,14 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
         count=1;
         if (ORTE_SUCCESS != (rc = opal_dss.unpack(buffer, &job, &count, ORTE_JOBID))) {
             ORTE_ERROR_LOG(rc);
-            goto CLEANUP;
+            goto DEPART;
         }
         name.jobid = job;
         /* get the job object */
         if (NULL == (jdata = orte_get_job_data_object(job))) {
             ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-            goto CLEANUP;
+            rc = ORTE_ERR_NOT_FOUND;
+            goto DEPART;
         }
         count=1;
         while (ORTE_SUCCESS == opal_dss.unpack(buffer, &vpid, &count, ORTE_VPID)) {
@@ -341,6 +351,23 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
                 break;
             }
             name.vpid = vpid;
+            /* unpack the mpi proc flag */
+            count=1;
+            if (OPAL_SUCCESS != (rc = opal_dss.unpack(buffer, &flag, &count, OPAL_INT8))) {
+                ORTE_ERROR_LOG(rc);
+                goto DEPART;
+            }
+            /* get the proc data object */
+            if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, vpid))) {
+                ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+                rc = ORTE_ERR_NOT_FOUND;
+                goto DEPART;
+            }
+            if (1 == flag) {
+                proc->mpi_proc = true;
+            } else {
+                proc->mpi_proc = false;
+            }
             ORTE_ACTIVATE_PROC_STATE(&name, ORTE_PROC_STATE_REGISTERED);
             count=1;
         }
@@ -349,7 +376,7 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
          */
         if (ORTE_SUCCESS != (rc = orte_routed.init_routes(job, buffer))) {
             ORTE_ERROR_LOG(rc);
-            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
         }
         break;
 
@@ -368,7 +395,7 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
     /* see if an error occurred - if so, wakeup the HNP so we can exit */
     if (ORTE_PROC_IS_HNP && ORTE_SUCCESS != rc) {
         jdata = NULL;
-        ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+        ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
     }
     
     OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
