@@ -283,8 +283,10 @@ static void rsh_wait_daemon(pid_t pid, int status, void* cbdata)
             opal_dss.pack(buf, &(daemon->name.vpid), 1, ORTE_VPID);
             opal_dss.pack(buf, &status, 1, OPAL_INT);
             orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, buf,
-                                    ORTE_RML_TAG_REPORT_REMOTE_LAUNCH, 0,
+                                    ORTE_RML_TAG_REPORT_REMOTE_LAUNCH,
                                     orte_rml_send_callback, NULL);
+            /* note that this daemon failed */
+            daemon->state = ORTE_PROC_STATE_FAILED_TO_START;
         } else {
             jdata = orte_get_job_data_object(ORTE_PROC_MY_NAME->jobid);
             
@@ -876,7 +878,7 @@ cleanup:
         opal_dss.pack(buf, &target.vpid, 1, ORTE_VPID);
         opal_dss.pack(buf, &rc, 1, OPAL_INT);
         orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, buf,
-                                ORTE_RML_TAG_REPORT_REMOTE_LAUNCH, 0,
+                                ORTE_RML_TAG_REPORT_REMOTE_LAUNCH,
                                 orte_rml_send_callback, NULL);
     }
     
@@ -1098,8 +1100,8 @@ static void launch_daemons(int fd, short args, void *cbdata)
             OBJ_RELEASE(orte_tree_launch_cmd);
             goto cleanup;
         }
-        /* construct a nodemap */
-        if (ORTE_SUCCESS != (rc = orte_util_encode_nodemap(&bo))) {
+        /* construct a nodemap of all daemons we know about */
+        if (ORTE_SUCCESS != (rc = orte_util_encode_nodemap(&bo, false))) {
             ORTE_ERROR_LOG(rc);
             OBJ_RELEASE(orte_tree_launch_cmd);
             goto cleanup;
@@ -1232,7 +1234,7 @@ static void launch_daemons(int fd, short args, void *cbdata)
     
 cleanup:
     OBJ_RELEASE(state);
-    ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+    ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
 }
 
 /**
