@@ -263,6 +263,9 @@ static void launch_daemons(int fd, short args, void *cbdata)
     /* add the srun command */
     opal_argv_append(&argc, &argv, "srun");
 
+    /* start one orted on each node */
+    opal_argv_append(&argc, &argv, "--ntasks-per-node=1");
+
     /* alert us if any orteds die during startup */
     opal_argv_append(&argc, &argv, "--kill-on-bad-exit");
 
@@ -429,7 +432,7 @@ static void launch_daemons(int fd, short args, void *cbdata)
 
     /* check for failed launch - if so, force terminate */
     if (failed_launch) {
-        ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+        ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
     }
 }
 
@@ -448,11 +451,21 @@ static int plm_slurm_terminate_orteds(void)
      * exit. Instead, we simply trigger an exit for ourselves
      */
     if (primary_pid_set) {
-        /* tell them to die without sending a reply - we will rely on the
-         * waitpid to tell us when they have exited!
-         */
-        if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_HALT_VM_CMD))) {
-            ORTE_ERROR_LOG(rc);
+        if (orte_abnormal_term_ordered) {
+            /* cannot know if a daemon is able to
+             * tell us it died, so just ensure they
+             * all terminate
+             */
+            if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_HALT_VM_CMD))) {
+                ORTE_ERROR_LOG(rc);
+            }
+        } else {
+            /* we need them to "phone home"
+             * so we can know that they have exited
+             */
+            if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_EXIT_CMD))) {
+                ORTE_ERROR_LOG(rc);
+            }
         }
     } else {
         OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
@@ -635,17 +648,14 @@ static int plm_slurm_start_proc(int argc, char **argv, char **env,
             if (fd >= 0) {
                 if (fd != 1) {
                     dup2(fd,1);
-                }
-                if (fd != 2) {
+                } else if (fd != 2) {
                     dup2(fd,2);
+                } else {
+                    close(fd);
                 }
             }
         }
 
-        if (fd > 2) {
-            close(fd);
-        }
-
         /* get the srun process out of orterun's process group so that
            signals sent from the shell (like those resulting from
            cntl-c) don't get sent to srun */
