@@ -53,7 +53,7 @@ typedef struct orte_ras_alps_sysconfig_t {
 } orte_ras_alps_sysconfig_t;
 
 /* /// Local Functions /// */
-static int orte_ras_alps_allocate(opal_list_t *nodes);
+static int orte_ras_alps_allocate(orte_job_t *jdata, opal_list_t *nodes);
 
 static int orte_ras_alps_finalize(void);
 
@@ -80,7 +80,9 @@ static const orte_ras_alps_sysconfig_t sysconfigs[] = {
 
 /* /// Global Variables /// */
 orte_ras_base_module_t orte_ras_alps_module = {
+    NULL,
     orte_ras_alps_allocate,
+    NULL,
     orte_ras_alps_finalize
 };
 
@@ -297,7 +299,7 @@ orte_ras_get_appinfo_path(void)
  * requested number of nodes/process slots to the job.
  */
 static int
-orte_ras_alps_allocate(opal_list_t *nodes)
+orte_ras_alps_allocate(orte_job_t *jdata, opal_list_t *nodes)
 {
     int ret;
     char *appinfo_path = NULL;
