@@ -32,6 +32,7 @@
 #include "opal/class/opal_list.h"
 #include "opal/util/output.h"
 #include "opal/dss/dss.h"
+#include "opal/util/argv.h"
 
 #include "orte/util/show_help.h"
 #include "orte/mca/errmgr/errmgr.h"
@@ -48,16 +49,15 @@
 
 #include "orte/mca/ras/base/ras_private.h"
 
-/* static function to display allocation */
-static void display_alloc(void)
+/* function to display allocation */
+void orte_ras_base_display_alloc(void)
 {
-    char *tmp=NULL, *tmp2, *tmp3, *pfx=NULL;
+    char *tmp=NULL, *tmp2, *tmp3;
     int i, istart;
     orte_node_t *alloc;
     
     if (orte_xml_output) {
         asprintf(&tmp, "<allocation>\n");
-        pfx = "\t";
     } else {
         asprintf(&tmp, "\n======================   ALLOCATED NODES   ======================\n");
     }
@@ -70,7 +70,16 @@ static void display_alloc(void)
         if (NULL == (alloc = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, i))) {
             continue;
         }
-        opal_dss.print(&tmp2, pfx, alloc, ORTE_NODE);
+        if (orte_xml_output) {
+            /* need to create the output in XML format */
+            asprintf(&tmp2, "\t<host name=\"%s\" slots=\"%d\" max_slots=\"%d\" slots_inuse=\"%d\">\n",
+                     (NULL == alloc->name) ? "UNKNOWN" : alloc->name,
+                     (int)alloc->slots, (int)alloc->slots_max, (int)alloc->slots_inuse);
+        } else {
+            asprintf(&tmp2, "\t%s: slots=%d max_slots=%d slots_inuse=%d\n",
+                     (NULL == alloc->name) ? "UNKNOWN" : alloc->name,
+                     (int)alloc->slots, (int)alloc->slots_max, (int)alloc->slots_inuse);
+        }
         if (NULL == tmp) {
             tmp = tmp2;
         } else {
@@ -84,7 +93,7 @@ static void display_alloc(void)
         fprintf(orte_xml_fp, "%s</allocation>\n", tmp);
         fflush(orte_xml_fp);
     } else {
-        opal_output(orte_clean_output, "%s\n\n=================================================================\n", tmp);
+        opal_output(orte_clean_output, "%s=================================================================\n", tmp);
     }
     free(tmp);    
 }
@@ -102,7 +111,6 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
     orte_std_cntr_t i;
     orte_app_context_t *app;
     orte_state_caddy_t *caddy = (orte_state_caddy_t*)cbdata;
-    bool default_hostfile_used;
 
     OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                          "%s ras:base:allocate",
@@ -140,7 +148,13 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
      */
     if (NULL != orte_ras_base.active_module)  {
         /* read the allocation */
-        if (ORTE_SUCCESS != (rc = orte_ras_base.active_module->allocate(&nodes))) {
+        if (ORTE_SUCCESS != (rc = orte_ras_base.active_module->allocate(jdata, &nodes))) {
+            if (ORTE_ERR_ALLOCATION_PENDING == rc) {
+                /* an allocation request is underway, so just do nothing */
+                OBJ_DESTRUCT(&nodes);
+                OBJ_RELEASE(caddy);
+                return;
+            }
             if (ORTE_ERR_SYSTEM_WILL_BOOTSTRAP == rc) {
                 /* this module indicates that nodes will be discovered
                  * on a bootstrap basis, so all we do here is add our
@@ -148,9 +162,29 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
                  */
                 goto addlocal;
             }
+            if (ORTE_ERR_TAKE_NEXT_OPTION == rc) {
+                /* we have an active module, but it is unable to
+                 * allocate anything for this job - this indicates
+                 * that it isn't a fatal error, but could be if
+                 * an allocation is required
+                 */
+                if (orte_allocation_required) {
+                    /* an allocation is required, so this is fatal */
+                    OBJ_DESTRUCT(&nodes);
+                    orte_show_help("help-ras-base.txt", "ras-base:no-allocation", true);
+                    ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+                    OBJ_RELEASE(caddy);
+                    return;
+                } else {
+                    /* an allocation is not required, so we can just
+                     * run on the local node - go add it
+                     */
+                    goto addlocal;
+                }
+            }
             ORTE_ERROR_LOG(rc);
             OBJ_DESTRUCT(&nodes);
-            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
             OBJ_RELEASE(caddy);
             return;
         }
@@ -163,7 +197,7 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
         if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
             ORTE_ERROR_LOG(rc);
             OBJ_DESTRUCT(&nodes);
-            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
             OBJ_RELEASE(caddy);
             return;
         }
@@ -181,7 +215,7 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
          */
         OBJ_DESTRUCT(&nodes);
         orte_show_help("help-ras-base.txt", "ras-base:no-allocation", true);
-        ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+        ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
         OBJ_RELEASE(caddy);
         return;
     }
@@ -202,17 +236,47 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
      * the resulting list contains the UNION of all nodes specified
      * in hostfiles from across all app_contexts
      *
-     * Any app that has no hostfile but has a dash-host, will have
-     * those nodes added to the list
-     *
-     * Any app that fails to have a hostfile or a dash-host will be given the
-     * default hostfile, if we have it
+     * We then continue to add any hosts provided by dash-host and
+     * the default hostfile, if we have it. We will then filter out
+     * all the non-desired hosts (i.e., those not specified by
+     * -host and/or -hostfile) when we start the mapping process
      *
      * Note that any relative node syntax found in the hostfiles will
      * generate an error in this scenario, so only non-relative syntax
      * can be present
      */
-    default_hostfile_used = false;
+    if (NULL != orte_default_hostfile) {
+        OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
+                             "%s ras:base:allocate parsing default hostfile %s",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                             orte_default_hostfile));
+        
+        /* a default hostfile was provided - parse it */
+        if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
+                                                               orte_default_hostfile))) {
+            ORTE_ERROR_LOG(rc);
+            OBJ_DESTRUCT(&nodes);
+            ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            OBJ_RELEASE(caddy);
+            return;
+        }
+    }
+    if (NULL != orte_rankfile) {
+        OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
+                             "%s ras:base:allocate parsing rankfile %s",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                             orte_rankfile));
+        
+        /* a rankfile was provided - parse it */
+        if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
+                                                               orte_rankfile))) {
+            ORTE_ERROR_LOG(rc);
+            OBJ_DESTRUCT(&nodes);
+            ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            OBJ_RELEASE(caddy);
+            return;
+        }
+    }
     for (i=0; i < jdata->apps->size; i++) {
         if (NULL == (app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, i))) {
             continue;
@@ -229,11 +293,18 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
                 ORTE_ERROR_LOG(rc);
                 OBJ_DESTRUCT(&nodes);
                 /* set an error event */
-                ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+                ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
                 OBJ_RELEASE(caddy);
                 return;
             }
-        } else if (NULL != app->dash_host) {
+        } else if (!orte_soft_locations && NULL != app->dash_host) {
+            /* if we are using soft locations, then any dash-host would
+             * just include desired nodes and not required. We don't want
+             * to pick them up here as this would mean the request was
+             * always satisfied - instead, we want to allow the request
+             * to fail later on and use whatever nodes are actually
+             * available
+             */
             OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                                  "%s ras:base:allocate adding dash_hosts",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
@@ -241,29 +312,10 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
                                                                     app->dash_host))) {
                 ORTE_ERROR_LOG(rc);
                 OBJ_DESTRUCT(&nodes);
-                ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+                ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
                 OBJ_RELEASE(caddy);
                 return;
             }
-        } else if (!default_hostfile_used) {
-            if (NULL != orte_default_hostfile) {
-                OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
-                                     "%s ras:base:allocate parsing default hostfile %s",
-                                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                     orte_default_hostfile));
-        
-                /* a default hostfile was provided - parse it */
-                if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
-                                                                       orte_default_hostfile))) {
-                    ORTE_ERROR_LOG(rc);
-                    OBJ_DESTRUCT(&nodes);
-                    ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
-                    OBJ_RELEASE(caddy);
-                    return;
-                }
-            }
-            /* only look at it once */
-            default_hostfile_used = true;
         }
     }
 
@@ -276,7 +328,7 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
          */
         if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
             ORTE_ERROR_LOG(rc);
-            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
             OBJ_RELEASE(caddy);
             return;
         }
@@ -298,7 +350,7 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
                                                                orte_rankfile))) {
             ORTE_ERROR_LOG(rc);
             OBJ_DESTRUCT(&nodes);
-            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
             OBJ_RELEASE(caddy);
             return ;
         }
@@ -312,7 +364,7 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
          */
         if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
             ORTE_ERROR_LOG(rc);
-            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
             OBJ_RELEASE(caddy);
             return;
         }
@@ -338,7 +390,7 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
     if (NULL == node) {
         ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
         OBJ_DESTRUCT(&nodes);
-        ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+        ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
         OBJ_RELEASE(caddy);
         return;
     }
@@ -358,7 +410,7 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
     if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
         ORTE_ERROR_LOG(rc);
         OBJ_DESTRUCT(&nodes);
-        ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+        ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
         OBJ_RELEASE(caddy);
         return;
     }
@@ -366,8 +418,8 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
 
  DISPLAY:
     /* shall we display the results? */
-    if (4 < opal_output_get_verbosity(orte_ras_base_framework.framework_output) || orte_ras_base.display_alloc) {
-        display_alloc();
+    if (4 < opal_output_get_verbosity(orte_ras_base_framework.framework_output)) {
+        orte_ras_base_display_alloc();
     }
 
  next_state:
@@ -375,7 +427,7 @@ void orte_ras_base_allocate(int fd, short args, void *cbdata)
     if (orte_report_events) {
         if (ORTE_SUCCESS != (rc = orte_util_comm_report_event(ORTE_COMM_EVENT_ALLOCATE))) {
             ORTE_ERROR_LOG(rc);
-            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
             OBJ_RELEASE(caddy);
         }
     }
@@ -430,6 +482,9 @@ int orte_ras_base_add_hosts(orte_job_t *jdata)
                 OBJ_DESTRUCT(&nodes);
                 return rc;
             }
+            /* now indicate that this app is to run across it */
+            app->hostfile = app->add_hostfile;
+            app->add_hostfile = NULL;
         }
     }
 
@@ -447,12 +502,21 @@ int orte_ras_base_add_hosts(orte_job_t *jdata)
             continue;
         }
         if (NULL != app->add_host) {
+            if (4 < opal_output_get_verbosity(orte_ras_base_framework.framework_output)) {
+                char *fff = opal_argv_join(app->add_host, ',');
+                opal_output(0, "%s ras:base:add_hosts checking add-host %s",
+                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), fff);
+                free(fff);
+            }
             if (ORTE_SUCCESS != (rc = orte_util_add_dash_host_nodes(&nodes,
                                                                     app->add_host))) {
                 ORTE_ERROR_LOG(rc);
                 OBJ_DESTRUCT(&nodes);
                 return rc;
             }
+            /* now indicate that this app is to run across them */
+            app->dash_host = app->add_host;
+            app->add_host = NULL;
         }
     }
     
@@ -469,8 +533,8 @@ int orte_ras_base_add_hosts(orte_job_t *jdata)
     }
     
     /* shall we display the results? */
-    if (0 < opal_output_get_verbosity(orte_ras_base_framework.framework_output) || orte_ras_base.display_alloc) {
-        display_alloc();
+    if (0 < opal_output_get_verbosity(orte_ras_base_framework.framework_output)) {
+        orte_ras_base_display_alloc();
     }
     
     return ORTE_SUCCESS;
