@@ -44,9 +44,6 @@ BEGIN_C_DECLS
  * MCA Framework
  */
 ORTE_DECLSPEC extern mca_base_framework_t orte_rmaps_base_framework;
-
-#if !ORTE_DISABLE_FULL_SUPPORT
-
 /* select a component */
 ORTE_DECLSPEC    int orte_rmaps_base_select(void);
 
@@ -125,7 +122,11 @@ ORTE_DECLSPEC int orte_rmaps_base_filter_nodes(orte_app_context_t *app,
                                                opal_list_t *nodes,
                                                bool remove);
 
-#endif /* ORTE_DISABLE_FULL_SUPPORT */
+ORTE_DECLSPEC int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
+                                                     char **device, char *spec);
+ORTE_DECLSPEC int orte_rmaps_base_set_ranking_policy(orte_ranking_policy_t *policy,
+                                                     orte_mapping_policy_t mapping,
+                                                     char *spec);
 
 END_C_DECLS
 
