@@ -12,6 +12,7 @@
  * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2012 Los Alamos National Security, LLC.
  *                         All rights reserved.
+ * Copyright (c) 2013      Intel, Inc. All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -50,6 +51,56 @@
 
 static bool membind_warned=false;
 
+static void reset_usage(orte_node_t *node, orte_jobid_t jobid)
+{
+    int j;
+    orte_proc_t *proc;
+    opal_hwloc_obj_data_t *data=NULL;
+
+    opal_output_verbose(10, orte_rmaps_base_framework.framework_output,
+                        "%s reset_usage: node %s has %d procs on it",
+                        ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                        node->name, node->num_procs);
+
+    /* start by clearing any existing info */
+    opal_hwloc_base_clear_usage(node->topology);
+
+    /* cycle thru the procs on the node and record
+     * their usage in the topology
+     */
+    for (j=0; j < node->procs->size; j++) {
+        if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
+            continue;
+        }
+        /* ignore procs from this job */
+        if (proc->name.jobid == jobid) {
+            opal_output_verbose(10, orte_rmaps_base_framework.framework_output,
+                                "%s reset_usage: ignoring proc %s",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                ORTE_NAME_PRINT(&proc->name));
+            continue;
+        }
+        if (NULL == proc->bind_location) {
+            /* this proc isn't bound - ignore it */
+            opal_output_verbose(10, orte_rmaps_base_framework.framework_output,
+                                "%s reset_usage: proc %s has no bind location",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                ORTE_NAME_PRINT(&proc->name));
+            continue;
+        }
+        data = (opal_hwloc_obj_data_t*)proc->bind_location->userdata;
+        if (NULL == data) {
+            data = OBJ_NEW(opal_hwloc_obj_data_t);
+            proc->bind_location->userdata = data;
+        }
+        data->num_bound++;
+        opal_output_verbose(10, orte_rmaps_base_framework.framework_output,
+                            "%s reset_usage: proc %s is bound - total %d",
+                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                            ORTE_NAME_PRINT(&proc->name), data->num_bound);
+    }
+}
+
 static int bind_upwards(orte_job_t *jdata,
                         hwloc_obj_type_t target,
                         unsigned cache_level)
@@ -91,7 +142,8 @@ static int bind_upwards(orte_job_t *jdata,
              */
             if (!support->cpubind->set_thisproc_cpubind &&
                 !support->cpubind->set_thisthread_cpubind) {
-                if (!OPAL_BINDING_REQUIRED(opal_hwloc_binding_policy)) {
+                if (!OPAL_BINDING_REQUIRED(opal_hwloc_binding_policy) ||
+                    !(OPAL_BIND_GIVEN & opal_hwloc_binding_policy)) {
                     /* we are not required to bind, so ignore this */
                     continue;
                 }
@@ -103,10 +155,12 @@ static int bind_upwards(orte_job_t *jdata,
              * expected. Per hwloc, Linux memory binding is at the thread,
              * and not process, level. Thus, hwloc sets the "thisproc" flag
              * to "false" on all Linux systems, and uses the "thisthread" flag
-             * to indicate binding capability
+             * to indicate binding capability - don't warn if the user didn't
+             * specifically request binding
              */
             if (!support->membind->set_thisproc_membind &&
-                !support->membind->set_thisthread_membind) {
+                !support->membind->set_thisthread_membind &&
+                (OPAL_BIND_GIVEN & opal_hwloc_binding_policy)) {
                 if (OPAL_HWLOC_BASE_MBFA_WARN == opal_hwloc_base_mbfa && !membind_warned) {
                     orte_show_help("help-orte-rmaps-base.txt", "rmaps:membind-not-supported", true, node->name);
                     membind_warned = true;
@@ -117,8 +171,24 @@ static int bind_upwards(orte_job_t *jdata,
             }
         }
 
-        /* clear the topology of any prior usage numbers */
-        opal_hwloc_base_clear_usage(node->topology);
+        /* some systems do not report cores, and so we can get a situation where our
+         * default binding policy will fail for no necessary reason. So if we are
+         * computing a binding due to our default policy, and no cores are found
+         * on this node, just silently skip it - we will not bind
+         */
+        if (!(OPAL_BIND_GIVEN & opal_hwloc_binding_policy) &&
+            HWLOC_TYPE_DEPTH_UNKNOWN == hwloc_get_type_depth(node->topology, HWLOC_OBJ_CORE)) {
+            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                "Unable to bind-to core by default on node %s as no cores detected",
+                                node->name);
+            continue;
+        }
+
+        /* we share topologies in order
+         * to save space, so we need to reset the usage info to reflect
+         * our own current state
+         */
+        reset_usage(node, jdata->jobid);
 
         /* cycle thru the procs */
         for (j=0; j < node->procs->size; j++) {
@@ -137,17 +207,20 @@ static int bind_upwards(orte_job_t *jdata,
             }
             /* bozo check */
             if (NULL == proc->locale) {
-                opal_output(0, "BIND UPWARDS: LOCALE FOR PROC %s IS NULL", ORTE_NAME_PRINT(&proc->name));
+                opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                    "BIND UPWARDS: LOCALE FOR PROC %s IS NULL",
+                                    ORTE_NAME_PRINT(&proc->name));
                 return ORTE_ERR_SILENT;
             }
             /* starting at the locale, move up thru the parents
              * to find the target object type
              */
             for (obj = proc->locale->parent; NULL != obj; obj = obj->parent) {
-                opal_output(0, "%s bind:upward target %s type %s",
-                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                            hwloc_obj_type_string(target),
-                            hwloc_obj_type_string(obj->type));
+                opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                    "%s bind:upward target %s type %s",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                    hwloc_obj_type_string(target),
+                                    hwloc_obj_type_string(obj->type));
                 if (target == obj->type) {
                     if (HWLOC_OBJ_CACHE == target && cache_level != obj->attr->cache.depth) {
                         continue;
@@ -164,18 +237,22 @@ static int bind_upwards(orte_job_t *jdata,
                         orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-available-cpus", true, node->name);
                         return ORTE_ERR_SILENT;
                     }
-                    /* error out if adding a proc would cause overload and that wasn't allowed */
+                    /* error out if adding a proc would cause overload and that wasn't allowed,
+                     * and it wasn't a default binding policy (i.e., the user requested it)
+                     */
                     if (ncpus < data->num_bound &&
-                        !OPAL_BIND_OVERLOAD_ALLOWED(jdata->map->binding)) {
+                        !OPAL_BIND_OVERLOAD_ALLOWED(jdata->map->binding) &&
+                        (OPAL_BIND_GIVEN & opal_hwloc_binding_policy)) {
                         orte_show_help("help-orte-rmaps-base.txt", "rmaps:binding-overload", true,
                                        opal_hwloc_base_print_binding(map->binding), node->name,
                                        data->num_bound, ncpus);
                         return ORTE_ERR_SILENT;
                     }
                     /* bind it here */
-                    proc->bind_idx = idx;
                     cpus = opal_hwloc_base_get_available_cpus(node->topology, obj);
                     hwloc_bitmap_list_asprintf(&proc->cpu_bitmap, cpus);
+                    /* record the location */
+                    proc->bind_location = obj;
                     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                         "%s BOUND PROC %s TO %s[%s:%u] on node %s",
                                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -208,11 +285,13 @@ static int bind_downwards(orte_job_t *jdata,
     orte_job_map_t *map;
     orte_node_t *node;
     orte_proc_t *proc;
-    hwloc_obj_t trg_obj;
+    hwloc_obj_t trg_obj, nxt_obj;
     hwloc_cpuset_t cpus;
-    unsigned int ncpus, idx;
+    unsigned int ncpus;
     struct hwloc_topology_support *support;
     opal_hwloc_obj_data_t *data;
+    int total_cpus;
+    hwloc_cpuset_t totalcpuset;
 
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                         "mca:rmaps: bind downward for job %s with bindings %s",
@@ -220,6 +299,7 @@ static int bind_downwards(orte_job_t *jdata,
                         opal_hwloc_base_print_binding(jdata->map->binding));
     /* initialize */
     map = jdata->map;
+    totalcpuset = hwloc_bitmap_alloc();
 
     for (i=0; i < map->nodes->size; i++) {
         if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
@@ -237,11 +317,13 @@ static int bind_downwards(orte_job_t *jdata,
              */
             if (!support->cpubind->set_thisproc_cpubind &&
                 !support->cpubind->set_thisthread_cpubind) {
-                if (!OPAL_BINDING_REQUIRED(opal_hwloc_binding_policy)) {
+                if (!OPAL_BINDING_REQUIRED(opal_hwloc_binding_policy) ||
+                    !(OPAL_BIND_GIVEN & opal_hwloc_binding_policy)) {
                     /* we are not required to bind, so ignore this */
                     continue;
                 }
                 orte_show_help("help-orte-rmaps-base.txt", "rmaps:cpubind-not-supported", true, node->name);
+                hwloc_bitmap_free(totalcpuset);
                 return ORTE_ERR_SILENT;
             }
             /* check if topology supports membind - have to be careful here
@@ -249,24 +331,43 @@ static int bind_downwards(orte_job_t *jdata,
              * expected. Per hwloc, Linux memory binding is at the thread,
              * and not process, level. Thus, hwloc sets the "thisproc" flag
              * to "false" on all Linux systems, and uses the "thisthread" flag
-             * to indicate binding capability
+             * to indicate binding capability - don't warn if the user didn't
+             * specifically request binding
              */
             if (!support->membind->set_thisproc_membind &&
-                !support->membind->set_thisthread_membind) {
+                !support->membind->set_thisthread_membind &&
+                (OPAL_BIND_GIVEN & opal_hwloc_binding_policy)) {
                 if (OPAL_HWLOC_BASE_MBFA_WARN == opal_hwloc_base_mbfa && !membind_warned) {
                     orte_show_help("help-orte-rmaps-base.txt", "rmaps:membind-not-supported", true, node->name);
                     membind_warned = true;
                 } else if (OPAL_HWLOC_BASE_MBFA_ERROR == opal_hwloc_base_mbfa) {
                     orte_show_help("help-orte-rmaps-base.txt", "rmaps:membind-not-supported-fatal", true, node->name);
+                    hwloc_bitmap_free(totalcpuset);
                     return ORTE_ERR_SILENT;
                 }
             }
         }
 
-        /* clear the topology of any prior usage numbers */
-        opal_hwloc_base_clear_usage(node->topology);
+        /* some systems do not report cores, and so we can get a situation where our
+         * default binding policy will fail for no necessary reason. So if we are
+         * computing a binding due to our default policy, and no cores are found
+         * on this node, just silently skip it - we will not bind
+         */
+        if (!(OPAL_BIND_GIVEN & opal_hwloc_binding_policy) &&
+            HWLOC_TYPE_DEPTH_UNKNOWN == hwloc_get_type_depth(node->topology, HWLOC_OBJ_CORE)) {
+            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                "Unable to bind-to core by default on node %s as no cores detected",
+                                node->name);
+            continue;
+        }
 
-       /* cycle thru the procs */
+        /* we share topologies in order
+         * to save space, so we need to reset the usage info to reflect
+         * our own current state
+         */
+        reset_usage(node, jdata->jobid);
+
+        /* cycle thru the procs */
         for (j=0; j < node->procs->size; j++) {
             if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
                 continue;
@@ -283,7 +384,7 @@ static int bind_downwards(orte_job_t *jdata,
             }
             /* we don't know if the target is a direct child of this locale,
              * or if it is some depth below it, so we have to conduct a bit
-             * of a search. Let hwloc find the min usage one for us
+             * of a search. Let hwloc find the min usage one for us.
              */
             trg_obj = opal_hwloc_base_find_min_bound_target_under_obj(node->topology,
                                                                       proc->locale,
@@ -291,48 +392,84 @@ static int bind_downwards(orte_job_t *jdata,
             if (NULL == trg_obj) {
                 /* there aren't any such targets under this object */
                 orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-available-cpus", true, node->name);
+                hwloc_bitmap_free(totalcpuset);
                 return ORTE_ERR_SILENT;
             }
-            /* get the index of the object */
-            if (UINT_MAX == (idx = opal_hwloc_base_get_obj_idx(node->topology, trg_obj, OPAL_HWLOC_AVAILABLE))) {
-                return ORTE_ERR_SILENT;
-            }
-             /* track the number bound */
-            data = (opal_hwloc_obj_data_t*)trg_obj->userdata;
-            data->num_bound++;
-            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                "%s GETTING NUMBER OF CPUS UNDER OBJECT %s[%d]",
+            /* record the location */
+            proc->bind_location = trg_obj;
+           /* start with a clean slate */
+            hwloc_bitmap_zero(totalcpuset);
+            total_cpus = 0;
+            nxt_obj = trg_obj;
+            do {
+                if (NULL == nxt_obj) {
+                    /* could not find enough cpus to meet request */
+                    orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-available-cpus", true, node->name);
+                    hwloc_bitmap_free(totalcpuset);
+                    return ORTE_ERR_SILENT;
+                }
+                trg_obj = nxt_obj;
+                /* get the number of cpus under this location */
+                ncpus = opal_hwloc_base_get_npus(node->topology, trg_obj);
+                opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                    "%s GOT %d CPUS",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), ncpus);
+                /* track the number bound */
+                if (NULL == (data = (opal_hwloc_obj_data_t*)trg_obj->userdata)) {
+                    data = OBJ_NEW(opal_hwloc_obj_data_t);
+                    trg_obj->userdata = data;
+                }
+                data->num_bound++;
+                /* error out if adding a proc would cause overload and that wasn't allowed,
+                 * and it wasn't a default binding policy (i.e., the user requested it)
+                 */
+                if (ncpus < data->num_bound &&
+                    !OPAL_BIND_OVERLOAD_ALLOWED(jdata->map->binding)) {
+                    if (OPAL_BIND_GIVEN & opal_hwloc_binding_policy) {
+                        orte_show_help("help-orte-rmaps-base.txt", "rmaps:binding-overload", true,
+                                       opal_hwloc_base_print_binding(map->binding), node->name,
+                                       data->num_bound, ncpus);
+                        hwloc_bitmap_free(totalcpuset);
+                        return ORTE_ERR_SILENT;
+                    } else {
+                        /* if this is the default binding policy, then just don't
+                         * bind this proc
+                         */
+                        data->num_bound--;  // maintain count
+                        /* show the proc as not bound */
+                        proc->bind_location = NULL;
+                        hwloc_bitmap_zero(totalcpuset);
+                        break;
+                    }
+                }
+                /* bind the proc here */
+                cpus = opal_hwloc_base_get_available_cpus(node->topology, trg_obj);
+                hwloc_bitmap_or(totalcpuset, totalcpuset, cpus);
+                /* track total #cpus */
+                total_cpus += ncpus;
+                /* move to the next location, in case we need it */
+                nxt_obj = trg_obj->next_cousin;
+            } while (total_cpus < orte_rmaps_base.cpus_per_rank);
+            hwloc_bitmap_list_asprintf(&proc->cpu_bitmap, totalcpuset);
+            if (4 < opal_output_get_verbosity(orte_rmaps_base_framework.framework_output)) {
+                char tmp1[1024], tmp2[1024];
+                if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2str(tmp1, sizeof(tmp1), totalcpuset)) {
+                    opal_output(orte_rmaps_base_framework.framework_output,
+                                "%s PROC %s ON %s IS NOT BOUND",
                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                hwloc_obj_type_string(target), trg_obj->logical_index);
-            /* get the number of cpus under this location */
-            ncpus = opal_hwloc_base_get_npus(node->topology, trg_obj);
-            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                "%s GOT %d CPUS",
-                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), ncpus);
-            if (0 == ncpus) {
-                orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-available-cpus", true, node->name);
-                return ORTE_ERR_SILENT;
-            }
-            /* error out if adding a proc would cause overload and that wasn't allowed */
-            if (ncpus < data->num_bound &&
-                !OPAL_BIND_OVERLOAD_ALLOWED(jdata->map->binding)) {
-                orte_show_help("help-orte-rmaps-base.txt", "rmaps:binding-overload", true,
-                               opal_hwloc_base_print_binding(map->binding), node->name,
-                               data->num_bound, ncpus);
-                return ORTE_ERR_SILENT;
-            }
-            /* bind the proc here */
-            proc->bind_idx = idx;
-            cpus = opal_hwloc_base_get_available_cpus(node->topology, trg_obj);
-            hwloc_bitmap_list_asprintf(&proc->cpu_bitmap, cpus);
-            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                "%s BOUND PROC %s TO %s[%s:%u] on node %s",
+                                ORTE_NAME_PRINT(&proc->name), node->name);
+                } else {
+                    opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), totalcpuset);
+                    opal_output(orte_rmaps_base_framework.framework_output,
+                                "%s BOUND PROC %s[%s] TO %s: %s",
                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                ORTE_NAME_PRINT(&proc->name),
-                                proc->cpu_bitmap, hwloc_obj_type_string(trg_obj->type),
-                                proc->bind_idx, node->name);
+                                ORTE_NAME_PRINT(&proc->name), node->name,
+                                tmp1, tmp2);
+                }
+            }
         }
     }
+    hwloc_bitmap_free(totalcpuset);
 
     return ORTE_SUCCESS;
 }
@@ -377,7 +514,8 @@ static int bind_in_place(orte_job_t *jdata,
              */
             if (!support->cpubind->set_thisproc_cpubind &&
                 !support->cpubind->set_thisthread_cpubind) {
-                if (!OPAL_BINDING_REQUIRED(opal_hwloc_binding_policy)) {
+                if (!OPAL_BINDING_REQUIRED(opal_hwloc_binding_policy) ||
+                    !(OPAL_BIND_GIVEN & opal_hwloc_binding_policy)) {
                     /* we are not required to bind, so ignore this */
                     continue;
                 }
@@ -389,10 +527,12 @@ static int bind_in_place(orte_job_t *jdata,
              * expected. Per hwloc, Linux memory binding is at the thread,
              * and not process, level. Thus, hwloc sets the "thisproc" flag
              * to "false" on all Linux systems, and uses the "thisthread" flag
-             * to indicate binding capability
+             * to indicate binding capability - don't warn if the user didn't
+             * specifically request binding
              */
             if (!support->membind->set_thisproc_membind &&
-                !support->membind->set_thisthread_membind) {
+                !support->membind->set_thisthread_membind &&
+                (OPAL_BIND_GIVEN & opal_hwloc_binding_policy)) {
                 if (OPAL_HWLOC_BASE_MBFA_WARN == opal_hwloc_base_mbfa && !membind_warned) {
                     orte_show_help("help-orte-rmaps-base.txt", "rmaps:membind-not-supported", true, node->name);
                     membind_warned = true;
@@ -403,10 +543,26 @@ static int bind_in_place(orte_job_t *jdata,
             }
         }
 
-         /* clear the topology of any prior usage numbers */
-        opal_hwloc_base_clear_usage(node->topology);
+        /* some systems do not report cores, and so we can get a situation where our
+         * default binding policy will fail for no necessary reason. So if we are
+         * computing a binding due to our default policy, and no cores are found
+         * on this node, just silently skip it - we will not bind
+         */
+        if (!(OPAL_BIND_GIVEN & opal_hwloc_binding_policy) &&
+            HWLOC_TYPE_DEPTH_UNKNOWN == hwloc_get_type_depth(node->topology, HWLOC_OBJ_CORE)) {
+            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                "Unable to bind-to core by default on node %s as no cores detected",
+                                node->name);
+            continue;
+        }
 
-       /* cycle thru the procs */
+        /* we share topologies in order
+         * to save space, so we need to reset the usage info to reflect
+         * our own current state
+         */
+        reset_usage(node, jdata->jobid);
+
+        /* cycle thru the procs */
         for (j=0; j < node->procs->size; j++) {
             if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
                 continue;
@@ -437,18 +593,22 @@ static int bind_in_place(orte_job_t *jdata,
                 orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-available-cpus", true, node->name);
                 return ORTE_ERR_SILENT;
             }
-            /* error out if adding a proc would cause overload and that wasn't allowed */
+            /* error out if adding a proc would cause overload and that wasn't allowed,
+             * and it wasn't a default binding policy (i.e., the user requested it)
+             */
             if (ncpus < data->num_bound &&
-                !OPAL_BIND_OVERLOAD_ALLOWED(jdata->map->binding)) {
+                !OPAL_BIND_OVERLOAD_ALLOWED(jdata->map->binding) &&
+                (OPAL_BIND_GIVEN & opal_hwloc_binding_policy)) {
                 orte_show_help("help-orte-rmaps-base.txt", "rmaps:binding-overload", true,
                                opal_hwloc_base_print_binding(map->binding), node->name,
                                data->num_bound, ncpus);
                 return ORTE_ERR_SILENT;
             }
             /* bind the proc here */
-            proc->bind_idx = idx;
             cpus = opal_hwloc_base_get_available_cpus(node->topology, proc->locale);
             hwloc_bitmap_list_asprintf(&proc->cpu_bitmap, cpus);
+            /* record the location */
+            proc->bind_location = proc->locale;
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "%s BOUND PROC %s TO %s[%s:%u] on node %s",
                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -555,6 +715,11 @@ static int bind_to_cpuset(orte_job_t *jdata)
 
 int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
 {
+    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                        "mca:rmaps: compute bindings for job %s with policy %s",
+                        ORTE_JOBID_PRINT(jdata->jobid),
+                        opal_hwloc_base_print_binding(jdata->map->binding));
+
     if (ORTE_MAPPING_BYUSER == ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping)) {
         /* user specified binding by rankfile - nothing for us to do */
         return ORTE_SUCCESS;
@@ -569,8 +734,7 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
         return rc;
     }
 
-    if (!OPAL_BINDING_POLICY_IS_SET(jdata->map->binding) ||
-        OPAL_BIND_TO_NONE == OPAL_GET_BINDING_POLICY(jdata->map->binding)) {
+    if (OPAL_BIND_TO_NONE == OPAL_GET_BINDING_POLICY(jdata->map->binding)) {
         /* no binding requested */
         return ORTE_SUCCESS;
     }
@@ -638,8 +802,6 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
         return rc;
     } else if (OPAL_BIND_TO_HWTHREAD == OPAL_GET_BINDING_POLICY(jdata->map->binding)) {
         int rc;
-        /* record the level for locality purposes */
-        jdata->map->bind_level = OPAL_HWLOC_HWTHREAD_LEVEL;
         if (ORTE_MAPPING_BYHWTHREAD == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "mca:rmaps: bindings for job %s - hwthread to hwthread",
@@ -658,8 +820,6 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
         return rc;
     } else if (OPAL_BIND_TO_CORE == OPAL_GET_BINDING_POLICY(jdata->map->binding)) {
         int rc;
-        /* record the level for locality purposes */
-        jdata->map->bind_level = OPAL_HWLOC_CORE_LEVEL;
         if (ORTE_MAPPING_BYCORE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "mca:rmaps: bindings for job %s - core to core",
@@ -689,8 +849,6 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
         return rc;
     } else if (OPAL_BIND_TO_L1CACHE == OPAL_GET_BINDING_POLICY(jdata->map->binding)) {
         int rc;
-        /* record the level for locality purposes */
-        jdata->map->bind_level = OPAL_HWLOC_L1CACHE_LEVEL;
         if (ORTE_MAPPING_BYL1CACHE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "mca:rmaps: bindings for job %s - L1cache to L1cache",
@@ -715,8 +873,6 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
         return rc;
     } else if (OPAL_BIND_TO_L2CACHE == OPAL_GET_BINDING_POLICY(jdata->map->binding)) {
         int rc;
-        /* record the level for locality purposes */
-        jdata->map->bind_level = OPAL_HWLOC_L2CACHE_LEVEL;
         if (ORTE_MAPPING_BYL2CACHE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "mca:rmaps: bindings for job %s - L2cache to L2cache",
@@ -741,8 +897,6 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
         return rc;
     } else if (OPAL_BIND_TO_L3CACHE == OPAL_GET_BINDING_POLICY(jdata->map->binding)) {
         int rc;
-        /* record the level for locality purposes */
-        jdata->map->bind_level = OPAL_HWLOC_L3CACHE_LEVEL;
         if (ORTE_MAPPING_BYL3CACHE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "mca:rmaps: bindings for job %s - L3cache to L3cache",
@@ -767,8 +921,6 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
         return rc;
     } else if (OPAL_BIND_TO_SOCKET == OPAL_GET_BINDING_POLICY(jdata->map->binding)) {
         int rc;
-        /* record the level for locality purposes */
-        jdata->map->bind_level = OPAL_HWLOC_SOCKET_LEVEL;
         if (ORTE_MAPPING_BYSOCKET == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "mca:rmaps: bindings for job %s - socket to socket",
@@ -793,8 +945,6 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
         return rc;
     } else if (OPAL_BIND_TO_NUMA == OPAL_GET_BINDING_POLICY(jdata->map->binding)) {
         int rc;
-        /* record the level for locality purposes */
-        jdata->map->bind_level = OPAL_HWLOC_NUMA_LEVEL;
         if (ORTE_MAPPING_BYNUMA == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "mca:rmaps: bindings for job %s - numa to numa",
@@ -817,10 +967,9 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
             }
         }
         return rc;
-    } else {
-        ORTE_ERROR_LOG(ORTE_ERR_NOT_SUPPORTED);
-        return ORTE_ERR_NOT_SUPPORTED;
     }
 
-    return ORTE_SUCCESS;
+    /* if we get here, then the binding policy isn't supported */
+    ORTE_ERROR_LOG(ORTE_ERR_NOT_SUPPORTED);
+    return ORTE_ERR_NOT_SUPPORTED;
 }
