@@ -31,6 +31,8 @@
 #include <string.h>
 #endif  /* HAVE_STRING_H */
 
+#include "opal/mca/base/mca_base_var.h"
+
 #include "orte/util/show_help.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/util/error_strings.h"
@@ -62,9 +64,9 @@ static int mindist_map(orte_job_t *jdata)
     orte_node_t *node;
     orte_proc_t *proc;
     int nprocs_mapped;
-    int extra_procs, navg, nextra;
+    int extra_procs, navg, nextra=0;
     orte_std_cntr_t num_nodes, num_slots;
-    unsigned int npus, total_npus, num_procs_to_assign, required;
+    unsigned int npus, total_npus, num_procs_to_assign=0, required;
     int rc;
     mca_base_component_t *c = &mca_rmaps_mindist_component.base_version;
     bool initial_map=true;
@@ -157,7 +159,7 @@ static int mindist_map(orte_job_t *jdata)
          * option
          */
         if(ORTE_SUCCESS != (rc = orte_rmaps_base_get_target_nodes(&node_list, &num_slots, app,
-                                                                  jdata->map->mapping, initial_map))) {
+                                                                  jdata->map->mapping, initial_map, false))) {
             ORTE_ERROR_LOG(rc);
             goto error;
         }
@@ -210,17 +212,6 @@ static int mindist_map(orte_job_t *jdata)
                 goto error;
             }
             
-            /* add the node to the map, if needed */
-            if (!node->mapped) {
-                if (ORTE_SUCCESS > (rc = opal_pointer_array_add(jdata->map->nodes, (void*)node))) {
-                    ORTE_ERROR_LOG(rc);
-                    goto error;
-                }
-                node->mapped = true;
-                OBJ_RETAIN(node);  /* maintain accounting on object */
-                jdata->map->num_nodes++;
-            }
-
             /* get the number of available pus */
             if (opal_hwloc_use_hwthreads_as_cpus) {
                 total_npus = opal_hwloc_base_get_nbobjs_by_type(node->topology, HWLOC_OBJ_PU, 0, OPAL_HWLOC_AVAILABLE);
@@ -246,14 +237,16 @@ static int mindist_map(orte_job_t *jdata)
             OBJ_CONSTRUCT(&numa_list, opal_list_t);
             ret = opal_hwloc_get_sorted_numa_list(node->topology, orte_rmaps_base.device, &numa_list);
             if (ret > 1) {
-                orte_show_help("help-orte-rmaps-md.txt", "orte-rmaps-mindist:several-hca-devices",
-                        true, ret, node->name);
-                rc = ORTE_ERR_SILENT;
+                orte_show_help("help-orte-rmaps-md.txt", "orte-rmaps-mindist:several-devices",
+                               true, orte_rmaps_base.device, ret, node->name);
+                ORTE_SET_MAPPING_POLICY(jdata->map->mapping, ORTE_MAPPING_BYSLOT);
+                rc = ORTE_ERR_TAKE_NEXT_OPTION;
                 goto error;
             } else if (ret < 0) {
                 orte_show_help("help-orte-rmaps-md.txt", "orte-rmaps-mindist:device-not-found",
                         true, orte_rmaps_base.device, node->name);
-                rc = ORTE_ERR_SILENT;
+                ORTE_SET_MAPPING_POLICY(jdata->map->mapping, ORTE_MAPPING_BYSLOT);
+                rc = ORTE_ERR_TAKE_NEXT_OPTION;
                 goto error;
             }
             if (opal_list_get_size(&numa_list) > 0) {
@@ -291,7 +284,8 @@ static int mindist_map(orte_job_t *jdata)
                 /* don't have info about pci locality */
                 orte_show_help("help-orte-rmaps-md.txt", "orte-rmaps-mindist:no-pci-locality-info",
                         true, node->name);
-                rc = ORTE_ERR_SILENT;
+                ORTE_SET_MAPPING_POLICY(jdata->map->mapping, ORTE_MAPPING_BYSLOT);
+                rc = ORTE_ERR_TAKE_NEXT_OPTION;
                 goto error;
             }
             while (NULL != (numa_item = opal_list_remove_first(&numa_list))) {
@@ -304,6 +298,18 @@ static int mindist_map(orte_job_t *jdata)
                     num_procs_to_assign--;
                 }
             }
+            
+            /* add the node to the map, if needed */
+            if (!node->mapped) {
+                if (ORTE_SUCCESS > (rc = opal_pointer_array_add(jdata->map->nodes, (void*)node))) {
+                    ORTE_ERROR_LOG(rc);
+                    goto error;
+                }
+                node->mapped = true;
+                OBJ_RETAIN(node);  /* maintain accounting on object */
+                jdata->map->num_nodes++;
+            }
+
         }
 
         /* If we get to the end of all the nodes and still have procs remaining, then
