@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2013 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2011      Los Alamos National Security, LLC.
+ * Copyright (c) 2011-2012 Los Alamos National Security, LLC.
  *                         All rights reserved.
  * $COPYRIGHT$
  * 
@@ -117,7 +117,7 @@ static int orte_rmaps_rr_map(orte_job_t *jdata)
          * option
          */
         if(ORTE_SUCCESS != (rc = orte_rmaps_base_get_target_nodes(&node_list, &num_slots, app,
-                                                                  jdata->map->mapping, initial_map))) {
+                                                                  jdata->map->mapping, initial_map, false))) {
             ORTE_ERROR_LOG(rc);
             goto error;
         }
@@ -128,8 +128,10 @@ static int orte_rmaps_rr_map(orte_job_t *jdata)
         jdata->bookmark = orte_rmaps_base_get_starting_point(&node_list, jdata);
         
         if (0 == app->num_procs) {
-            /* set the num_procs to equal the number of slots on these mapped nodes */
-            app->num_procs = num_slots;
+            /* set the num_procs to equal the number of slots on these
+             * mapped nodes, taking into account the number of cpus/rank
+             */
+            app->num_procs = num_slots / orte_rmaps_base.cpus_per_rank;
         }
         
         /* Make assignments */
@@ -143,24 +145,80 @@ static int orte_rmaps_rr_map(orte_job_t *jdata)
         } else if (ORTE_MAPPING_BYHWTHREAD == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             rc = orte_rmaps_rr_byobj(jdata, app, &node_list, num_slots,
                                      app->num_procs, HWLOC_OBJ_PU, 0);
+            if (ORTE_ERR_NOT_SUPPORTED == rc) {
+                /* if the mapper couldn't map by this object because
+                 * it isn't available, but the error allows us to try
+                 * byslot, then do so
+                 */
+                rc = orte_rmaps_rr_byslot(jdata, app, &node_list, num_slots,
+                                          app->num_procs);
+            }
         } else if (ORTE_MAPPING_BYCORE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             rc = orte_rmaps_rr_byobj(jdata, app, &node_list, num_slots,
                                      app->num_procs, HWLOC_OBJ_CORE, 0);
+            if (ORTE_ERR_NOT_SUPPORTED == rc) {
+                /* if the mapper couldn't map by this object because
+                 * it isn't available, but the error allows us to try
+                 * byslot, then do so
+                 */
+                rc = orte_rmaps_rr_byslot(jdata, app, &node_list, num_slots,
+                                          app->num_procs);
+            }
         } else if (ORTE_MAPPING_BYL1CACHE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             rc = orte_rmaps_rr_byobj(jdata, app, &node_list, num_slots,
                                      app->num_procs, HWLOC_OBJ_CACHE, 1);
+            if (ORTE_ERR_NOT_SUPPORTED == rc) {
+                /* if the mapper couldn't map by this object because
+                 * it isn't available, but the error allows us to try
+                 * byslot, then do so
+                 */
+                rc = orte_rmaps_rr_byslot(jdata, app, &node_list, num_slots,
+                                          app->num_procs);
+            }
         } else if (ORTE_MAPPING_BYL2CACHE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             rc = orte_rmaps_rr_byobj(jdata, app, &node_list, num_slots,
                                      app->num_procs, HWLOC_OBJ_CACHE, 2);
+            if (ORTE_ERR_NOT_SUPPORTED == rc) {
+                /* if the mapper couldn't map by this object because
+                 * it isn't available, but the error allows us to try
+                 * byslot, then do so
+                 */
+                rc = orte_rmaps_rr_byslot(jdata, app, &node_list, num_slots,
+                                          app->num_procs);
+            }
         } else if (ORTE_MAPPING_BYL3CACHE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             rc = orte_rmaps_rr_byobj(jdata, app, &node_list, num_slots,
                                      app->num_procs, HWLOC_OBJ_CACHE, 3);
+            if (ORTE_ERR_NOT_SUPPORTED == rc) {
+                /* if the mapper couldn't map by this object because
+                 * it isn't available, but the error allows us to try
+                 * byslot, then do so
+                 */
+                rc = orte_rmaps_rr_byslot(jdata, app, &node_list, num_slots,
+                                          app->num_procs);
+            }
         } else if (ORTE_MAPPING_BYSOCKET == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             rc = orte_rmaps_rr_byobj(jdata, app, &node_list, num_slots,
                                      app->num_procs, HWLOC_OBJ_SOCKET, 0);
+            if (ORTE_ERR_NOT_SUPPORTED == rc) {
+                /* if the mapper couldn't map by this object because
+                 * it isn't available, but the error allows us to try
+                 * byslot, then do so
+                 */
+                rc = orte_rmaps_rr_byslot(jdata, app, &node_list, num_slots,
+                                          app->num_procs);
+            }
         } else if (ORTE_MAPPING_BYNUMA == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
             rc = orte_rmaps_rr_byobj(jdata, app, &node_list, num_slots,
                                      app->num_procs, HWLOC_OBJ_NODE, 0);
+            if (ORTE_ERR_NOT_SUPPORTED == rc) {
+                /* if the mapper couldn't map by this object because
+                 * it isn't available, but the error allows us to try
+                 * byslot, then do so
+                 */
+                rc = orte_rmaps_rr_byslot(jdata, app, &node_list, num_slots,
+                                          app->num_procs);
+            }
 #endif
         } else {
             /* unrecognized mapping directive */
