@@ -10,6 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2009-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -56,7 +57,7 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
                         ORTE_JOBID_PRINT(jdata->jobid), (int)num_slots, (unsigned long)num_procs);
 
     /* check to see if we can map all the procs */
-    if (num_slots < (int)app->num_procs) {
+    if (num_slots < ((int)app->num_procs * orte_rmaps_base.cpus_per_rank)) {
         if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
             orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
                            true, app->num_procs, app->app);
@@ -77,19 +78,27 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
                             node->name);
 #if OPAL_HAVE_HWLOC
         /* get the root object as we are not assigning
-         * locale except at the node level
+         * locale here except at the node level
          */
         if (NULL != node->topology) {
             obj = hwloc_get_root_obj(node->topology);
         }
 #endif
-        if (node->slots_alloc <= node->slots_inuse) {
+        if (node->slots <= node->slots_inuse) {
             opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
-                                "mca:rmaps:rr:slot working node %s is full - skipping",
+                                "mca:rmaps:rr:slot node %s is full - skipping",
                                 node->name);
             continue;
         }
-        num_procs_to_assign = node->slots_alloc - node->slots_inuse;
+        /* assign a number of procs equal to the number of available
+         * slots divided by the number of cpus/rank the user
+         * requested
+         */
+        num_procs_to_assign = (node->slots - node->slots_inuse) / orte_rmaps_base.cpus_per_rank;
+        opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
+                            "mca:rmaps:rr:slot assigning %d procs to node %s",
+                            (int)num_procs_to_assign, node->name);
+
         for (i=0; i < num_procs_to_assign && nprocs_mapped < app->num_procs; i++) {
             /* add this node to the map - do it only once */
             if (!node->mapped) {
@@ -170,7 +179,7 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
                 --nxtra_nodes;
             }
         }
-        num_procs_to_assign = (node->slots_alloc - node->slots_inuse) + extra_procs_to_assign;
+        num_procs_to_assign = ((node->slots - node->slots_inuse)/orte_rmaps_base.cpus_per_rank) + extra_procs_to_assign;
         opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
                             "mca:rmaps:rr:slot adding up to %d procs to node %s",
                             num_procs_to_assign, node->name);
@@ -186,7 +195,7 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
         /* not all nodes are equal, so only set oversubscribed for
          * this node if it is in that state
          */
-        if (node->slots_alloc < (int)node->num_procs) {
+        if (node->slots < (int)node->num_procs) {
             /* flag the node as oversubscribed so that sched-yield gets
              * properly set
              */
@@ -221,7 +230,7 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
                         (int)num_slots, (unsigned long)num_procs);
 
     /* quick check to see if we can map all the procs */
-    if (num_slots < (int)app->num_procs) {
+    if (num_slots < ((int)app->num_procs * orte_rmaps_base.cpus_per_rank)) {
         if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
             orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
                            true, app->num_procs, app->app);
@@ -306,7 +315,7 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
              * have to track how many procs to "shift" elsewhere
              * to make up the difference
              */
-            if (node->slots_alloc <= node->slots_inuse) {
+            if (node->slots <= node->slots_inuse) {
                 /* if there are no extras to take, then we can
                  * ignore this node
                  */
@@ -320,23 +329,25 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
                 /* update how many we are lagging behind */
                 lag += navg;
             } else {
-                /* if slots_alloc < avg, then take all */
-                if ((node->slots_alloc - node->slots_inuse) < navg) {
-                    num_procs_to_assign = (node->slots_alloc - node->slots_inuse) + extra_procs_to_assign;
+                /* if slots < avg (adjusted for cpus/proc), then take all */
+                if ((node->slots - node->slots_inuse) < (navg * orte_rmaps_base.cpus_per_rank)) {
+                    num_procs_to_assign = (node->slots - node->slots_inuse)/orte_rmaps_base.cpus_per_rank;
                     /* update how many we are lagging behind */
-                    lag += navg - (node->slots_alloc - node->slots_inuse);
+                    lag += navg - num_procs_to_assign;
                 } else {
                     /* take the avg plus as much of the "lag" as we can */
                     delta = 0;
                     if (0 < lag) {
-                        delta = (node->slots_alloc - node->slots_inuse) - navg;
+                        delta = ((node->slots - node->slots_inuse)/orte_rmaps_base.cpus_per_rank) - navg;
                         if (lag < delta) {
                             delta = lag;
                         }
                         lag -= delta;
                     }
-                    num_procs_to_assign = navg + delta + extra_procs_to_assign;
+                    num_procs_to_assign = navg;
                 }
+                /* add in the extras */
+                num_procs_to_assign += extra_procs_to_assign;
             }
         }
         for (j=0; j < num_procs_to_assign && nprocs_mapped < app->num_procs; j++) {
@@ -351,7 +362,7 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
         /* not all nodes are equal, so only set oversubscribed for
          * this node if it is in that state
          */
-        if (node->slots_alloc < (int)node->num_procs) {
+        if (node->slots < ((int)node->num_procs * orte_rmaps_base.cpus_per_rank)) {
             /* flag the node as oversubscribed so that sched-yield gets
              * properly set
              */
@@ -381,7 +392,7 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
         /* not all nodes are equal, so only set oversubscribed for
          * this node if it is in that state
          */
-        if (node->slots_alloc < (int)node->num_procs) {
+        if (node->slots < ((int)node->num_procs * orte_rmaps_base.cpus_per_rank)) {
             /* flag the node as oversubscribed so that sched-yield gets
              * properly set
              */
@@ -440,7 +451,7 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
      * to the next node. Thus, procs tend to be "front loaded" onto the
      * list of nodes, as opposed to being "load balanced" in the span mode
      */
-    if (ORTE_MAPPING_SPAN & jdata->map->mapping) {
+    if (ORTE_MAPPING_SPAN & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
         return byobj_span(jdata, app, node_list, num_slots,
                           num_procs, target, cache_level);
     }
@@ -455,24 +466,29 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
      * do more because we don't know how many total objects exist
      * across all the nodes
      */
-    if (num_slots < (int)app->num_procs) {
+    if (num_slots < (app->num_procs * orte_rmaps_base.cpus_per_rank)) {
         if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
             orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
                            true, app->num_procs, app->app);
             return ORTE_ERR_SILENT;
         }
         /* compute how many extra procs to put on each node */
-        balance = (float)((jdata->num_procs + app->num_procs) - num_slots) / (float)opal_list_get_size(node_list);
-        extra_procs_to_assign = (int)balance;
-        if (0 < (balance - (float)extra_procs_to_assign)) {
-            /* compute how many nodes need an extra proc */
-            nxtra_nodes = (jdata->num_procs + app->num_procs) - num_slots - (extra_procs_to_assign * opal_list_get_size(node_list));
-            /* add one so that we add an extra proc to the first nodes
-             * until all procs are mapped
-             */
-            extra_procs_to_assign++;
-            /* flag that we added one */
-            add_one = true;
+        if (1 == opal_list_get_size(node_list)) {
+            /* if there is only one node, then they all have to go on it */
+            extra_procs_to_assign = app->num_procs;
+        } else {
+            balance = (float)(((jdata->num_procs + app->num_procs)*orte_rmaps_base.cpus_per_rank) - num_slots) / (float)opal_list_get_size(node_list);
+            extra_procs_to_assign = (int)balance;
+            if (0 < (balance - (float)extra_procs_to_assign)) {
+                /* compute how many nodes need an extra proc */
+                nxtra_nodes = ((jdata->num_procs + app->num_procs)*orte_rmaps_base.cpus_per_rank) - num_slots - (extra_procs_to_assign * opal_list_get_size(node_list));
+                /* add one so that we add an extra proc to the first nodes
+                 * until all procs are mapped
+                 */
+                extra_procs_to_assign++;
+                /* flag that we added one */
+                add_one = true;
+            }
         }
     }
 
@@ -492,6 +508,39 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
                            true, node->name);
             return ORTE_ERR_SILENT;
         }
+
+        /* compute the number of procs to go on this node */
+        if (add_one) {
+            if (0 == nxtra_nodes) {
+                --extra_procs_to_assign;
+                add_one = false;
+            } else {
+                --nxtra_nodes;
+            }
+        }
+        if (node->slots <= node->slots_inuse) {
+            /* everybody takes at least the extras */
+            num_procs_to_assign = extra_procs_to_assign;
+        } else {
+            num_procs_to_assign = (node->slots - node->slots_inuse)/orte_rmaps_base.cpus_per_rank + extra_procs_to_assign;
+            if (app->num_procs < num_procs_to_assign) {
+                /* might have more slots than procs */
+                num_procs_to_assign = app->num_procs;
+            }
+        }
+
+        /* if this would oversubscribe the node and the user hasn't permitted
+         * oversubscription, then don't use it - since the total number of
+         * slots is adequate for this app, there should be room somewhere else
+         */
+        if (node->slots < (node->slots_inuse + num_procs_to_assign) &&
+            ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
+            opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
+                                "mca:rmaps:rr: mapping no-span would oversubscribe node %s - ignoring it",
+                                node->name);
+            continue;
+        }
+
         /* add this node to the map, if reqd */
         if (!node->mapped) {
             if (ORTE_SUCCESS > (idx = opal_pointer_array_add(jdata->map->nodes, (void*)node))) {
@@ -503,34 +552,48 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
             ++(jdata->map->num_nodes);
         }
 
-         /* compute the number of procs to go on this node */
-        if (add_one) {
-            if (0 == nxtra_nodes) {
-                --extra_procs_to_assign;
-                add_one = false;
-            } else {
-                --nxtra_nodes;
-            }
-        }
-        if (node->slots_alloc <= node->slots_inuse) {
-            /* everybody takes at least the extras */
-            num_procs_to_assign = extra_procs_to_assign;
-        } else {
-            num_procs_to_assign = (node->slots_alloc - node->slots_inuse) + extra_procs_to_assign;
-        }
-
         /* get the number of objects of this type on this node */
         nobjs = opal_hwloc_base_get_nbobjs_by_type(node->topology, target, cache_level, OPAL_HWLOC_AVAILABLE);
         opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
-                            "mca:rmaps:rr:byobj: found %d objs on node %s", nobjs, node->name);
+                            "mca:rmaps:rr:byobj: nprocs-to-assign %d for %d objs on node %s", num_procs_to_assign, nobjs, node->name);
         /* if there are no objects of this type, then report the error
          * and abort - this can happen, for example, on systems that
-         * don't report "sockets" as an independent object
+         * don't report "sockets" as an independent object. However, IF
+         * this object is the default one - i.e., not specified by the
+         * user - then we can fall back to mapping by slot
          */
         if (0 == nobjs) {
-            orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:no-objects",
-                           true,  hwloc_obj_type_string(target), node->name);
-            return ORTE_ERR_SILENT;
+            if (ORTE_MAPPING_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping)) {
+                orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:no-objects",
+                               true,  hwloc_obj_type_string(target), node->name);
+                return ORTE_ERR_SILENT;
+            } else {
+                /* this was the default mapping policy, so clear the map
+                 * of any prior work and indicate that map-by slot is reqd
+                 */
+                for (i=0; i < jdata->map->nodes->size; i++) {
+                    if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(jdata->map->nodes, i))) {
+                        continue;
+                    }
+                    for (idx=0; idx < node->procs->size; idx++) {
+                        if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, idx))) {
+                            continue;
+                        }
+                        if (proc->name.jobid != jdata->jobid) {
+                            continue;
+                        }
+                        --node->num_procs;
+                        OBJ_RELEASE(proc);
+                        opal_pointer_array_set_item(node->procs, idx, NULL);
+                    }
+                    if (0 == node->num_procs) {
+                        node->mapped = false;
+                        OBJ_RELEASE(node);
+                        opal_pointer_array_set_item(jdata->map->nodes, i, NULL);
+                    }
+                }
+                return ORTE_ERR_NOT_SUPPORTED;
+            }
         }
 
         /* compute the number of procs to go on each object */
@@ -568,7 +631,7 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
         /* not all nodes are equal, so only set oversubscribed for
          * this node if it is in that state
          */
-        if (node->slots_alloc < (int)node->num_procs) {
+        if (node->slots < (int)node->num_procs) {
             /* flag the node as oversubscribed so that sched-yield gets
              * properly set
              */
@@ -697,7 +760,7 @@ static int byobj_span(orte_job_t *jdata,
              * have to track how many procs to "shift" elsewhere
              * to make up the difference
              */
-            if (node->slots_alloc <= node->slots_inuse) {
+            if (node->slots <= node->slots_inuse) {
                 /* if there are no extras to take, then we can
                  * safely remove this node as we don't need it
                  */
@@ -714,16 +777,16 @@ static int byobj_span(orte_job_t *jdata,
                 /* update how many we are lagging behind */
                 lag += navg;
             } else {
-                /* if slots_alloc < avg, then take all */
-                if ((node->slots_alloc - node->slots_inuse) < navg) {
-                    num_procs_to_assign = (node->slots_alloc - node->slots_inuse) + extra_procs_to_assign;
+                /* if slots < avg, then take all */
+                if ((node->slots - node->slots_inuse) < navg) {
+                    num_procs_to_assign = (node->slots - node->slots_inuse) + extra_procs_to_assign;
                     /* update how many we are lagging behind */
-                    lag += navg - (node->slots_alloc - node->slots_inuse);
+                    lag += navg - (node->slots - node->slots_inuse);
                 } else {
                     /* take the avg plus as much of the "lag" as we can */
                     delta = 0;
                     if (0 < lag) {
-                        delta = (node->slots_alloc - node->slots_inuse) - navg;
+                        delta = (node->slots - node->slots_inuse) - navg;
                         if (lag < delta) {
                             delta = lag;
                         }
@@ -775,7 +838,7 @@ static int byobj_span(orte_job_t *jdata,
         /* not all nodes are equal, so only set oversubscribed for
          * this node if it is in that state
          */
-        if (node->slots_alloc < (int)node->num_procs) {
+        if (node->slots < (int)node->num_procs) {
             /* flag the node as oversubscribed so that sched-yield gets
              * properly set
              */
