@@ -44,6 +44,11 @@
 
 BEGIN_C_DECLS
 
+/* Convenience def for readability */
+#define ORTE_RML_PERSISTENT      true
+#define ORTE_RML_NON_PERSISTENT  false
+
+
 /**
  * Constant tag values for well-known services
  */
@@ -95,23 +100,19 @@ BEGIN_C_DECLS
 
 /* collectives */
 #define ORTE_RML_TAG_COLLECTIVE             30
-#define ORTE_RML_TAG_COLL_ID                50
-#define ORTE_RML_TAG_DAEMON_COLL            52
-#define ORTE_RML_TAG_COLL_ID_REQ            53
+#define ORTE_RML_TAG_COLL_ID                31
+#define ORTE_RML_TAG_DAEMON_COLL            32
+#define ORTE_RML_TAG_COLL_ID_REQ            33
 
 /* show help */
-#define ORTE_RML_TAG_SHOW_HELP              31
+#define ORTE_RML_TAG_SHOW_HELP              34
 
 /* debugger release */
-#define ORTE_RML_TAG_DEBUGGER_RELEASE       32
+#define ORTE_RML_TAG_DEBUGGER_RELEASE       35
 
 /* bootstrap */
-#define ORTE_RML_TAG_BOOTSTRAP              34
+#define ORTE_RML_TAG_BOOTSTRAP              36
 
-/* TCP "fake" multicast */
-#define ORTE_RML_TAG_MULTICAST              35
-/* multicast messages sent direct */
-#define ORTE_RML_TAG_MULTICAST_DIRECT       36
 /* report a missed msg */
 #define ORTE_RML_TAG_MISSED_MSG             37
 
@@ -122,21 +123,31 @@ BEGIN_C_DECLS
 #define ORTE_RML_TAG_HEARTBEAT              39
 
 /* Process Migration Tool Tag */
-#define ORTE_RML_TAG_MIGRATE                43
+#define ORTE_RML_TAG_MIGRATE                40
 
 /* For SStore Framework */
-#define ORTE_RML_TAG_SSTORE                 44
-#define ORTE_RML_TAG_SSTORE_INTERNAL        45
+#define ORTE_RML_TAG_SSTORE                 41
+#define ORTE_RML_TAG_SSTORE_INTERNAL        42
 
-#define ORTE_RML_TAG_SUBSCRIBE              46
+#define ORTE_RML_TAG_SUBSCRIBE              43
 
 
 /* Notify of failed processes */
-#define ORTE_RML_TAG_FAILURE_NOTICE         48
+#define ORTE_RML_TAG_FAILURE_NOTICE         44
+
+/* distributed file system */
+#define ORTE_RML_TAG_DFS_CMD                45
+#define ORTE_RML_TAG_DFS_DATA               46
+
+/* sensor data */
+#define ORTE_RML_TAG_SENSOR_DATA            47
 
 #define ORTE_RML_TAG_MAX                   100
 
 
+#define ORTE_RML_TAG_NTOH(t) ntohl(t)
+#define ORTE_RML_TAG_HTON(t) htonl(t)
+
 /** 
  * Message matching tag
  *
@@ -159,47 +170,6 @@ typedef uint8_t orte_rml_cmd_flag_t;
 #define ORTE_RML_UPDATE_CMD    1
 
 
-/* ******************************************************************** */
-/* Flags to send/recv */
-
-/**
- * Non-persistent request that can be deleted when the request is
- * completed.  This is the default behavior.
- */
-#define ORTE_RML_NON_PERSISTENT          0x00000000
-
-/**
- * flag to oob_recv to allow caller to peek a portion of the next
- * available message w/out removing the message from the queue.
- */
-#define ORTE_RML_PEEK                    0x00000001
-
-/** 
- * flag to oob_recv to return the actual size of the message even if
- * the receive buffer is smaller than the number of bytes available 
- */
-#define ORTE_RML_TRUNC                   0x00000002
-
-/** 
- * flag to oob_recv to request the oob to allocate a buffer of the
- * appropriate size for the receive and return the allocated buffer
- * and size in the first element of the iovec array.
- */
-#define ORTE_RML_ALLOC                   0x00000004
-
-/**
- * posted non-blocking recv is persistent 
- */
-#define ORTE_RML_PERSISTENT              0x00000008
-
-/**
- * The request is a non-blocking request that can have its callback
- * triggered as soon as the request is completed, even if the OOB is
- * currently in the middle of another non-blocking request callback.
- */
-#define ORTE_RML_FLAG_RECURSIVE_CALLBACK 0x00000010
-
-
 typedef enum {
     ORTE_RML_PEER_UNREACH,
     ORTE_RML_PEER_DISCONNECTED
