@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2007-2011 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2007-2012 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
  * Copyright (c) 2009      Sun Microsystems, Inc. All rights reserved.
  * Copyright (c) 2010-2011 Oak Ridge National Labs.  All rights reserved.
@@ -83,9 +83,6 @@
  */
 static char *get_orted_comm_cmd_str(int command);
 
-/* instantiate this - it is shared via orted.h */
-struct timeval orte_daemon_msg_recvd;
-
 static opal_pointer_array_t *procs_prev_ordered_to_terminate = NULL;
 
 void orte_daemon_recv(int status, orte_process_name_t* sender,
@@ -414,7 +411,7 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
                 }
             } else {
                 /* just deliver it to ourselves */
-                if ((ret = orte_rml.send_buffer_nb(ORTE_PROC_MY_NAME, relay_msg, target_tag, 0,
+                if ((ret = orte_rml.send_buffer_nb(ORTE_PROC_MY_NAME, relay_msg, target_tag,
                                                    orte_rml_send_callback, NULL)) < 0) {
                     ORTE_ERROR_LOG(ret);
                     OBJ_RELEASE(relay_msg);
@@ -501,7 +498,7 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
             goto CLEANUP;
         }
         /* return response */
-        if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
+        if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL,
                                                orte_rml_send_callback, NULL))) {
             ORTE_ERROR_LOG(ret);
             OBJ_RELEASE(answer);
@@ -531,7 +528,7 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
             goto CLEANUP;
         }
             
-        if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
+        if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL,
                                                orte_rml_send_callback, NULL))) {
             ORTE_ERROR_LOG(ret);
             OBJ_RELEASE(answer);
@@ -556,7 +553,7 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
                 OBJ_RELEASE(answer);
                 goto CLEANUP;
             }
-            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
+            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL,
                                                    orte_rml_send_callback, NULL))) {
                 ORTE_ERROR_LOG(ret);
                 OBJ_RELEASE(answer);
@@ -629,7 +626,7 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
                     }
                 }
             }
-            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
+            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL,
                                                    orte_rml_send_callback, NULL))) {
                 ORTE_ERROR_LOG(ret);
                 OBJ_RELEASE(answer);
@@ -655,7 +652,7 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
                 OBJ_RELEASE(answer);
                 goto CLEANUP;
             }
-            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
+            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL,
                                                    orte_rml_send_callback, NULL))) {
                 ORTE_ERROR_LOG(ret);
                 OBJ_RELEASE(answer);
@@ -724,7 +721,7 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
                 }
             }
             /* send the info */
-            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
+            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL,
                                                    orte_rml_send_callback, NULL))) {
                 ORTE_ERROR_LOG(ret);
                 OBJ_RELEASE(answer);
@@ -750,7 +747,7 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
                 OBJ_RELEASE(answer);
                 goto CLEANUP;
             }
-            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
+            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL,
                                                    orte_rml_send_callback, NULL))) {
                 ORTE_ERROR_LOG(ret);
                 OBJ_RELEASE(answer);
@@ -761,7 +758,8 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
             orte_proc_t *proc;
             orte_vpid_t vpid;
             int32_t i, num_procs;
-                
+            char *nid;
+
             /* setup the answer */
             answer = OBJ_NEW(opal_buffer_t);
                 
@@ -804,8 +802,25 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
                             ORTE_ERROR_LOG(ret);
                             goto CLEANUP;
                         }
+                        /* the vpid and nodename for this proc are no longer packed
+                         * in the ORTE_PROC packing routines to save space for other
+                         * uses, so we have to pack them separately
+                         */
+                        if (ORTE_SUCCESS != (ret = opal_dss.pack(answer, &proc->pid, 1, OPAL_PID))) {
+                            ORTE_ERROR_LOG(ret);
+                            goto CLEANUP;
+                        }
+                        if (NULL == proc->node) {
+                            nid = "UNKNOWN";
+                        } else {
+                            nid = proc->node->name;
+                        }
+                        if (ORTE_SUCCESS != (ret = opal_dss.pack(answer, &nid, 1, OPAL_STRING))) {
+                            ORTE_ERROR_LOG(ret);
+                            goto CLEANUP;
+                        }
                         break;
-                    }
+                    }                    
                 }
             } else {
                 /* count number of procs */
@@ -829,11 +844,28 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
                             OBJ_RELEASE(answer);
                             goto CLEANUP;
                         }
+                        /* the vpid and nodename for this proc are no longer packed
+                         * in the ORTE_PROC packing routines to save space for other
+                         * uses, so we have to pack them separately
+                         */
+                        if (ORTE_SUCCESS != (ret = opal_dss.pack(answer, &proc->pid, 1, OPAL_PID))) {
+                            ORTE_ERROR_LOG(ret);
+                            goto CLEANUP;
+                        }
+                        if (NULL == proc->node) {
+                            nid = "UNKNOWN";
+                        } else {
+                            nid = proc->node->name;
+                        }
+                        if (ORTE_SUCCESS != (ret = opal_dss.pack(answer, &nid, 1, OPAL_STRING))) {
+                            ORTE_ERROR_LOG(ret);
+                            goto CLEANUP;
+                        }
                     }
                 }
             }
             /* send the info */
-            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL, 0,
+            if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_TOOL,
                                                    orte_rml_send_callback, NULL))) {
                 ORTE_ERROR_LOG(ret);
                 OBJ_RELEASE(answer);
@@ -916,7 +948,9 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
                             goto SEND_TOP_ANSWER;
                         }
                         /* the callback function will release relay_msg buffer */
-                        if (0 > orte_rml.send_buffer(&proc2, relay_msg, ORTE_RML_TAG_DAEMON, 0)) {
+                        if (0 > orte_rml.send_buffer_nb(&proc2, relay_msg,
+                                                        ORTE_RML_TAG_DAEMON,
+                                                        orte_rml_send_callback, NULL)) {
                             ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
                             OBJ_RELEASE(relay_msg);
                             ret = ORTE_ERR_COMM_FAILURE;
@@ -965,7 +999,9 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
                         goto SEND_TOP_ANSWER;
                     }
                     /* the callback function will release relay_msg buffer */
-                    if (0 > orte_rml.send_buffer(&proc2, relay_msg, ORTE_RML_TAG_DAEMON, 0)) {
+                    if (0 > orte_rml.send_buffer_nb(&proc2, relay_msg,
+                                                    ORTE_RML_TAG_DAEMON,
+                                                    orte_rml_send_callback, NULL)) {
                         ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
                         OBJ_RELEASE(relay_msg);
                         ret = ORTE_ERR_COMM_FAILURE;
@@ -1027,7 +1063,7 @@ void orte_daemon_recv(int status, orte_process_name_t* sender,
             ret = ORTE_ERR_COMM_FAILURE;
             break;
         }
-        if (0 > (ret = orte_rml.send_buffer_nb(return_addr, answer, ORTE_RML_TAG_TOOL, 0,
+        if (0 > (ret = orte_rml.send_buffer_nb(return_addr, answer, ORTE_RML_TAG_TOOL,
                                                orte_rml_send_callback, NULL))) {
             ORTE_ERROR_LOG(ret);
             OBJ_RELEASE(answer);
