@@ -11,6 +11,8 @@
  *                         All rights reserved.
  * Copyright (c) 2007-2011 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved. 
+ * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
+ *                         All rights reserved
  * Copyright (c) 2013      Intel, Inc. All rights reserved
  * $COPYRIGHT$
  * 
@@ -40,6 +42,7 @@
 #include "orte/runtime/orte_globals.h"
 
 static bool passed_thru = false;
+static int orte_progress_thread_debug_level = -1;
 static char *orte_timing_file = NULL;
 static char *orte_xml_file = NULL;
 static char *orte_fork_agent_string = NULL;
@@ -49,6 +52,9 @@ static char *orte_remote_tmpdir_base = NULL;
 
 int orte_register_params(void)
 {
+    int id;
+    opal_output_stream_t lds;
+
     /* only go thru this once - mpirun calls it twice, which causes
      * any error messages to show up twice
      */
@@ -57,20 +63,15 @@ int orte_register_params(void)
     }
     passed_thru = true;
     
-#if !ORTE_DISABLE_FULL_SUPPORT
     /* get a clean output channel too - need to do this here because
      * we use it below, and orterun and some other tools call this
      * function prior to calling orte_init
      */
-    {
-        opal_output_stream_t lds;
-        OBJ_CONSTRUCT(&lds, opal_output_stream_t);
-        lds.lds_want_stdout = true;
-        orte_clean_output = opal_output_open(&lds);
-        OBJ_DESTRUCT(&lds);
-    }
-#endif /* !ORTE_DISABLE_FULL_SUPPORT */
-
+    OBJ_CONSTRUCT(&lds, opal_output_stream_t);
+    lds.lds_want_stdout = true;
+    orte_clean_output = opal_output_open(&lds);
+    OBJ_DESTRUCT(&lds);
+    
     orte_help_want_aggregate = true;
     (void) mca_base_var_register ("orte", "orte", "base", "help_aggregate",
                                   "If orte_base_help_aggregate is true, duplicate help messages will be aggregated rather than displayed individually.  This can be helpful for parallel jobs that experience multiple identical failures; rather than print out the same help/failure message N times, display it once with a count of how many processes sent the same message.",
@@ -176,8 +177,6 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL,
                                   &orte_report_silent_errors);
 
-#if !ORTE_DISABLE_FULL_SUPPORT
-
     orte_debug_flag = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "debug",
                                   "Top-level ORTE debug switch (default: false)",
@@ -218,6 +217,20 @@ int orte_register_params(void)
                                       OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL,
                                       &orte_debug_daemons_flag);
     }
+
+    orte_progress_thread_debug_level = -1;
+    (void) mca_base_var_register ("orte", "orte", NULL, "progress_thread_debug",
+                                  "Debug level for ORTE progress threads",
+                                  MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL,
+                                  &orte_progress_thread_debug_level);
+
+    if (0 <= orte_progress_thread_debug_level) {
+        orte_progress_thread_debug = opal_output_open(NULL);
+        opal_output_set_verbosity(orte_progress_thread_debug,
+                                  orte_progress_thread_debug_level);
+    }
+
     /* do we want session output left open? */
     orte_leave_session_attached = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "leave_session_attached",
@@ -301,7 +314,7 @@ int orte_register_params(void)
 
     orte_startup_timeout = 0;
     (void) mca_base_var_register ("orte", "orte", NULL, "startup_timeout",
-                                  "Seconds/daemon to wait for startup before declaring failed_to_start (default: 0 => do not check)",
+                                  "Seconds to wait for startup or job launch before declaring failed_to_start (default: 0 => do not check)",
                                   MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_startup_timeout);
@@ -389,14 +402,6 @@ int orte_register_params(void)
         orte_default_hostfile_given = true;
     }
     
-#ifdef __WINDOWS__
-    mca_base_param_reg_string_name("orte", "ccp_headnode",
-                                   "Name of the cluster head node. (For Windows CCP only.)",
-                                   false, false,
-                                   NULL, &orte_ccp_headnode);
-#endif
-    
-
     /* regex of nodes in system */
     orte_node_regex = NULL;
     (void) mca_base_var_register ("orte", "orte", NULL, "node_regex",
@@ -654,6 +659,20 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_abort_non_zero_exit);
 
+    orte_allowed_exit_without_sync = false;
+    (void) mca_base_var_register ("orte", "orte", NULL, "allowed_exit_without_sync",
+                                  "Process exiting without calling finalize will not trigger job termination",
+                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_allowed_exit_without_sync);
+
+    orte_staged_execution = false;
+    (void) mca_base_var_register ("orte", "orte", NULL, "staged_execution",
+                                  "Staged execution is being used",
+                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_staged_execution);
+
     orte_report_child_jobs_separately = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "report_child_jobs_separately",
                                   "Return the exit status of the primary job only",
@@ -693,7 +712,46 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_max_vm_size);
 
-#endif /* ORTE_DISABLE_FULL_SUPPORT */
-    
+    orte_set_slots = NULL;
+    (void) mca_base_var_register ("orte", "orte", NULL, "set_default_slots",
+                                  "Set the number of slots on nodes that lack such info to the number of specified objects [a number, \"cores\", \"numas\", \"sockets\", or \"hwthreads\"]",
+                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_set_slots);
+
+    /* should we display the allocation after determining it? */
+    orte_display_allocation = false;
+    id = mca_base_var_register ("orte", "orte", NULL, "display_alloc",
+                                "Whether to display the allocation after it is determined",
+                                MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                &orte_display_allocation);
+    /* register a synonym for old name -- should we remove this now? */
+    mca_base_var_register_synonym (id, "orte", "ras", "base", "display_alloc", MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
+
+    /* should we display a detailed (developer-quality) version of the allocation after determining it? */
+    orte_devel_level_output = false;
+    id = mca_base_var_register ("orte", "orte", NULL, "display_devel_alloc",
+                                "Whether to display a developer-detail allocation after it is determined",
+                                MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                &orte_devel_level_output);
+    /* register a synonym for old name -- should we remove this now? */
+    mca_base_var_register_synonym (id, "orte", "ras", "base", "display_devel_alloc", MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
+
+    if (orte_devel_level_output) {
+        orte_display_allocation = true;
+    }
+
+    /* should we treat any -host directives as "soft" - i.e., desired
+     * but not required
+     */
+    orte_soft_locations = false;
+    (void) mca_base_var_register ("orte", "orte", NULL, "soft_locations",
+                                  "Treat -host directives as desired, but not required",
+                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_soft_locations);
+
     return ORTE_SUCCESS;
 }
