@@ -193,6 +193,9 @@ int orte_err2str(int errnum, const char **errmsg)
     case ORTE_ERR_ALLOCATION_PENDING:
         retval = "Allocation pending";
         break;
+    case ORTE_ERR_NO_PATH_TO_TARGET:
+        retval = "No OOB path to target";
+        break;
     default:
         if (orte_report_silent_errors) {
             retval = "Unknown error";
@@ -290,6 +293,8 @@ const char *orte_job_state_to_str(orte_job_state_t state)
         return "ERROR REPORTED ELSEWHERE";
     case ORTE_JOB_STATE_REPORT_PROGRESS:
         return "REPORT PROGRESS";
+    case ORTE_JOB_STATE_ALLOC_FAILED:
+        return "ALLOCATION FAILED";
     case ORTE_JOB_STATE_ANY:
         return "ANY";
     default:
@@ -297,6 +302,24 @@ const char *orte_job_state_to_str(orte_job_state_t state)
     }
 }
 
+const char *orte_app_ctx_state_to_str(orte_app_state_t state)
+{
+    switch(state) {
+    case ORTE_APP_STATE_UNDEF:
+        return "UNDEFINED";
+    case ORTE_APP_STATE_INIT:
+        return "PENDING INIT";
+    case ORTE_APP_STATE_ALL_MAPPED:
+        return "ALL MAPPED";
+    case ORTE_APP_STATE_RUNNING:
+        return "RUNNING";
+    case ORTE_APP_STATE_COMPLETED:
+        return "COMPLETED";
+    default:
+        return "UNKNOWN STATE!";
+    }
+}
+
 const char *orte_proc_state_to_str(orte_proc_state_t state)
 {
     switch(state) {
@@ -348,6 +371,10 @@ const char *orte_proc_state_to_str(orte_proc_state_t state)
         return "EXITED WITH NON-ZERO STATUS";
     case ORTE_PROC_STATE_FAILED_TO_LAUNCH:
         return "FAILED TO LAUNCH";
+    case ORTE_PROC_STATE_UNABLE_TO_SEND_MSG:
+        return "UNABLE TO SEND MSG";
+    case ORTE_PROC_STATE_LIFELINE_LOST:
+        return "LIFELINE LOST";
     case ORTE_PROC_STATE_ANY:
         return "ANY";
     default:
