@@ -10,6 +10,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2011 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2012-2013 Los Alamos National Security, LLC. 
+ *                         All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -58,59 +60,6 @@ bool orte_show_help_is_available(void)
     return ready;
 }
 
-/************************************************************************/
-
-/* Section for systems without RML and/or HNP support (e.g., Cray) --
-   just output directly; don't do any fancy RML sending to the HNP. */
-#if ORTE_DISABLE_FULL_SUPPORT
-
-int orte_show_help_init(void)
-{
-    ready = true;
-    return ORTE_SUCCESS;
-}
-
-void orte_show_help_finalize(void)
-{
-    ready = false;
-    return;
-}
-
-int orte_show_help(const char *filename, const char *topic, 
-                   bool want_error_header, ...)
-{
-    va_list arglist;
-    char *output;
-    
-    if (orte_execute_quiet) {
-        return ORTE_SUCCESS;
-    }
-    
-    va_start(arglist, want_error_header);
-    output = opal_show_help_vstring(filename, topic, want_error_header, 
-                                    arglist);
-    va_end(arglist);
-    
-    /* If nothing came back, there's nothing to do */
-    if (NULL == output) {
-        return ORTE_SUCCESS;
-    }
-    
-    opal_output(0, "%s", output);
-    return ORTE_SUCCESS;
-}
-
-int orte_show_help_suppress(const char *filename, const char *topic)
-{
-    return ORTE_ERR_NOT_SUPPORTED;
-}
-
-#else
-
-/************************************************************************/
-
-/* Section for systems that have full RML/HNP support */
-
 /* List items for holding (filename, topic) tuples */
 typedef struct {
     opal_list_item_t super;
@@ -497,7 +446,7 @@ static int show_help(const char *filename, const char *topic,
             fflush(orte_xml_fp);
             free(tmp);
         } else {
-            fprintf(stderr, "%s", output);
+            opal_output(orte_clean_output, "%s", output);
         }
         if (!show_help_timer_set) {
             show_help_time_last_displayed = now;
@@ -679,7 +628,7 @@ int orte_show_help_norender(const char *filename, const char *topic,
      * is process this locally
      */
     if (ORTE_PROC_IS_HNP ||
-        NULL == orte_rml.send_buffer ||
+        NULL == orte_rml.send_buffer_nb ||
         NULL == orte_routed.get_route ||
         NULL == orte_process_info.my_hnp_uri) {
         rc = show_help(filename, topic, output, ORTE_PROC_MY_NAME);
@@ -689,7 +638,7 @@ int orte_show_help_norender(const char *filename, const char *topic,
      * the HNP for processing
      */
     else {
-        opal_buffer_t buf;
+        opal_buffer_t *buf;
         static bool am_inside = false;
 
         /* JMS Note that we *may* have a recursion situation here where
@@ -702,22 +651,24 @@ int orte_show_help_norender(const char *filename, const char *topic,
             am_inside = true;
         
             /* build the message to the HNP */
-            OBJ_CONSTRUCT(&buf, opal_buffer_t);
+            buf = OBJ_NEW(opal_buffer_t);
             /* pack the filename of the show_help text file */
-            opal_dss.pack(&buf, &filename, 1, OPAL_STRING);
+            opal_dss.pack(buf, &filename, 1, OPAL_STRING);
             /* pack the topic tag */
-            opal_dss.pack(&buf, &topic, 1, OPAL_STRING);
+            opal_dss.pack(buf, &topic, 1, OPAL_STRING);
             /* pack the flag that we have a string */
-            opal_dss.pack(&buf, &have_output, 1, OPAL_INT8);
+            opal_dss.pack(buf, &have_output, 1, OPAL_INT8);
             /* pack the resulting string */
-            opal_dss.pack(&buf, &output, 1, OPAL_STRING);
+            opal_dss.pack(buf, &output, 1, OPAL_STRING);
             /* send it to the HNP */
-            if (0 > (rc = orte_rml.send_buffer(ORTE_PROC_MY_HNP, &buf, ORTE_RML_TAG_SHOW_HELP, 0))) {
+            if (ORTE_SUCCESS != (rc = orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, buf,
+                                                              ORTE_RML_TAG_SHOW_HELP,
+                                                              orte_rml_send_callback, NULL))) {
                 ORTE_ERROR_LOG(rc);
+                OBJ_RELEASE(buf);
             } else {
                 rc = ORTE_SUCCESS;
             }
-            OBJ_DESTRUCT(&buf);
             am_inside = false;
         }
     }
@@ -745,7 +696,7 @@ int orte_show_help_suppress(const char *filename, const char *topic)
        has not been setup, or we weren't given an HNP, then all we can
        do is process this locally. */
     if (ORTE_PROC_IS_HNP ||
-        NULL == orte_rml.send_buffer ||
+        NULL == orte_rml.send_buffer_nb ||
         NULL == orte_routed.get_route ||
         NULL == orte_process_info.my_hnp_uri) {
         rc = show_help(filename, topic, NULL, ORTE_PROC_MY_NAME);
@@ -755,7 +706,7 @@ int orte_show_help_suppress(const char *filename, const char *topic)
      * the HNP for processing
      */
     else {
-        opal_buffer_t buf;
+        opal_buffer_t *buf;
         static bool am_inside = false;
 
         /* JMS Note that we *may* have a recursion situation here where
@@ -768,24 +719,23 @@ int orte_show_help_suppress(const char *filename, const char *topic)
             am_inside = true;
         
             /* build the message to the HNP */
-            OBJ_CONSTRUCT(&buf, opal_buffer_t);
+            buf = OBJ_NEW(opal_buffer_t);
             /* pack the filename of the show_help text file */
-            opal_dss.pack(&buf, &filename, 1, OPAL_STRING);
+            opal_dss.pack(buf, &filename, 1, OPAL_STRING);
             /* pack the topic tag */
-            opal_dss.pack(&buf, &topic, 1, OPAL_STRING);
+            opal_dss.pack(buf, &topic, 1, OPAL_STRING);
             /* pack the flag that we DO NOT have a string */
-            opal_dss.pack(&buf, &have_output, 1, OPAL_INT8);
+            opal_dss.pack(buf, &have_output, 1, OPAL_INT8);
             /* send it to the HNP */
-            if (0 > (rc = orte_rml.send_buffer(ORTE_PROC_MY_HNP, &buf, ORTE_RML_TAG_SHOW_HELP, 0))) {
+            if (ORTE_SUCCESS != (rc = orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, buf,
+                                                              ORTE_RML_TAG_SHOW_HELP,
+                                                              orte_rml_send_callback, NULL))) {
                 ORTE_ERROR_LOG(rc);
+                OBJ_RELEASE(buf);
             }
-            OBJ_DESTRUCT(&buf);
             am_inside = false;
         }
     }
     
     return ORTE_SUCCESS;
 }
-
-#endif /* ORTE_DISABLE_FULL_SUPPORT */
-
